! *************************************************************************************************
! sue_sed_opal.f90
! Opal sediment diagenesis routines
! *************************************************************************************************



MODULE sue_sed_opal

  
  USE sue_config
  USE sue_lib
  IMPLICIT NONE
  SAVE


! *** global configuration parameters ***
! set the maximum real number precision to be used
  INTEGER,PARAMETER::opal_nzmax = 52 ! total number of sediment sub-layers
  INTEGER,PARAMETER::opal_zimax = 51 ! number of sediment sub-layers used to calculate the dissolution flux
  REAL(KIND=fp_1),DIMENSION(opal_nzmax)::opal_delz

! *** global constants ***
  REAL(KIND=fp_1),PARAMETER::opal_DSi0 = 4.59E-06   ! [Si] diffusivity (cm2 s-1) in free water at 0'C
  REAL(KIND=fp_1),PARAMETER::opal_BSi0 = 4.75E-09   ! bioturbational diffusivity (cm2 s-1) (== 150 cm2 kyr-1)
  REAL(KIND=fp_1),PARAMETER::opal_df_thesh = 1.0E-5 ! threshold for virtual steady-state (== 0.001%)
  REAL(KIND=fp_1),PARAMETER::opal_t_step = 2.0E+06  ! time step (s)
  INTEGER,PARAMETER::opal_t_loop_max = 1000000 ! maximum allowed time steps


CONTAINS


! ******************************************
! *** calculate sediment sub-layer depth ***
! ******************************************
  FUNCTION calc_z()
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::calc_z
! local variables
    INTEGER::zi
! calculate z and return function value
    calc_z(1) = 0.0
    DO zi = 2,opal_zimax
      calc_z(zi) = 0.5 * opal_delz(zi) + SUM(opal_delz(1:zi - 1))
    END DO
  END FUNCTION calc_z
! ******************************************


! *******************************************
! *** calculate sediment porosity profile ***
! *******************************************
  FUNCTION calc_pore(z)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::calc_pore
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::z
! local variables
    INTEGER::zi
    REAL(KIND=fp_1)::pore_min,exp_pore
! calculate porosity profile and return function value
    calc_pore(1) = 1.0
    DO zi = 2,opal_zimax
      calc_pore(zi) = 0.690 + 0.260 * (z(zi) + 1.0)**-1.2
    ENDDO
    calc_pore(opal_zimax + 1:opal_nzmax) = calc_pore(opal_zimax)
  END FUNCTION calc_pore
! *******************************************


! ******************************************
! *** calculate opal solubility constant ***
! ******************************************
  FUNCTION calc_conc_Si_eq(botT)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::sed_pore
    REAL(KIND=fp_1)::calc_conc_Si_eq
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::botT
! calculate conc_Si_eq and return function value
! NOTE: change units from (umol kg-1) to (mol kg-1)
    calc_conc_Si_eq = 10.0**(6.44 - 968.0 / botT)
    calc_conc_Si_eq = 1.0E-06 * calc_conc_Si_eq
  END FUNCTION calc_conc_Si_eq
! ******************************************


! ************************************************************
! *** convert equilibriun [Si] to sediment asymptotic [Si] ***
! ************************************************************
  FUNCTION conv_si_eqlibtoasympt(opaltorefrac)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::conv_si_eqlibtoasympt
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::opaltorefrac
! convert equilibriun [Si] to sediment asymptotic [Si] according sediment detrital content
    conv_si_eqlibtoasympt = 1.0 - (0.045 * opaltorefrac)**0.60
  END FUNCTION conv_si_eqlibtoasympt
! ************************************************************


! ******************************************************
! *** calculate Si dissolution rate constant profile ***
! ******************************************************
  FUNCTION calc_KSi(KSi0,concSieq,sedconcSi)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::calc_KSi
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::KSi0
    REAL(KIND=fp_1),INTENT(in)::concSieq
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::sedconcSi
! local variables
    INTEGER::zi
    REAL(KIND=fp_1)::r_Si
! calculate KSi and return function value
    calc_KSi(1) = 0.0
    DO zi = 2,opal_zimax
      IF (sedconcSi(zi) > concSieq) THEN
	calc_KSi(zi) = 0.0
      ELSE
	r_Si = (concSieq - sedconcSi(zi)) / concSieq
	calc_KSi(zi) = KSi0 * r_Si
      ENDIF
    END DO
    calc_KSi(opal_zimax + 1:opal_nzmax) = 0.0
  END FUNCTION calc_KSi
! ******************************************************


! ********************************************************************
! *** calculate Si dissolution rate constant profile (alternative) ***
! ********************************************************************
  FUNCTION calc_KSi_alt(z,KSi0,botT,concSieq,sedconcSi)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::calc_KSi_alt
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::z
    REAL(KIND=fp_1),INTENT(in)::KSi0
    REAL(KIND=fp_1),INTENT(in)::botT
    REAL(KIND=fp_1),INTENT(in)::concSieq
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::sedconcSi
! local variables
    INTEGER::zi
    REAL(KIND=fp_1)::T_C
    REAL(KIND=fp_1)::r_Si
! calculate local constants
    T_C = botT - const_zeroc
! calculate KSi and return function value
! NOTE: exponential decay of kSi with down-core depth after Van Cappellen and Qiu [1997b]
! NOTE: non-linear dependence of kSi with (concSieq - sedconcSi(k)) / concSieq,
!       (plus additional ad hoc temperature dependence), after Van Cappellen and Qiu [1997b]
    calc_KSi_alt(1) = 0.0
    DO zi = 2,opal_zimax
      IF (sedconcSi(zi) > concSieq) THEN
	calc_KSi_alt(zi) = 0.0
      ELSE
	r_Si = (concSieq - sedconcSi(zi)) / concSieq
	calc_KSi_alt(zi) = KSi0 * (0.25 + 0.75 * EXP(-z(zi) / 6.0)) * &
	  & (0.160 * (1.0 + T_C / 15.0) * r_Si + 0.5 * ((1.0 + T_C / 400.0)**4.0 * r_Si)**7.0) / 0.660
      ENDIF
    END DO
    calc_KSi_alt(opal_zimax + 1:opal_nzmax) = 0.0
  END FUNCTION calc_KSi_alt
! ********************************************************************


! ******************************************
! *** calculate [Si] diffusivity profile ***
! ******************************************
  FUNCTION calc_DSi(botT,sedpore)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::calc_DSi
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::botT
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::sedpore
! local variables
    INTEGER::zi
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::form ! formation factor [Ullman and Aller, 1982]
! calculate [Si] diffusivity profile and return function value
! NOTE: [Si] molecular diffusion coefficient (at 0'C) adjusted for temperature
!       (adapted from Hensen et al. [1998])
! NOTE: set the k=1 sub-layer diffusivity equal to that at k=2
! NOTE: calc_DSi(k) = DSi0 / (sedpore(k) * form(k)) : Ullman and Aller [1982]
!       (equivalent to DSi0 * sedpore(k)**1.5)
! NOTE: calc_DSi(k) = DSi0 * sedpore(k)**1.333 : Millington [1959]
    DO zi = 2,opal_zimax
      form(zi) = 1.0 / (sedpore(zi)**2.5)
      calc_DSi(zi) = (opal_DSi0 + 1.74E-07 * (botT - const_zeroc)) / (sedpore(zi) * form(zi))
    ENDDO
    calc_DSi(1) = opal_DSi0 + 1.74E-07 * (botT - const_zeroc)
    calc_DSi(opal_zimax + 1:opal_nzmax) = calc_DSi(opal_zimax)
  END FUNCTION calc_DSi
! ******************************************


! *****************************************
! *** initialize opal diagenesis scheme ***
! *****************************************
  SUBROUTINE init_sedflx_Si(botconcSi,sedconcSi)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::botconcSi
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(opal_nzmax)::sedconcSi
! initialzie [Si] to ambient
    sedconcSi(:) = botconcSi
  END SUBROUTINE init_sedflx_Si
! *****************************************


! ******************************************************************
! *** calculate sediment Si distribution change due to diffusion ***
! ******************************************************************
  FUNCTION calc_dsed_Si_diff(DSi,sedconcSi)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::calc_dsed_Si_diff
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::DSi
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::sedconcSi
! local variables
    INTEGER::zi
! calculate change in [Si] in each sub-layer pore-space and return function value
! NOTE: use average Si diffusivity
! NOTE: as [Si] is in stored in units of (mol kg-1),
!       the units must be converted into (mol cm-3) in order to calculate the Si flux
    calc_dsed_Si_diff(1) = 0.0
    DO zi = 2,opal_zimax
      calc_dsed_Si_diff(zi) = &
	& 0.5 * (DSi(zi + 1) + DSi(zi)) * &
	& (sedconcSi(zi + 1) - sedconcSi(zi)) / (0.5 * (opal_delz(zi + 1) + opal_delz(zi))) + &
	& 0.5 * (DSi(zi - 1) + DSi(zi)) * &
	& (sedconcSi(zi - 1) - sedconcSi(zi)) / (0.5 * (opal_delz(zi - 1) + opal_delz(zi)))
    ENDDO
    calc_dsed_Si_diff(opal_zimax + 1:opal_nzmax) = 0.0
    calc_dsed_Si_diff(:) = conv_cm3_kg * calc_dsed_Si_diff(:)
  END FUNCTION calc_dsed_Si_diff
! ******************************************************************


! ********************************************************************
! *** calculate sediment Si distribution change due to dissolution ***
! ********************************************************************
  FUNCTION calc_dsed_Si_diss(sedconcopal,delz,sedpore,KSi)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::calc_dsed_Si_diss
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::sedconcopal
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::delz
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::sedpore
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::KSi
! local variables
    INTEGER::zi
! return function value
    calc_dsed_Si_diss(1) = 0.0
    DO zi = 2,opal_zimax
      calc_dsed_Si_diss(zi) = KSi(zi) * sedconcopal * delz(zi) * (1.0 - sedpore(zi)) * conv_opal_cm3_mol
    ENDDO
    calc_dsed_Si_diss(opal_zimax + 1:opal_nzmax) = 0.0
  END FUNCTION calc_dsed_Si_diss
! ********************************************************************


! *********************************************************************
! *** calculate sediment Si distribution change due to bioturbation ***
! *********************************************************************
  FUNCTION calc_dsed_Si_biot(sedconcSi)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::calc_dsed_Si_biot
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::sedconcSi
! local variables
    INTEGER::zi
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::temp_sedconcSi
! make a temporary sediment [Si] profile
! NOTE: set the k = 1 layer [Si] to that of k = 2
    temp_sedconcSi(:) = sedconcSi(:)
    temp_sedconcSi(1) = sedconcSi(2)
! calculate change in [Si] in each sub-layer pore-space and return function value
! NOTE: assume a uniform bioturbation rate with depth in the sediments
! NOTE: as [Si] is in stored in units of (mol kg-1),
!       the units must be converted into (mol cm-3) in order to calculate the Si flux
    calc_dsed_Si_biot(1) = 0.0
    DO zi = 2,opal_zimax
      calc_dsed_Si_biot(zi) = opal_BSi0 * ( &
	& (sedconcSi(zi + 1) - sedconcSi(zi)) / (0.5 * (opal_delz(zi + 1) + opal_delz(zi))) + &
	& (sedconcSi(zi - 1) - sedconcSi(zi)) / (0.5 * (opal_delz(zi - 1) + opal_delz(zi))) )
    ENDDO
    calc_dsed_Si_biot(opal_zimax + 1:opal_nzmax) = 0.0
    calc_dsed_Si_biot(:) = conv_cm3_kg * calc_dsed_Si_biot(:)
  END FUNCTION calc_dsed_Si_biot
! *********************************************************************


! *********************************
! *** [Si] dierivative function ***
! *********************************
  FUNCTION calc_dSidt(sedconcopal,sedconcSi,sedpore,DSi,KSi)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::calc_dSidt
! dummy variables
    REAL(KIND=fp_1),INTENT(in)::sedconcopal
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::sedconcSi
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::sedpore
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::DSi
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::KSi
! local variables
    INTEGER::zi
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::dsed_Si_diff
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::dsed_Si_diss
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::dsed_Si_biot
!
    dsed_Si_diff = calc_dsed_Si_diff(DSi,sedconcSi)
    dsed_Si_diss = calc_dsed_Si_diss(sedconcopal,opal_delz,sedpore,KSi)
    dsed_Si_biot = calc_dsed_Si_biot(sedconcSi)
!
    calc_dSidt(1) = 0.0
    DO zi = 2,opal_zimax
      calc_dSidt(zi) = (dsed_Si_diff(zi) + dsed_Si_diss(zi) + dsed_Si_biot(zi)) / &
	& (sedpore(zi) * opal_delz(zi))
    END DO
    calc_dSidt(opal_zimax + 1:opal_nzmax) = calc_dSidt(opal_zimax)
  END FUNCTION calc_dSidt
! *********************************


! *******************************************************************
! *** 4th order Runge-Kutta differential equation solving routine ***
! *******************************************************************
  SUBROUTINE runge_kutta_4_opal(dt,sedconcopal,sedconcSi,sedpore,DSi,KSi)
    IMPLICIT NONE
! dummy variables
    REAL(KIND=fp_1),INTENT(in)::dt
    REAL(KIND=fp_1),INTENT(in)::sedconcopal
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(opal_nzmax)::sedconcSi
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::sedpore
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::DSi
    REAL(KIND=fp_1),INTENT(in),DIMENSION(opal_nzmax)::KSi
! local variables
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::conc_Si0
    REAL(KIND=fp_1),DIMENSION(opal_nzmax)::k1,k2,k3,k4
! store initial value of Si
    conc_Si0 = sedconcSi
! calculate coefficient 'k1'
    k1 = dt * calc_dSidt(sedconcopal,conc_Si0,sedpore,DSi,KSi)
! calculate coefficient 'k2'
    k2 = dt * calc_dSidt(sedconcopal,conc_Si0 + 0.5 * k1,sedpore,DSi,KSi)
! calculate coefficient 'k3'
    k3 = dt * calc_dSidt(sedconcopal,conc_Si0 + 0.5 * k2,sedpore,DSi,KSi)
! calculate coefficient 'k4'
    k4 = dt * calc_dSidt(sedconcopal,conc_Si0 + k3,sedpore,DSi,KSi)
! estimate the value at forward time t + dt
    sedconcSi = conc_Si0 + (k1 + 2.0 * k2 + 2.0 * k3 + k4) / 6.0
  END SUBROUTINE runge_kutta_4_opal
! *******************************************************************


END MODULE sue_sed_opal
