! *************************************************************************************************
! sue_box.f90
! Ocean-atmosphere box model
! *************************************************************************************************



MODULE sue_box


  USE sue_config
  USE sue_lib
  USE sue_co2calc
  IMPLICIT NONE
  SAVE


CONTAINS


! *************************************************************************************************
! *** BOX CALCULATION ROUTINES ********************************************************************
! *************************************************************************************************


! *****************************************************
! *** CALCULATE BOX SURFACE AREAS FROM LAT-LON DATA ***
! *****************************************************
  SUBROUTINE calc_Alatlon(PIGphys)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_lay,n_phys)::PIGphys
! local variables
    INTEGER::l
    REAL(KIND=fp_1),DIMENSION(n_col)::PIG_surA ! PIG ocean surface area
    REAL(KIND=fp_1),DIMENSION(n_col)::lat_N    ! northern latitude angle (radians)
    REAL(KIND=fp_1),DIMENSION(n_col)::lat_S    ! southern latitude angle (radians)

! calculate northern and southern latitudes in radians
    lat_N(:) = (2.0 * const_PI / 360.0) * (PIGphys(:,1,ip_lat) + 0.5 * PIGphys(:,1,ip_dlat))
    lat_S(:) = (2.0 * const_PI / 360.0) * (PIGphys(:,1,ip_lat) - 0.5 * PIGphys(:,1,ip_dlat))
! calculate ocean surface areas from latitude and longitude data
    PIG_surA(:) = 2.0 * const_PI * (const_rEarth)**2 * &
      & (PIGphys(:,1,ip_dlon) / 360.0) * &
      & (SIN(lat_N(:)) - SIN(lat_S(:)))
! set maximum potential (PIG) area equally for all ocean layers
    DO l = 1,n_lay
      PIGphys(:,l,ip_A_top) = PIG_surA(:)
      PIGphys(:,l,ip_A_bot) = PIG_surA(:)
    END DO

  END SUBROUTINE calc_Alatlon
! *****************************************************


! ****************************************************************************
! *** calculate required bathymetry data for ocean boxes from raw data-set ***
! ****************************************************************************
  SUBROUTINE calc_bathdata_box(bathdata,phys,sealevel,PIGphys)
! NOTE: this assumes that only the bottom layer box of each water column changes in volume -
! this might potentially be a problem in some configurations as the bottom box volumn could go negative!!!
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_bath_depth,n_bath_par)::bathdata
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(in)::sealevel
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::PIGphys
! local variables
    INTEGER::c,l,o
    REAL(KIND=fp_1),DIMENSION(n_col)::area_frac       ! area fraction
    REAL(KIND=fp_1),DIMENSION(n_col)::lat_N           ! northern latitude angle (radians)
    REAL(KIND=fp_1),DIMENSION(n_col)::lat_S           ! southern latitude angle (radians)
    REAL(KIND=fp_1),DIMENSION(n_col)::len_N           ! northern latitude surface distance (m)
    REAL(KIND=fp_1),DIMENSION(n_col)::len_S           ! southern latitude surface distance (m)
    REAL(KIND=fp_1),DIMENSION(n_col,n_lay)::int_vol   ! integrated volume
    REAL(KIND=fp_1),DIMENSION(n_col,n_lay)::int_csA_N ! integrated cross-sectionional area (N boundary)
    REAL(KIND=fp_1),DIMENSION(n_col,n_lay)::int_csA_S ! integrated cross-sectionional area (S boundary)
    REAL(KIND=fp_1)::area_frac_max                    ! maximum (PIG) total area fraction

! *** initialize local variables ***
    area_frac     = 0.0
    lat_N         = 0.0
    lat_S         = 0.0
    int_vol       = 0.0
    int_csA_N     = 0.0
    int_csA_S     = 0.0
    area_frac_max = 0.0

! *** calculate local variables ***
! calculate northern and southern latitudes in radians
    lat_N(:) = (2.0 * const_PI / 360.0) * (PIGphys(:,1,ip_lat) + 0.5 * PIGphys(:,1,ip_dlat))
    lat_S(:) = (2.0 * const_PI / 360.0) * (PIGphys(:,1,ip_lat) - 0.5 * PIGphys(:,1,ip_dlat))
! calculate northern and southern latitude surface longitudinal distances (m)
    len_N(:) = 2.0 * const_PI * const_rEarth * &
      & (PIGphys(:,1,ip_dlon) / 360.0) * &
      & COS(lat_N(:))
    len_S(:) = 2.0 * const_PI * const_rEarth * &
      & (PIGphys(:,1,ip_dlon) / 360.0) * &
      & COS(lat_S(:))

! *** calculate ocean box depths from sealevel change ***
! NOTE: keep the top and bottom depth below the ocean surface of each ocean box invarient,
! allow only the bottom depth of the lowest layer box to vary
    phys(:,:,ip_D_top)     = PIGphys(:,:,ip_D_top)
    phys(:,:,ip_D_bot)     = PIGphys(:,:,ip_D_bot)
    phys(:,n_lay,ip_D_bot) = PIGphys(:,n_lay,ip_D_bot) + sealevel

! *** read in raw area fraction data-set and integrate between model depth intervals ***
! NOTE: the bathymetry data-set is as an area fraction (SUM = 1.0)
! NOTE: integrate from bottom up, as the bottom depth of the bottom layer is invarient - 
!       it should be noted that this loop direction is the opposite of that used to calculate sediment areas
! NOTE: the volume of each box is calculated by integrating the area slices up the bathymetry,
!       assuming a slice thickness of 1.0 m
! NOTE: set an invarient default area of bottom layer (zero)
    DO c = 1,n_col
! zero local variables
      area_frac(c)   = 0.0
      int_vol(c,:)   = 0.0
      int_csA_N(c,:) = 0.0
      int_csA_S(c,:) = 0.0
      l = n_lay
      o = n_bath_depth
      phys(c,l,ip_A_bot) = 0.0
      DO
	IF (bathdata(c,o,ibath_D) > (phys(c,l,ip_D_top) - sealevel)) THEN
	  area_frac(c)   = area_frac(c)   + bathdata(c,o,ibath_Afrac)
	  int_vol(c,l)   = int_vol(c,l)   + PIGphys(c,1,ip_A_top) * area_frac(c) * 1.0
	  int_csA_N(c,l) = int_csA_N(c,l) + len_N(c) * area_frac(c) * 1.0
	  int_csA_S(c,l) = int_csA_S(c,l) + len_S(c) * area_frac(c) * 1.0
	ELSE ! store area, volume and mass values
	  phys(c,l,ip_A_top) = PIGphys(c,1,ip_A_top) * area_frac(c)
	  phys(c,l,ip_V)     = int_vol(c,l)
	  phys(c,l,ip_csA_N) = int_csA_N(c,l)
	  phys(c,l,ip_csA_S) = int_csA_S(c,l)
	  phys(c,l,ip_M)     = phys(c,l,ip_V) * conv_m3_kg
	  IF (l > 1) THEN ! set the bottom area of the new layer equal to the top area of the previous layer
	    l = l - 1
	    phys(c,l,ip_A_bot) = phys(c,l + 1,ip_A_top)
	    area_frac(c)   = area_frac(c) + bathdata(c,o,ibath_Afrac)
	    int_vol(c,l)   = 0.0          + PIGphys(c,1,ip_A_top) * area_frac(c) * 1.0
	    int_csA_N(c,l) = 0.0          + len_N(c) * area_frac(c) * 1.0
	    int_csA_S(c,l) = 0.0          + len_S(c) * area_frac(c) * 1.0
	  ELSE
	    EXIT ! exit the current DO loop: the top ocean layer has been reached
	  ENDIF
	ENDIF
	o = o - 1
	IF (o < 1) THEN
	  phys(c,l,ip_A_top) = PIGphys(c,1,ip_A_top) * area_frac(c)
	  phys(c,l,ip_V)     = int_vol(c,l)
	  phys(c,l,ip_csA_N) = int_csA_N(c,l)
	  phys(c,l,ip_csA_S) = int_csA_S(c,l)
	  phys(c,l,ip_M)     = phys(c,l,ip_V) * conv_m3_kg
	  EXIT ! exit the current DO loop: the top of the bathymetry data set has been reached
	END IF
      END DO
    END DO

! *** numerical and physical-plausibility checks ***
! check that:
! (1) area fraction SUM is equal to unity!
! NOTE: only carry this out when sealevel = 0.0
! (2) if any box has zero volume assign a nominal non-zero mass
!     this is to avoid problems with gaian coupling (which divides by box mass)
! (1):
    DO c = 1,n_col
      IF ((area_frac(c) < 0.999999 .OR. area_frac(c) > 1.000001) .and. sealevel == 0.0) THEN
	PRINT*,"FATAL ERROR [sue_box.f90, calc_bathdata_box]:"
	PRINT*,"ocean-floor area fraction = ",area_frac
	PRINT*,"in ocean column: ",c
	STOP
      ENDIF
    ENDDO
! (2):
    DO l = 1,n_lay
      DO c = 1,n_col
	IF (phys(c,l,ip_V) == 0.0) THEN
	  phys(c,l,ip_V) = 0.0
	  phys(c,l,ip_M) = 1.0
	ENDIF
      END DO
    END DO

  END SUBROUTINE calc_bathdata_box
! ****************************************************************************


! ************************************************
! *** CHECK THE MASS BALANCE IN EACH OCEAN BOX ***
! ************************************************
  SUBROUTINE chk_boxmassbal(oceancir)
    IMPLICIT NONE
! dummy arguments
    TYPE(ocean_circulation),INTENT(in),DIMENSION(1:par_n_wflux)::oceancir
! local variables
    INTEGER::c,l
    INTEGER::v
    REAL(KIND=fp_1)::wflux_net
    
! *** check for possible mass imbalances in each ocean box for ocean circulation scheme #1 ***
    DO c = 1,n_col
      DO l = 1,n_lay
	wflux_net = 0.0
	DO v = 1,par_n_wflux
	  IF ((oceancir(v)%ioc_ce == c) .AND. (oceancir(v)%ioc_le == l)) THEN
	    wflux_net = wflux_net + oceancir(v)%ioc_wf_1 / (conv_yr_s * 1.0E+06 * conv_m3_kg)
	  END IF
	  IF ((oceancir(v)%ioc_cs == c) .AND. (oceancir(v)%ioc_ls == l)) THEN
	    wflux_net = wflux_net - oceancir(v)%ioc_wf_1 / (conv_yr_s * 1.0E+06 * conv_m3_kg)
	  END IF
	END DO
	IF (ABS(wflux_net) > 0.000001) THEN
	  PRINT*,"FATAL ERROR [sue_box.f90, chk_boxmassbal]:"
	  PRINT*,"there is a mass flux imbalance of (Sv, +ve into box)"
	  PRINT*,wflux_net
	  PRINT*,"at location (column,layer)"
	  PRINT*,c,l
	  PRINT*,"in ocean circulation scheme #1"
	  PRINT*,""
	  STOP
	ENDIF
      END DO
    END DO
    
! *** check for possible mass imbalances in each ocean box for ocean circulation scheme #2 ***
    DO c = 1,n_col
      DO l = 1,n_lay
	wflux_net = 0.0
	DO v = 1,par_n_wflux
	  IF ((oceancir(v)%ioc_ce == c) .AND. (oceancir(v)%ioc_le == l)) THEN
	    wflux_net = wflux_net + oceancir(v)%ioc_wf_2 / (conv_yr_s * 1.0E+06 * conv_m3_kg)
	  END IF
	  IF ((oceancir(v)%ioc_cs == c) .AND. (oceancir(v)%ioc_ls == l)) THEN
	    wflux_net = wflux_net - oceancir(v)%ioc_wf_2 / (conv_yr_s * 1.0E+06 * conv_m3_kg)
	  END IF
	END DO
	IF (ABS(wflux_net) > 0.000001) THEN
	  PRINT*,"FATAL ERROR [sue_box.f90, chk_boxmassbal]:"
	  PRINT*,"there is a mass flux imbalance of (Sv, +ve into box)"
	  PRINT*,wflux_net
	  PRINT*,"at location (column,layer)"
	  PRINT*,c,l
	  PRINT*,"in ocean circulation scheme #2"
	  PRINT*,""
	  STOP
	ENDIF
      END DO
    END DO

  END SUBROUTINE chk_boxmassbal
! ************************************************


! ************************************************************
! *** CHECK THE FRACTIONAL MASS EXCHANGE IN EACH OCEAN BOX ***
! ************************************************************
  SUBROUTINE chk_boxfracmassex(dt,oceancir,phys)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::dt
    TYPE(ocean_circulation),INTENT(in),DIMENSION(1:par_n_wflux)::oceancir
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
    INTEGER::c,l
    INTEGER::v
    REAL(KIND=fp_1)::wflux_in
    REAL(KIND=fp_1)::wflux_out
    CHARACTER(LEN=1)::answer

! initialize local variables
    answer = "y"

! check that the maxx flux into or out of each ocean box
! is not too high a fraction of the total box mass
    DO c = 1,n_col
      DO l = 1,n_lay
	wflux_in  = 0.0
	wflux_out = 0.0
	DO v = 1,par_n_wflux
	  IF ((oceancir(v)%ioc_ce == c) .AND. (oceancir(v)%ioc_le == l)) THEN
	    wflux_in  = wflux_in  + dt * oceancir(v)%ioc_wf
	  END IF
	  IF ((oceancir(v)%ioc_cs == c) .AND. (oceancir(v)%ioc_ls == l)) THEN
	    wflux_out = wflux_out + dt * oceancir(v)%ioc_wf
	  END IF
	END DO
	IF (wflux_in > par_boxfracmassex_threshold * phys(c,l,ip_M)) THEN
	  PRINT*,""
	  PRINT*,"WARNING [sue_box.f90, chk_boxfracmassex]:"
	  PRINT*,"mass flux per time step into ocean box (column,layer)"
	  PRINT*,c,l
	  PRINT*,"is ",100.0 * wflux_in / phys(c,l,ip_M),"% of the total box mass"
	  PRINT*,""
	  PRINT*,"CONTINUE (y/n)?"
	  READ*,answer
	  PRINT*,""
	ENDIF
	IF ((answer == "n") .OR. (answer == "N")) THEN
	  PRINT*,""
	  STOP
	ENDIF
	IF (wflux_out > par_boxfracmassex_threshold * phys(c,l,ip_M)) THEN
	  PRINT*,""
	  PRINT*,"WARNING [sue_box.f90, chk_boxfracmassex]:"
	  PRINT*,"mass flux per time step out of ocean box (column,layer)"
	  PRINT*,c,l
	  PRINT*,"is ",100.0 * wflux_in / phys(c,l,ip_M),"% of the total box mass"
	  PRINT*,""
	  PRINT*,"CONTINUE (y/n)?"
	  READ*,answer
	  PRINT*,""
	ENDIF
	IF ((answer == "n") .OR. (answer == "N")) THEN
	  PRINT*,""
	  STOP
	ENDIF
      END DO
    END DO

  END SUBROUTINE chk_boxfracmassex
! ************************************************************


! **********************************************************************
! *** scale tracer variable (to account for changes in ocean volume) ***
! **********************************************************************
  SUBROUTINE scale_tracer(vol,volold,tracer)
! NOTE: the volume scale factor is > 1.0 when the ocean box volumn has INCREASED,
! and thus tracer concentration have to be divided by this to reduce their value
! NOTE: it is assumed that all boxes change in tracer concentration in the same proportion,
! i.e., a whole-ocean volume scale factor is used,
! and there complete ocean mixing is assumed on time-scales less than the removal and addition of water
! NOTE: do not re-scale the temperature or salinity tracer fields (or at least scale them back again!).
    IMPLICIT NONE
! dummy variables
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay)::vol
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay)::volold
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_lay,n_tracer)::tracer
! local variables
    INTEGER::it,c,l
    REAL(KIND=fp_1),DIMENSION(n_col,n_lay)::vol_scale_factor
! set volume scale factor
    DO c = 1,n_col
      DO l = 1,n_lay
	IF ((volold(c,l) == 0.0) .OR. (vol(c,l) == 0.0)) THEN
	  vol_scale_factor(c,l) = 1.0
	ELSE
	  vol_scale_factor(c,l) = vol(c,l) / volold(c,l)
	ENDIF
      END DO
    END DO
! scale tracer concentration
    DO it = 1,n_tracer
      tracer(:,:,it) = tracer(:,:,it) / vol_scale_factor(:,:)
    ENDDO
! reverse scaling for temperature and salinity
    tracer(:,:,it_T) = tracer(:,:,it_T) * vol_scale_factor(:,:)
    tracer(:,:,it_S) = tracer(:,:,it_S) * vol_scale_factor(:,:)
  END SUBROUTINE scale_tracer
! **********************************************************************


! **************************************************************
! *** SET OCEAN d18O ISOTOPE RATIO (in ocean and atmosphere) ***
! **************************************************************
  SUBROUTINE set_d18O(d18O,oatracer)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::d18O
    TYPE(ocean_atm_tracer),INTENT(inout)::oatracer
! set d18O
    oatracer%ioat_t(:,:,it_18O_O2) = calc_frac_18O(d18O) * oatracer%ioat_t(:,:,it_O2)
    oatracer%ioat_a(ia_18O_pO2)    = calc_frac_18O(d18O) * oatracer%ioat_a(ia_pO2)
  END SUBROUTINE set_d18O
! **************************************************************


! ***************************************************************
! *** CALCULATE SURFACE OCEAN CARBONATE EQUILIBRIUM CONSTANTS ***
! ***************************************************************
  SUBROUTINE calc_sur_disconst(disconstsur,physsur,S)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_dis)::disconstsur
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_phys)::physsur
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::S
! local variables
    INTEGER::c
    REAL(KIND=fp_1)::D_av ! average depth of the surface layer
! return function value
    DO c = 1,n_col
      D_av = (physsur(c,ip_D_top) + physsur(c,ip_D_bot)) / 2.0
      CALL co2const_ex(physsur(c,ip_T),S(c),D_av,disconstsur(c,:))
    END DO
  END SUBROUTINE calc_sur_disconst
! ***************************************************************


! ********************************************************************
! *** CALCULATE SURFACE OCEAN CARBONATE EQUILIBRIUM - [H+] IN ONLY ***
! ********************************************************************
  SUBROUTINE calc_sur_carbchem_Hin(carbchemsur,physsur,tracersur,carbconstsur)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sscc)::carbchemsur
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_phys)::physsur
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_tracer)::tracersur
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_dis)::carbconstsur
! local variables
    INTEGER::c
    REAL(KIND=fp_1),DIMENSION(n_col)::fug_CO2,conc_CO2,conc_CO3,conc_HCO3,conc_H
! return function value
    DO c = 1,n_col
      conc_H(c) = physsur(c,ip_H)
      CALL calccarb_sur(c,carbconstsur(c,:),tracersur(c,it_S),tracersur(c,it_alk), &
	& tracersur(c,it_CO2),tracersur(c,it_PO4),tracersur(c,it_Si), &
	& fug_CO2(c),conc_CO2(c),conc_CO3(c),conc_HCO3(c),conc_H(c))
      carbchemsur(c,isscc_fCO2)  = fug_CO2(c)
      carbchemsur(c,isscc_cCO2)  = conc_CO2(c)
      carbchemsur(c,isscc_cCO3)  = conc_CO3(c)
      carbchemsur(c,isscc_cHCO3) = conc_HCO3(c)
    END DO
  END SUBROUTINE calc_sur_carbchem_Hin
! ********************************************************************


! *********************************************************************
! *** CALCULATE SURFACE OCEAN CARBONATE EQUILIBRIUM - [H+] IN / OUT ***
! *********************************************************************
  SUBROUTINE calc_sur_carbchem_Hinout(carbchemsur,physsur,tracersur,carbconstsur)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_sscc)::carbchemsur
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_phys)::physsur
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_tracer)::tracersur
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_dis)::carbconstsur
! local variables
    INTEGER::c
    REAL(KIND=fp_1),DIMENSION(n_col)::fug_CO2,conc_CO2,conc_CO3,conc_HCO3,conc_H
! return function value
    DO c = 1,n_col
      conc_H(c) = physsur(c,ip_H)
      CALL calccarb_sur(c,carbconstsur(c,:),tracersur(c,it_S),tracersur(c,it_alk), &
	& tracersur(c,it_CO2),tracersur(c,it_PO4),tracersur(c,it_Si), &
	& fug_CO2(c),conc_CO2(c),conc_CO3(c),conc_HCO3(c),conc_H(c))
      carbchemsur(c,isscc_fCO2)  = fug_CO2(c)
      carbchemsur(c,isscc_cCO2)  = conc_CO2(c)
      carbchemsur(c,isscc_cCO3)  = conc_CO3(c)
      carbchemsur(c,isscc_cHCO3) = conc_HCO3(c)
      physsur(c,ip_H) = conc_H(c)
    END DO
  END SUBROUTINE calc_sur_carbchem_Hinout
! *********************************************************************


! *********************************************
! *** calculate carbonate saturation depths ***
! *********************************************
  SUBROUTINE calc_D_sat(phys,tracer)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_tracer)::tracer
! local variables
    INTEGER::c,l
    REAL(KIND=fp_1)::carbsat_Ca                     !
    REAL(KIND=fp_1)::carbsat_CO3                    !
    REAL(KIND=fp_1)::carbsat_ohmcal                 !
    REAL(KIND=fp_1)::carbsat_ohmarg                 !
    REAL(KIND=fp_1)::carbsat_dCO3cal                !
    REAL(KIND=fp_1)::carbsat_dCO3arg                !
    REAL(KIND=fp_1)::carbsat_pH                     !
    REAL(KIND=fp_1),DIMENSION(n_dis)::carbsat_dis   !
    REAL(KIND=fp_1),DIMENSION(n_phys)::carbsat_phys !
    REAL(KIND=fp_1)::fug_CO2,conc_CO2,conc_HCO3     ! 

! *** loop through all ocean columns layers ***
    DO c = 1,n_col
      DO l = 1,n_lay

! *** calculate calcite saturation depth ***
! (1) use previous value for the calcite saturation depth as a best guess,
! and calculate carbonate dissoliation constants
	CALL co2const_ex(phys(c,l,ip_T), &
	  & tracer(c,l,it_S), &
	  & phys(c,l,ip_D_calsat), &
	  & carbsat_dis(:))
! (2) calculate carbonate system
	CALL calccarb_ex(carbsat_dis(:), &
	  & tracer(c,l,it_S), &
	  & tracer(c,l,it_alk), &
	  & tracer(c,l,it_CO2), &
	  & tracer(c,l,it_PO4), &
	  & tracer(c,l,it_Si), &
	  & carbsat_Ca, &
	  & fug_CO2, &
	  & conc_CO2, &
	  & carbsat_CO3, &
	  & conc_HCO3, &
	  & carbsat_ohmcal, &
	  & carbsat_ohmarg, &
	  & carbsat_dCO3cal, &
	  & carbsat_dCO3arg, &
	  & carbsat_pH)
! (3) estimate the aragonite saturation depth
! NOTE: different calcite saturation depths are allowed in different ocean boxes (including vertically)
	phys(c,l,ip_D_calsat) = calc_D_calsat(phys(c,l,ip_T), &
	  & tracer(c,l,it_S), &
	  & carbsat_Ca, &
	  & carbsat_CO3)
! (4) checkcalcite compensation depth value
	IF (phys(c,l,ip_D_calsat) < 0.0) THEN
	  IF (runopt(ropt_warn) == yes) THEN
	    PRINT*,"WARNING: calcite saturation depth is less than zero"
	    PRINT*,c,l,phys(c,l,ip_D_calsat)
	  ENDIF
	  program_error = yes
	  warning(iwarn_Dcalsat) = yes
	  phys(c,l,ip_D_calsat) = 0.0
	ENDIF

! *** calculate aragonite saturation depth ***
! (1) use previous value for the aragonite saturation depth as a best guess,
! and calculate carbonate dissoliation constants
	CALL co2const_ex(phys(c,l,ip_T), &
	  & tracer(c,l,it_S), &
	  & phys(c,l,ip_D_argsat), &
	  & carbsat_dis(:))
! (2) calculate carbonate system
	CALL calccarb_ex(carbsat_dis(:), &
	  & tracer(c,l,it_S), &
	  & tracer(c,l,it_alk), &
	  & tracer(c,l,it_CO2), &
	  & tracer(c,l,it_PO4), &
	  & tracer(c,l,it_Si), &
	  & carbsat_Ca, &
	  & fug_CO2, &
	  & conc_CO2, &
	  & carbsat_CO3, &
	  & conc_HCO3, &
	  & carbsat_ohmcal, &
	  & carbsat_ohmarg, &
	  & carbsat_dCO3cal, &
	  & carbsat_dCO3arg, &
	  & carbsat_pH)
! (3) estimate the aragonite saturation depth
! NOTE: different aragonaie saturation depths are allowed in different ocean boxes (including vertically)
	phys(c,l,ip_D_argsat) = calc_D_argsat(phys(c,l,ip_T), &
	  & tracer(c,l,it_S), &
	  & carbsat_Ca, &
	  & carbsat_CO3)
! (4) check aragonite compensation depth value
	IF (phys(c,l,ip_D_argsat) < 0.0) THEN
	  IF (runopt(ropt_warn) == yes) THEN
	    PRINT*,"WARNING: aragonite saturation depth is less than zero"
	    PRINT*,c,l,phys(c,l,ip_D_argsat)
	  ENDIF
	  program_error = yes
	  warning(iwarn_Dargsat) = yes
	  phys(c,l,ip_D_argsat) = 0.0
	ENDIF

      END DO
    ENDDO

  END SUBROUTINE calc_D_sat
! *********************************************


! *********************************
! *** CALCULATE REDFIELD RATIOS ***
! *********************************
  SUBROUTINE calc_rednp()
    IMPLICIT NONE

! *** calculate Redfield Ratios ***
! primary variables
    red_Porg_Corg = par_red_Porg_Corg
    red_Porg_Norg = par_red_Porg_Norg
    red_Porg_O2   = par_red_Porg_O2
    red_Corg_Porg = 1.0 / red_Porg_Corg 
    red_Corg_O2   = red_Porg_O2 / red_Porg_Corg
    red_Corg_Norg = red_Porg_Norg / red_Porg_Corg
! basic POM 
    red_Porg_Corg_dia(:) = red_Porg_Corg
    red_Porg_Corg_coc(:) = red_Porg_Corg
    red_Porg_Norg_dia(:) = red_Porg_Norg
    red_Porg_Norg_coc(:) = red_Porg_Norg
    red_Corg_Norg_dia(:) = red_Corg_Norg
    red_Corg_Norg_coc(:) = red_Corg_Norg
! reciprocal basic POM
    red_Corg_Porg_dia(:) = 1.0 / red_Porg_Corg_dia(:)
    red_Corg_Porg_coc(:) = 1.0 / red_Porg_Corg_coc(:)
    red_Norg_Porg_dia(:) = 1.0 / red_Porg_Norg_dia(:)
    red_Norg_Porg_coc(:) = 1.0 / red_Porg_Norg_coc(:)
    red_Norg_Corg_dia(:) = 1.0 / red_Corg_Norg_dia(:)
    red_Norg_Corg_coc(:) = 1.0 / red_Corg_Norg_coc(:)
! user-defined values
    red_Corg_opal_dia(:)  = par_red_Corg_opal_dia 
    red_Corg_cal_coc(:)   = par_red_Corg_cal_coc
    red_Corg_arg_coc(:)   = par_red_Corg_arg_coc
    red_Feorg_Corg_dia(:) = par_red_Feorg_Corg_dia
    red_Feorg_Corg_coc(:) = par_red_Feorg_Corg_coc 
! values affected by [Fe]
    red_Corg_Feorg_dia(:) = 1.0 / red_Feorg_Corg_dia(:)
    red_Corg_Feorg_coc(:) = 1.0 / red_Feorg_Corg_coc(:)
    red_Porg_Feorg_dia(:) = red_Porg_Corg_dia(:) * red_Corg_Feorg_dia(:)
    red_Porg_Feorg_coc(:) = red_Porg_Corg_coc(:) * red_Corg_Feorg_coc(:)
    red_Feorg_Porg_dia(:) = 1.0 / red_Porg_Feorg_dia(:)
    red_Feorg_Porg_coc(:) = 1.0 / red_Porg_Feorg_coc(:)
    red_Porg_opal_dia(:)  = red_Porg_Corg_dia(:) * red_Corg_opal_dia(:)
    red_opal_Porg_dia(:)  = 1.0 / red_Porg_opal_dia(:)
    red_opal_Corg_dia(:)  = red_opal_Porg_dia(:) * red_Porg_Corg_dia(:)

  END SUBROUTINE calc_rednp
! *********************************


! *********************************************************
! *** SET FIXED MAGNITUDE NEW PRODUCTION (MOL M-2 YR-1) ***
! *********************************************************
  SUBROUTINE set_bionp_fixed(bionpfixed)
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_bionp)::bionpfixed

! *** calculate new (fixed) production ***
! NOTE: fluxes in (mol m-2 yr-1)
    SELECT CASE (bioopt(bopt_np_alt))
    CASE ("2")
      bionpfixed(:,ibionp_Porg_dia) = red_Corg_Porg_dia(:) * &
	& (bionpfixed(:,ibionp_Corg_dia) + bionpfixed(:,ibionp_DOC_dia))
      bionpfixed(:,ibionp_Porg_coc) = red_Corg_Porg_coc(:) * &
	& (bionpfixed(:,ibionp_Corg_coc) + bionpfixed(:,ibionp_DOC_coc))
    CASE ("3")
! set primary new production variables
      bionpfixed(:,ibionp_Corg_dia)     = bionpfixed(:,ibionp_Corg_dia)
      bionpfixed(:,ibionp_Corg_coc)     = bionpfixed(:,ibionp_Corg_coc)
      bionpfixed(:,ibionp_cal_coc)      = bionpfixed(:,ibionp_cal_coc)
      bionpfixed(:,ibionp_arg_coc)      = bionpfixed(:,ibionp_arg_coc)
      bionpfixed(:,ibionp_opal_dia)     = bionpfixed(:,ibionp_opal_dia)
      bionpfixed(:,ibionp_Feorg_tot)    = bionpfixed(:,ibionp_Feorg_tot)
      bionpfixed(:,ibionp_13C_Corg_dia) = bionpfixed(:,ibionp_13C_Corg_dia)
      bionpfixed(:,ibionp_13C_Corg_coc) = bionpfixed(:,ibionp_13C_Corg_coc)
      bionpfixed(:,ibionp_13C_cal_coc)  = bionpfixed(:,ibionp_13C_cal_coc)
      bionpfixed(:,ibionp_13C_arg_coc)  = bionpfixed(:,ibionp_13C_arg_coc)
      bionpfixed(:,ibionp_DOC_dia)      = bionpfixed(:,ibionp_DOC_dia)
      bionpfixed(:,ibionp_DOC_coc)      = bionpfixed(:,ibionp_DOC_coc)
      bionpfixed(:,ibionp_DOFe_tot)     = bionpfixed(:,ibionp_DOFe_tot)
      bionpfixed(:,ibionp_13C_DOC_dia)  = bionpfixed(:,ibionp_13C_DOC_dia)
      bionpfixed(:,ibionp_13C_DOC_coc)  = bionpfixed(:,ibionp_13C_DOC_coc)
! calculate derived new production variables
      bionpfixed(:,ibionp_Porg_dia)     = red_Corg_Porg_dia(:) * bionpfixed(:,ibionp_Corg_dia)
      bionpfixed(:,ibionp_Porg_coc)     = red_Corg_Porg_coc(:) * bionpfixed(:,ibionp_Corg_coc)
      bionpfixed(:,ibionp_Porg_tot)     = &
	& bionpfixed(:,ibionp_Porg_dia) + bionpfixed(:,ibionp_Porg_coc)
      bionpfixed(:,ibionp_Corg_tot)     = &
	& bionpfixed(:,ibionp_Corg_dia) + bionpfixed(:,ibionp_Corg_coc)
      bionpfixed(:,ibionp_DOC_tot)      = &
	& bionpfixed(:,ibionp_DOC_dia) + bionpfixed(:,ibionp_DOC_coc)
      bionpfixed(:,ibionp_Norg_tot)     = &
	& red_Corg_Norg_dia(:) * bionpfixed(:,ibionp_Corg_dia) + &
	& red_Corg_Norg_coc(:) * bionpfixed(:,ibionp_Corg_coc)
      bionpfixed(:,ibionp_CaCO3_coc)    = &
	& bionpfixed(:,ibionp_cal_coc) + bionpfixed(:,ibionp_arg_coc)
      bionpfixed(:,ibionp_13C_Corg_tot) = &
	& bionpfixed(:,ibionp_13C_Corg_dia) + bionpfixed(:,ibionp_13C_Corg_coc)
      bionpfixed(:,ibionp_13C_DOC_tot)  = &
	& bionpfixed(:,ibionp_13C_DOC_dia) + bionpfixed(:,ibionp_13C_DOC_coc)
    CASE default
      bionpfixed(:,:) = 0.0
    END SELECT

  END SUBROUTINE set_bionp_fixed
! *********************************************************


! *****************************************************************
! *** calculate mean solar insolation as a function of latitude ***
! *****************************************************************
  SUBROUTINE calc_seas_insol(phys)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
    INTEGER::c
    INTEGER::day_num
    INTEGER::hour_num
    INTEGER::t
    INTEGER::t_np_num
    INTEGER::t_int
    INTEGER::out=6
    REAL(KIND=fp_1)::phi
    REAL(KIND=fp_1)::psi
    REAL(KIND=fp_1)::h
    REAL(KIND=fp_1)::delta
    REAL(KIND=fp_1)::d
    REAL(KIND=fp_1)::SL
    REAL(KIND=fp_1)::insol_temp
    REAL(KIND=fp_1)::insol_max
    REAL(KIND=fp_1),DIMENSION(n_col,1:8640)::insol_hourly
! local constants
    REAL(KIND=fp_1),PARAMETER::lamda_w = 0.0
    REAL(KIND=fp_1),PARAMETER::a1      = 1041.0
    REAL(KIND=fp_1),PARAMETER::a2      = -69.0
    
! *** calculate mean hourly insolation ***
! NOTE: day length of 360 days assumed
! calculate mean hourly insolation for each ocean surface box mean latitude
    DO c = 1,n_col
! convert latitude uints form (degrees) to (radians)
      psi = 2.0 * const_PI * phys(c,1,ip_lat) / 360.0
! START DAY LOOP
      DO day_num = 1,360
! calculate day number
	d = 360.0 * day_num / 360.0
! calculate solar longitude
	SL = 4.871 + 0.0175 * d + 0.033 * SIN(0.0178 * d)
! calculate solar declination
	delta = ASIN(0.398 * SIN(SL))
! START HOUR LOOP
	DO t = 1,24
! calculate hour angle
	  h = -lamda_w + 0.043 * SIN(2.0 * SL) - 0.033 * SIN(0.0178 * d) + 0.262 * t - const_PI
! calculate solar elevation
	  phi = ASIN(SIN(delta) * SIN(psi) + COS(delta) * COS(psi) * COS(h))
! calculate mean hourly insolation
! NOTE: ensure that it is not less than zero
	  insol_temp = a1 * SIN(phi) + a2
	  IF (insol_temp < 0.0) insol_temp = 0.0
	  hour_num = 24 * (day_num - 1) + t
	  insol_hourly(c,hour_num) = insol_temp
	END DO
      END DO
    END DO

! *** calculate mean np time step insolation ***
! initialize result variable
    seas_t_np_insol(:,:) = 0.0
! START OCEAN COLUMN LOOP
    DO c = 1,n_col
! iniialize np loop count
      t_np_num = 1
! START YEAR-HOUR LOOP
      DO hour_num = 1,(24 * 360)
	seas_t_np_insol(t_np_num,c) = seas_t_np_insol(t_np_num,c) + insol_hourly(c,hour_num)
!	seas_t_np_insol((t_np_num + (par_n_t_int * par_n_t_np)),c) = seas_t_np_insol(t_np_num,c)
	IF (hour_num > INT(((t_np_num * 24 * 360) / (par_n_t_int * par_n_t_np)))) THEN
	  t_np_num = t_np_num + 1
	  IF (t_np_num > (par_n_t_int * par_n_t_np)) EXIT
	END IF
      END DO
    END DO

! *** normalize data and return function value ***
! determine maximum mean insolation
    insol_max = MAXVAL(seas_t_np_insol(:,:))
! normalize mean insolation
    seas_t_np_insol(:,:) = seas_t_np_insol(:,:) / insol_max
    
! *** alternative function value ***
! if a latitude-explicit insolation function is not requested,
! then use the default insolation funtion
    IF (bioopt(bopt_np_latinsol) == no) THEN
      DO c = 1,n_col
	seas_t_np_insol(:,c) = phys(c,1,ip_I)
      END DO
    ENDIF

! *** derive int time step insolation from np time step data ***
    DO c = 1,n_col
      DO t_int = 1,par_n_t_int
	seas_t_int_insol(t_int,c) = &
	  & SUM(seas_t_np_insol((1 + (t_int - 1) * par_n_t_np):(t_int * par_n_t_np),c)) &
	  & / REAL(par_n_t_np,KIND=fp_1)
      END DO
    END DO

  END SUBROUTINE calc_seas_insol
! *****************************************************************


! ****************************************
! *** calculate seasonal seaice extent ***
! ****************************************
  SUBROUTINE calc_seas_seaice(phys)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
    INTEGER::c
    INTEGER::t
    INTEGER::t_int
    REAL(KIND=fp_1),DIMENSION(n_col)::insol_min
    REAL(KIND=fp_1),DIMENSION(n_col)::insol_max
    REAL(KIND=fp_1),DIMENSION(n_col)::seaice_min
    REAL(KIND=fp_1),DIMENSION(n_col)::seaice_max

! *** calculate np time step seaice extent ***
    DO c = 1,n_col
      insol_min(c) = 1.0
      insol_max(c) = 0.0
      DO t = 1,n_t_np_max
	IF (seas_t_np_insol(t,c) < insol_min(c)) insol_min(c) = seas_t_np_insol(t,c)
	IF (seas_t_np_insol(t,c) > insol_max(c)) insol_max(c) = seas_t_np_insol(t,c)
      END DO
      seaice_min(c) = phys(c,1,ip_seaice_min)
      seaice_max(c) = phys(c,1,ip_seaice_max)
      DO t = 1,n_t_np_max
	seas_t_np_seaice(t,c) = seaice_max(c) - &
	  & ((seas_t_np_insol(t,c) - insol_min(c)) / (insol_max(c) - insol_min(c))) * &
	  & (seaice_max(c) - seaice_min(c))
      END DO
    END DO

! *** calculate integration time step mean seaice extent
    DO c = 1,n_col
      DO t_int = 1,par_n_t_int
	seas_t_int_seaice(t_int,c) = &
	  & SUM(seas_t_np_seaice((1 + (t_int - 1) * par_n_t_np):(t_int * par_n_t_np),c)) &
	  & / REAL(par_n_t_np,KIND=fp_1)
      END DO
    END DO

  END SUBROUTINE calc_seas_seaice
! ****************************************


! *************************************************************************************************
! *** CALCULATE NEW PRODUCTION (MOL) **************************************************************
! *************************************************************************************************
  SUBROUTINE calc_bionp( &
    & dt,tint,SStracer,ftSSbionp,SSphys,SSPO4, &
    & bionpfixed,SSconv,                       &
    & bionp                                    &
    & )
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::dt
    INTEGER,INTENT(in)::tint
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_tracer)::SStracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_tracer)::ftSSbionp   
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_phys)::SSphys  
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::SSPO4
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_bionp)::bionpfixed
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::SSconv
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_bionp)::bionp
! local variables
    INTEGER::c,t
    INTEGER::t_np
    REAL(KIND=fp_1)::dt_np
    REAL(KIND=fp_1),DIMENSION(n_col)::local_conv_dSSPO4_conc_to_fluxtot ! convert [PO4] uptake to export flux
    REAL(KIND=fp_1),DIMENSION(n_col)::local_conv_dSSPO4_conc_to_fluxden ! convert [PO4] uptake to export flux
    REAL(KIND=fp_1),DIMENSION(n_col)::SS_PO4           ! sea surface [PO4]
    REAL(KIND=fp_1),DIMENSION(n_col)::SS_Si            ! sea surface [Si]
    REAL(KIND=fp_1),DIMENSION(n_col)::SS_Fe            ! sea surface [Fe]
    REAL(KIND=fp_1),DIMENSION(n_col)::SS_Feprime       ! sea surface [Fe']
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_PO4_dia      ! [PO4] decrement from diatom growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_PO4_coc      ! [PO4] decrement from coccolithophore growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_PO4_tot      ! [PO4] decrement from all phytoplankton growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_PO4_ip       ! [PO4] increment due to ocean mixing and gaian ip
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_Si_dia       ! [Si] decrement from diatom growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_Si_coc       ! [Si] decrement from coccolithophore growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_Si_tot       ! [Si] decrement from all phytoplankton growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_Si_ip        ! [Si] increment due to ocean mixing and gaian ip
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_Fe_dia       ! [Fe] decrement from diatom growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_Fe_coc       ! [Fe] decrement from coccolithophore growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_Fe_tot       ! [Fe] decrement from all phytoplankton growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_Fe_ip        ! [Fe] increment due to ocean mixing and gaian ip
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_PO4_dia_tot  ! total [PO4] decrement from diatom growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_PO4_coc_tot  ! total [PO4] decrement from coccolithophore growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_Si_dia_tot   ! total [Si] decrement from diatom growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_Si_coc_tot   ! total [Si] decrement from coccolithophore growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_Fe_dia_tot   ! total [Fe] decrement from diatom growth
    REAL(KIND=fp_1),DIMENSION(n_col)::dSS_Fe_coc_tot   ! total [Fe] decrement from coccolithophore growth
    REAL(KIND=fp_1),DIMENSION(n_col)::bio_kPO4_coc     ! coccolithophore Michaelis-Menten PO4 kinetics factor
    REAL(KIND=fp_1),DIMENSION(n_col)::bio_kPO4_dia     ! diatom Michaelis-Menten PO4 kinetics factor
    REAL(KIND=fp_1),DIMENSION(n_col)::bio_kSi_coc      ! coccolithophore Michaelis-Menten Si kinetics factor
    REAL(KIND=fp_1),DIMENSION(n_col)::bio_kSi_dia      ! diatom Michaelis-Menten Si kinetics factor
    REAL(KIND=fp_1),DIMENSION(n_col)::bio_kFe_dia      ! diatom Michaelis-Menten Fe kinetics factor
    REAL(KIND=fp_1),DIMENSION(n_col)::bio_kFe_coc      ! coccolithophore Michaelis-Menten Fe kinetics factor
    REAL(KIND=fp_1),DIMENSION(n_col)::bio_kFeprime_dia ! diatom Michaelis-Menten Fe' kinetics factor
    REAL(KIND=fp_1),DIMENSION(n_col)::bio_kFeprime_coc ! coccolithophore Michaelis-Menten Fe' kinetics factor
    REAL(KIND=fp_1),DIMENSION(n_col)::bio_kT_dia       ! diatom temperature-dependent growth factor
    REAL(KIND=fp_1),DIMENSION(n_col)::bio_kT_coc       ! coccolithophore temperature-dependent growth factor
    REAL(KIND=fp_1),DIMENSION(n_col)::bio_kI_dia       ! diatom temperature-dependent growth factor
    REAL(KIND=fp_1),DIMENSION(n_col)::bio_kI_coc       ! coccolithophore temperature-dependent growth factor


! *** initialize new production calculation ***
! (1) zero new production result array
! (2) calculate time step interval (as a fraction of the derivative function time step)
! (2) set initial local variable values (equal to initial ambient ocncentrations before any new production)
! (3) calculate temperature-dependent kinetics factor values for diataom and coccolithophore growth
! (4) calculate insolation-dependent kinetics factor values diataom and coccolithophore growth
! (5) temperature-dependent kinetic factors
! (6) calculate tracer concetnration increments due to ocean mixing and gaian fluxes to the ocean surface

! (1) zero results variables
    bionp(:,:)       = 0.0
! (2) zero local variables
    dSS_PO4_dia_tot(:) = 0.0
    dSS_PO4_coc_tot(:) = 0.0
    dSS_Si_dia_tot(:)  = 0.0
    dSS_Si_coc_tot(:)  = 0.0
    dSS_Fe_dia_tot(:)  = 0.0
    dSS_Fe_coc_tot(:)  = 0.0
! (3) calculate time step
    dt_np = 1.0 / REAL(par_n_t_np,KIND=fp_1)
! (4) set initial local variable values
    SS_PO4(:) = SStracer(:,it_PO4)
    SS_Si(:)  = SStracer(:,it_Si)
    SS_Fe(:)  = SStracer(:,it_Fe)
! (5) temperature-dependent kinetic factors
!     NOTE: assume that the diatom and coccolithophore temperature-dependent growth functions are identical
    bio_kT_dia(:) = EXP(par_bionp_kT_a1_dia * (SSphys(:,ip_T) - const_zeroc))
    bio_kT_coc(:) = EXP(par_bionp_kT_a1_coc * (SSphys(:,ip_T) - const_zeroc))
! (6) calculate tracer concetnration increments due to ocean mixing and gaian fluxes to the ocean surface
    dSS_PO4_ip(:) = dt_np * ftSSbionp(:,it_PO4) / SSphys(:,ip_M)
    dSS_Si_ip(:)  = dt_np * ftSSbionp(:,it_Si)  / SSphys(:,ip_M)
    dSS_Fe_ip(:)  = dt_np * ftSSbionp(:,it_Fe)  / SSphys(:,ip_M)
! (7) calculate local constant for converting change in surface PO4 concentration to fluxes
!     total flux (mol)
    local_conv_dSSPO4_conc_to_fluxtot(:) = (1.0 - seas_t_int_seaice(tint,:)) * SSphys(:,ip_A_bot) * &
      & conv_m3_kg * (SSphys(:,ip_D_bot) - SSphys(:,ip_D_top))
!     flux density (umol cm-2 yr-1)
!     NOTE: on an ice-free basis
    local_conv_dSSPO4_conc_to_fluxden(:) = (1.0 / (dt * dt_np)) * conv_cm2_m2 * conv_mol_umol * &
      & conv_m3_kg * (SSphys(:,ip_D_bot) - SSphys(:,ip_D_top))


! *** calculate new production ***

    DO t = 1,par_n_t_np

      t_np = t + (tint - 1) * par_n_t_np

! *** calculate new production (PO4-based) ***
! NOTE: units at this point are in mol P kg-1
! NOTE: new production calculated initially is in units of (mol kg-1),
!       and is irrespsective of either surface layer depth or sea ice coverage
! NOTE: fixed np is in units of (mol m-2 yr-1)
! the scheme for this is as follows:
! (A) set 'insolation' factor
! (B) update ocean surface tracer concentrations due to ocean mixing and gaian inputs
! (C) estimate [Fe(III)']
! (D) calculate variable Fe-related 'Redfield Ratios'
! (E) calculate diatom Michaelis-Menten kinetic growth limitation factors
! (F) calculate coccolithophore Michaelis-Menten kinetic growth limitation factors
! (G) estimate diatom-based new production and [PO4], [Si], and [Fe] consumed
! (H) estimate coccolithophore-based new production and [PO4], [Si], and [Fe] consumed
! (I) recalculate tracer concentrations
! (J) update estimate of total [PO4]-based new production
! (K) store SS nutrient and uptake data for current time step

! (A) set 'insolation' factor
      IF (bioopt(bopt_np_latinsol) == yes) THEN
	bio_kI_dia(:) = seas_t_np_insol(t_np,:)
	bio_kI_coc(:) = seas_t_np_insol(t_np,:)
      ELSE
	bio_kI_dia(:) = SSphys(:,ip_I)
	bio_kI_coc(:) = SSphys(:,ip_I)
      END IF
      DO c = 1,n_col
	IF (SSconv(c) > 0.0) THEN
	  bio_kI_dia(c) = par_bionp_kconv * bio_kI_dia(c)
	  bio_kI_coc(c) = par_bionp_kconv * bio_kI_coc(c)
	END IF
      END DO
! (B) update ocean surface tracer concentrations due to ocean mixing and gaian inputs
      SS_PO4(:)     = SS_PO4(:) + dSS_PO4_ip(:)
      SS_Si(:)      = SS_Si(:)  + dSS_Si_ip(:)
      SS_Fe(:)      = SS_Fe(:)  + dSS_Fe_ip(:)
! (C) estimate the concentration of uncomplexed iron(III) ([Fe(III)'])
      SS_Feprime(:) = par_conv_Fe_tot_inorg * SS_Fe(:)
! (D) calculate variable Fe-related 'Redfield Ratios'
!     NOTE: the power law regression equation used to estimate Fe:C requires 
!           dissolved INORGANIC iron ([Fe']), so that this must be first estimated from 
!           total dissolved iron ([Fe] or [Fe(tot)]) -
!           as an first-order fudge, it will be assumed that a fixed fraction of ligand-bound Fe
!           is made bioavailable by whatever mechanism
!     NOTE: as an aid to computational efficiency in this loop, the following has been substituted:
!           red_Porg_Corg_dia(:) / red_Feorg_Corg_dia(:) == red_Porg_Corg_dia(:) * red_Corg_Feorg_dia(:)
      IF (bioopt(bopt_np_fixed_FetoC_dia) == no) THEN
	red_Feorg_Corg_dia(:) = calc_red_Feorg_Corg_dia(SS_Feprime(:))
	red_Porg_Feorg_dia(:) = red_Porg_Corg_dia(:) / red_Feorg_Corg_dia(:)
      ENDIF
      IF (bioopt(bopt_np_fixed_FetoC_coc) == no) THEN
	red_Feorg_Corg_coc(:) = calc_red_Feorg_Corg_coc(SS_Feprime(:))
	red_Porg_Feorg_coc(:) = red_Porg_Corg_coc(:) / red_Feorg_Corg_coc(:)
      ENDIF
      IF (bioopt(bopt_np_fixed_Corgtoopal_dia) == no) THEN
	red_Corg_opal_dia(:)  = calc_red_Corg_opal_dia(SS_Fe(:))
	red_Porg_opal_dia(:)  = red_Porg_Corg_dia * red_Corg_opal_dia(:)
      ENDIF
! (E) calculate diatom Michaelis-Menten kinetic growth limitation factors
      bio_kPO4_dia(:) = SS_PO4(:) / (par_bionp_PO4_0_dia + SS_PO4(:))
      bio_kSi_dia(:)  = SS_Si(:)  / (par_bionp_Si_0_dia + SS_Si(:))
      bio_kFe_dia(:)  = SS_Fe(:)  / (par_bionp_Fe_0_dia + SS_Fe(:))
! (F) calculate coccolithophore Michaelis-Menten kinetic growth limitation factors
      bio_kPO4_coc(:) = SS_PO4(:) / (par_bionp_PO4_0_coc + SS_PO4(:))
      bio_kSi_coc(:)  = 1.0
      bio_kFe_coc(:)  = SS_Fe(:)  / (par_bionp_Fe_0_coc + SS_Fe(:))
! (G) estimate diatom-based new production
!     NOTE: in units of (mol kg-1)
!     NOTE: restricts the growth of silica-shelled species through the minimum of:
!     (a) [PO4] Michaelis-Menten kinetics factor
!     (b) [Si] Michaelis-Menten kinetics factor
!     (c) [Fe] Michaelis-Menten kinetics factor
!     NOTE: ensure that a -ve quantity of nutrients are not consumed
!           (such as might mappen if one of the limting nutrients goes -ve, 
!           which will result in the Michaelis-Menten term going -ve)
!     NOTE: the +ve nutrient consumption check allows for any one of the 3 primary limiting nutrients
!           to temporary be depleted slightly below zero
      SELECT CASE (bioopt(bopt_np_alt))
      CASE ("1")
	DO c = 1,n_col
	  dSS_PO4_dia(c) = dt * dt_np                            * &
	    & bio_kT_dia(c) * bio_kI_dia(c)                      * &
	    & MIN(bio_kPO4_dia(c),bio_kSi_dia(c),bio_kFe_dia(c)) * &
	    & par_bionp_dSSPO4_dia
	  IF (MIN(SS_PO4(c),SS_Si(c),SS_Fe(c)) < 0.0) THEN
	    dSS_PO4_dia(c) = 0.0
	  END IF
	ENDDO
      CASE ("2")
	DO c = 1,n_col
	  dSS_PO4_dia(c) = dt * dt_np * bionpfixed(c,ibionp_Porg_dia) *          &
	    & (1.0 / (conv_m3_kg * (SSphys(c,ip_D_bot) - SSphys(c,ip_D_top)))) / &
	    & (1.0 - seas_t_np_seaice(t_np,c))
	END DO
      CASE ("3")

      CASE ("4")
	DO c = 1,n_col
	  dSS_PO4_dia(c) = (par_bionp_dSSPO4_dia / (par_bionp_dSSPO4_dia + par_bionp_dSSPO4_coc)) * &
	    & ((dt * dt_np) / par_misc_SSPO4_tconst) * (SS_PO4(c) - SSPO4(c)) /                     &
	    & (1.0 - seas_t_np_seaice(t_np,c))
	  IF ((dSS_PO4_dia(c) < 0.0) .OR. (SS_Si(c) < 0.0)) THEN
	    dSS_PO4_dia(c) = 0.0
	  END IF
	END DO
      END SELECT
      dSS_Fe_dia(:) = red_Porg_Feorg_dia(:) * dSS_PO4_dia(:)
      dSS_Si_dia(:) = red_Porg_opal_dia(:)  * (1.0 - par_bionp_frac_DOM_dia) * dSS_PO4_dia(:)
! (H) estimate coccolithophore-based new production
!     NOTE: in units of (mol kg-1)
!     NOTE: restricts the growth of carbonate-shelled species through the minimum of:
!     (a) [PO4] Michaelis-Menten kinetics factor
!     (b) [Fe] Michaelis-Menten kinetics factor
!     NOTE: ensure that a -ve quantity of nutrients are not consumed
!           (such as might mappen if one of the limting nutrients goes -ve, 
!           which will result in the Michaelis-Menten term going -ve)
!     NOTE: the +ve nutrient consumption check allows for any one of the 2 primary limiting nutrients
!           to temporary be depleted slightly below zero
      SELECT CASE (bioopt(bopt_np_alt))
      CASE ("1")
	DO c = 1,n_col
	  dSS_PO4_coc(c) = dt * dt_np             * &
	    & bio_kT_coc(c) * bio_kI_coc(c)       * &
	    & MIN(bio_kPO4_coc(c),bio_kFe_coc(c)) * &
	    & par_bionp_dSSPO4_coc
	  IF (MIN(SS_PO4(c),SS_Fe(c)) < 0.0) THEN
	    dSS_PO4_coc(c) = 0.0
	  END IF
	ENDDO
      CASE ("2")
	DO c = 1,n_col
	  dSS_PO4_coc(c) = dt * dt_np * bionpfixed(c,ibionp_Porg_coc) *          &
	    & (1.0 / (conv_m3_kg * (SSphys(c,ip_D_bot) - SSphys(c,ip_D_top)))) / &
	    & (1.0 - seas_t_np_seaice(t_np,c))
	END DO
      CASE ("3")

      CASE ("4")
	DO c = 1,n_col
	  dSS_PO4_coc(c) = (par_bionp_dSSPO4_coc / (par_bionp_dSSPO4_dia + par_bionp_dSSPO4_coc)) * &
	    & ((dt * dt_np) / par_misc_SSPO4_tconst) * (SS_PO4(c) - SSPO4(c)) /                     &
	    & (1.0 - seas_t_np_seaice(t_np,c))
	  IF (dSS_PO4_coc(c) < 0.0) THEN
	    dSS_PO4_coc(c) = 0.0
	  END IF
	END DO
      END SELECT
      dSS_Fe_coc(:) = red_Porg_Feorg_coc(:) * dSS_PO4_coc(:)
      dSS_Si_coc(:) = 0.0
! (I) recalculate tracer concentrations
      SS_PO4(:)     = SS_PO4(:) - dSS_PO4_dia(:) - dSS_PO4_coc(:)
      SS_Si(:)      = SS_Si(:)  - dSS_Si_dia(:)  - dSS_Si_coc(:)
      SS_Fe(:)      = SS_Fe(:)  - dSS_Fe_dia(:)  - dSS_Fe_coc(:)
! (J) update estimate of total [PO4]-based new production
      dSS_PO4_dia_tot(:) = dSS_PO4_dia_tot(:) + dSS_PO4_dia(:)
      dSS_PO4_coc_tot(:) = dSS_PO4_coc_tot(:) + dSS_PO4_coc(:)
      dSS_Si_dia_tot(:)  = dSS_Si_dia_tot(:)  + dSS_Si_dia(:)
      dSS_Si_coc_tot(:)  = dSS_Si_coc_tot(:)  + dSS_Si_coc(:)
      dSS_Fe_dia_tot(:)  = dSS_Fe_dia_tot(:)  + dSS_Fe_dia(:)
      dSS_Fe_coc_tot(:)  = dSS_Fe_coc_tot(:)  + dSS_Fe_coc(:)
! (K) write SS tracer data for current time step
!     NOTE: write  directly to global array
      seas_t_np_dSS(t_np,:,it_PO4) = SS_PO4(:)
      seas_t_np_dSS(t_np,:,it_Si)  = SS_Si(:)
      seas_t_np_dSS(t_np,:,it_Fe)  = SS_Fe(:)
! (L) write np data for current time step
!     NOTE: write  directly to global array
      seas_t_np_dnp(t_np,:,ibionp_Porg_dia)  = local_conv_dSSPO4_conc_to_fluxden(:) * &
	(1.0 - par_bionp_frac_DOM_dia) * dSS_PO4_dia(:)
      seas_t_np_dnp(t_np,:,ibionp_Porg_coc)  = local_conv_dSSPO4_conc_to_fluxden(:) * &
	& (1.0 - par_bionp_frac_DOM_coc) * dSS_PO4_coc(:)
      seas_t_np_dnp(t_np,:,ibionp_Feorg_tot) = local_conv_dSSPO4_conc_to_fluxden(:) * &
	& ( &
	&   (1.0 - par_bionp_frac_DOM_dia) * dSS_Fe_dia(:) + &
	&   (1.0 - par_bionp_frac_DOM_coc) * dSS_Fe_coc(:) &
	& )
      seas_t_np_dnp(t_np,:,ibionp_opal_dia)  = local_conv_dSSPO4_conc_to_fluxden(:) * &
	& dSS_Si_dia(:)
      seas_t_np_dnp(t_np,:,ibionp_DOC_dia)   = local_conv_dSSPO4_conc_to_fluxden(:) * &
	& par_bionp_frac_DOM_dia * red_Porg_Corg_dia(:) * dSS_PO4_dia(:)
      seas_t_np_dnp(t_np,:,ibionp_DOC_coc)   = local_conv_dSSPO4_conc_to_fluxden(:) * &
	& par_bionp_frac_DOM_coc * red_Porg_Corg_coc(:) * dSS_PO4_coc(:)
      seas_t_np_dnp(t_np,:,ibionp_DOFe_tot)  = local_conv_dSSPO4_conc_to_fluxden(:) * &
	& ( &
	&   par_bionp_frac_DOM_dia * dSS_Fe_dia(:) + &
	&   par_bionp_frac_DOM_coc * dSS_Fe_coc(:) &
	& )
! (M) write kinetics data for current time step
!     NOTE: write  directly to global array
      seas_t_np_dk(t_np,:,ibiok_I_dia)   = bio_kI_dia(:)
      seas_t_np_dk(t_np,:,ibiok_I_coc)   = bio_kI_coc(:)
      seas_t_np_dk(t_np,:,ibiok_T_dia)   = bio_kT_dia(:)
      seas_t_np_dk(t_np,:,ibiok_T_coc)   = bio_kT_coc(:)
      seas_t_np_dk(t_np,:,ibiok_PO4_dia) = bio_kPO4_dia(:)
      seas_t_np_dk(t_np,:,ibiok_PO4_coc) = bio_kPO4_coc(:)
      seas_t_np_dk(t_np,:,ibiok_Si_dia)  = bio_kSi_dia(:)
      seas_t_np_dk(t_np,:,ibiok_Si_coc)  = bio_kSi_coc(:)
      seas_t_np_dk(t_np,:,ibiok_Fe_dia)  = bio_kFe_dia(:)
      seas_t_np_dk(t_np,:,ibiok_Fe_coc)  = bio_kFe_coc(:)
      seas_t_np_dk(t_np,:,ibiok_tot_dia) = dSS_PO4_dia(:) / (dt * dt_np * par_bionp_dSSPO4_dia)
      seas_t_np_dk(t_np,:,ibiok_tot_coc) = dSS_PO4_coc(:) / (dt * dt_np * par_bionp_dSSPO4_coc)

    ENDDO


! *** return function value ***
! (1) calculate primary fluxes
!     NOTE: change units from (mol kg-1) to (mol) for each ocean cell
!     NOTE: calculate the mass of each surface cell on the basis of 
!           the bottom surface area of the cell and the layer thicnkess, 
!           rather than on the entire mass of the cell 
!           (as np produced in the photic zone directly above the ocean bottom,
!           is assumed to be reseasd within the layer)
!     NOTE: account must be taken of sea ice coverage at this stage
!     NOTE: account must be taken of the fraction of new production as DOC
    bionp(:,ibionp_Porg_dia)  = local_conv_dSSPO4_conc_to_fluxtot(:) * &
      & (1.0 - par_bionp_frac_DOM_dia) * dSS_PO4_dia_tot(:)
    bionp(:,ibionp_Porg_coc)  = local_conv_dSSPO4_conc_to_fluxtot(:) * &
      & (1.0 - par_bionp_frac_DOM_coc) * dSS_PO4_coc_tot(:)
    bionp(:,ibionp_Feorg_tot) = local_conv_dSSPO4_conc_to_fluxtot(:) * &
      & ( &
      &   (1.0 - par_bionp_frac_DOM_dia) * dSS_Fe_dia_tot(:) + &
      &   (1.0 - par_bionp_frac_DOM_coc) * dSS_Fe_coc_tot(:) &
      & )
    bionp(:,ibionp_opal_dia)  = local_conv_dSSPO4_conc_to_fluxtot(:) * &
      & dSS_Si_dia_tot(:)
! (2) calculate primary DOM fluxes
    bionp(:,ibionp_DOC_dia)   = local_conv_dSSPO4_conc_to_fluxtot(:) * &
      & par_bionp_frac_DOM_dia * red_Porg_Corg_dia(:) * dSS_PO4_dia_tot(:)
    bionp(:,ibionp_DOC_coc)   = local_conv_dSSPO4_conc_to_fluxtot(:) * &
      & par_bionp_frac_DOM_coc * red_Porg_Corg_coc(:) * dSS_PO4_coc_tot(:)
    bionp(:,ibionp_DOFe_tot)  =  local_conv_dSSPO4_conc_to_fluxtot(:) * &
      & ( &
      &   par_bionp_frac_DOM_dia * dSS_Fe_dia_tot(:) + &
      &   par_bionp_frac_DOM_coc * dSS_Fe_coc_tot(:) &
      & )
! (3) calculate derived individual carbon fluxes
    bionp(:,ibionp_Corg_dia)  = red_Porg_Corg_dia(:) * bionp(:,ibionp_Porg_dia)
    bionp(:,ibionp_Corg_coc)  = red_Porg_Corg_coc(:) * bionp(:,ibionp_Porg_coc)
! (4) calculate derived total fluxes
    bionp(:,ibionp_Porg_tot)  = &
      & bionp(:,ibionp_Porg_dia) + &
      & bionp(:,ibionp_Porg_coc)
    bionp(:,ibionp_Corg_tot)  = &
      & bionp(:,ibionp_Corg_dia) + &
      & bionp(:,ibionp_Corg_coc)
    bionp(:,ibionp_Norg_tot)  = &
      & red_Corg_Norg_dia(:) * bionp(:,ibionp_Corg_dia) + &
      & red_Corg_Norg_coc(:) * bionp(:,ibionp_Corg_coc)
! (5) calcite and aragonite, then CaCO3
    bionp(:,ibionp_cal_coc)   = red_Corg_cal_coc(:) * bionp(:,ibionp_Corg_coc)
    bionp(:,ibionp_arg_coc)   = red_Corg_arg_coc(:) * bionp(:,ibionp_Corg_coc)
    bionp(:,ibionp_CaCO3_coc) = bionp(:,ibionp_cal_coc) + bionp(:,ibionp_arg_coc)
! (6) calculate derived total DOM fluxes
    bionp(:,ibionp_DOC_tot)   = &
      & bionp(:,ibionp_DOC_dia) + &
      & bionp(:,ibionp_DOC_coc)
! (7) calculate derived seasonal fluxes
    DO t = (1 + (tint - 1) * par_n_t_np),(tint * par_n_t_np)
      seas_t_np_dnp(t,:,ibionp_Corg_dia) = red_Porg_Corg_dia(:) * seas_t_np_dnp(t,:,ibionp_Porg_dia)
      seas_t_np_dnp(t,:,ibionp_Corg_coc) = red_Porg_Corg_coc(:) * seas_t_np_dnp(t,:,ibionp_Porg_coc)
      seas_t_np_dnp(t,:,ibionp_Porg_tot) = &
	& seas_t_np_dnp(t,:,ibionp_Porg_dia) + &
	& seas_t_np_dnp(t,:,ibionp_Porg_coc)
      seas_t_np_dnp(t,:,ibionp_Corg_tot) = &
	& seas_t_np_dnp(t,:,ibionp_Corg_dia) + &
	& seas_t_np_dnp(t,:,ibionp_Corg_coc)
      seas_t_np_dnp(t,:,ibionp_Norg_tot) = &
	& red_Corg_Norg_dia(:) * seas_t_np_dnp(t,:,ibionp_Corg_dia) + &
	& red_Corg_Norg_coc(:) * seas_t_np_dnp(t,:,ibionp_Corg_coc)
      seas_t_np_dnp(t,:,ibionp_cal_coc) = red_Corg_cal_coc(:) * seas_t_np_dnp(t,:,ibionp_CaCO3_coc)
      seas_t_np_dnp(t,:,ibionp_arg_coc) = red_Corg_arg_coc(:) * seas_t_np_dnp(t,:,ibionp_CaCO3_coc)
      seas_t_np_dnp(t,:,ibionp_CaCO3_coc) = &
	& seas_t_np_dnp(t,:,ibionp_cal_coc) + &
	& seas_t_np_dnp(t,:,ibionp_arg_coc)
      seas_t_np_dnp(t,:,ibionp_DOC_tot) = &
	& seas_t_np_dnp(t,:,ibionp_DOC_dia) + &
	& seas_t_np_dnp(t,:,ibionp_DOC_coc)
    END DO


! *** check function results ***
    IF (runopt(ropt_debug_box) == yes) THEN
! (1) estimated final tracer concentrations
      IF ((MINVAL(SS_PO4(:)) < 0.0 .OR. MINVAL(SS_Si(:)) < 0.0 .OR. MINVAL(SS_Fe(:)) < 0.0)) THEN
	PRINT*,"FATAL ERROR [sue_box.f90, calc_bionp]:"
	PRINT*,"estimated new production has resulted in nutrients being depleted below zero"
	PRINT*,"--- ini ---"
	PRINT*,SStracer(:,it_PO4)
	PRINT*,SStracer(:,it_Si)
	PRINT*,SStracer(:,it_Fe)
	PRINT*,"--- fin ---"
	PRINT*,SS_PO4(:)
	PRINT*,SS_Si(:)
	PRINT*,SS_Fe(:)
	PRINT*,"--- dia ---"
	PRINT*,bio_kPO4_dia(:)
	PRINT*,bio_kSi_dia(:)
	PRINT*,bio_kFeprime_dia(:)
	PRINT*,bio_kT_dia(:)
	PRINT*,bio_kI_dia(:)
	PRINT*,"--- coc ---"
	PRINT*,bio_kPO4_coc(:)
	PRINT*,bio_kSi_coc(:)
	PRINT*,bio_kFeprime_coc(:)
	PRINT*,bio_kT_coc(:)
	PRINT*,bio_kI_coc(:)
	STOP
      ENDIF
! (2) estimated np flux
      IF (MINVAL(bionp(:,:)) < 0.0) THEN
	PRINT*,"FATAL ERROR [sue_box.f90, calc_bionp]:"
	PRINT*,"estimated new production flux < 0.0"
	PRINT*,bionp(:,:)
	STOP
      ENDIF
    ENDIF


  END SUBROUTINE calc_bionp
! *************************************************************************************************


! *********************************************
! *** estimate diatom Fe:C 'Redfield Ratio' ***
! *********************************************
! NOTE: the power law regression equation used to estimate Fe:C requires dissolved INORGANIC iron ([Fe']),
!       NOT total dissolved iron ([Fe] or [Fe(tot)])
  FUNCTION calc_red_Feorg_Corg_dia(Feprime)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(n_col)::calc_red_Feorg_Corg_dia
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::Feprime
! local variables
    INTEGER::c
! calculate and return function value
    DO c = 1,n_col
      IF (Feprime(c) < 0.0) THEN
	calc_red_Feorg_Corg_dia(c) = 9.9910E9
      ELSEIF (Feprime(c) < red_Feorg_Corg_dia_offset) THEN
	calc_red_Feorg_Corg_dia(c) = par_red_Feorg_Corg_dia_max
      ELSE
	calc_red_Feorg_Corg_dia(c) = red_Feorg_Corg_dia_const + red_Feorg_Corg_dia_scale * &
	  & (conv_mol_pmol * (Feprime(c) - red_Feorg_Corg_dia_offset))** &
	  & red_Feorg_Corg_dia_power
	IF (calc_red_Feorg_Corg_dia(c) > par_red_Feorg_Corg_dia_max) THEN
	  calc_red_Feorg_Corg_dia(c) = par_red_Feorg_Corg_dia_max
	ENDIF
      ENDIF
    END DO
  END FUNCTION calc_red_Feorg_Corg_dia
! *********************************************


! ************************************************
! *** estimate coccolith Fe:C 'Redfield Ratio' ***
! ************************************************
! NOTE: the power law regression equation used to estimate Fe:C requires dissolved INORGANIC iron ([Fe']),
!       NOT total dissolved iron ([Fe] or [Fe(tot)])
  FUNCTION calc_red_Feorg_Corg_coc(Feprime)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(n_col)::calc_red_Feorg_Corg_coc
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::Feprime
! local variables
    INTEGER::c
! calculate and return function value
    DO c = 1,n_col
      IF (Feprime(c) < 0.0) THEN
	calc_red_Feorg_Corg_coc(c) = 9.9910E9
      ELSEIF (Feprime(c) < red_Feorg_Corg_coc_offset) THEN
	calc_red_Feorg_Corg_coc(c) = par_red_Feorg_Corg_coc_max
      ELSE
	calc_red_Feorg_Corg_coc(c) = red_Feorg_Corg_coc_const + red_Feorg_Corg_coc_scale * &
	  & (conv_mol_pmol * (Feprime(c) - red_Feorg_Corg_coc_offset))** &
	  & red_Feorg_Corg_coc_power
	IF (calc_red_Feorg_Corg_coc(c) > par_red_Feorg_Corg_coc_max) THEN
	  calc_red_Feorg_Corg_coc(c) = par_red_Feorg_Corg_coc_max
	ENDIF
      ENDIF
    END DO
  END FUNCTION calc_red_Feorg_Corg_coc
! ************************************************


! *********************************************
! *** estimate diatom C:Si 'Redfield Ratio' ***
! *********************************************
! NOTE: the estimate for the variable opal:Corg export ratio is based on 
!       Michelis-Menten Fe-limitation kinetics
  FUNCTION calc_red_Corg_opal_dia(Fe)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1),DIMENSION(n_col)::calc_red_Corg_opal_dia
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::Fe
! local variables
    INTEGER::c
    REAL(KIND=fp_1)::Fe_var
! calculate and return function value
    DO c = 1,n_col
      IF (Fe(c) < 0.0) THEN
	Fe_var = 0.0 + par_red_Corg_opal_dia_var_dFe
      ELSE
	Fe_var = Fe(c) + par_red_Corg_opal_dia_var_dFe
      ENDIF
      calc_red_Corg_opal_dia(c) = par_red_Corg_opal_dia_var_r0 * &
	& (1.0 / (Fe_var / (Fe_var + par_red_Corg_opal_dia_var_Fe0)))
    END DO
  END FUNCTION calc_red_Corg_opal_dia
! *********************************************


! *****************************
! *** SET OCEAN CIRCULATION ***
! *****************************
  SUBROUTINE set_ocir(ocir,oc)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::ocir
    TYPE(ocean_circulation),INTENT(inout),DIMENSION(1:par_n_wflux)::oc
! set ocean circulation
    oc(:)%ioc_wf = ocir * oc(:)%ioc_wf_1 + (1.0 - ocir) * oc(:)%ioc_wf_2
  END SUBROUTINE set_ocir
! *****************************


! *************************************************************************************************
! *** THE FUNCTION FOR THE DERIVATIVE *************************************************************
! *************************************************************************************************
  FUNCTION dsue(dt,tint,oatracer,goatracer,oc,SSdis,phys,oconvadjfixed,sedpar,sedcorpar,&
    & SST,SSS,SSPO4,bionpfixed,fdust)
    IMPLICIT NONE
! result variable
    TYPE(derivative_function)::dsue
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::dt
    INTEGER,INTENT(in)::tint
    TYPE(ocean_atm_tracer),INTENT(in)::oatracer
    TYPE(ocean_atm_tracer),INTENT(in)::goatracer
    TYPE(ocean_circulation),INTENT(in),DIMENSION(1:par_n_wflux)::oc
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_dis)::SSdis
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_t_int_max)::oconvadjfixed
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_sed_par)::sedcorpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::SST
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::SSS
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::SSPO4
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_bionp)::bionpfixed
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::fdust
! local variables
    INTEGER::c,l,m,p,v
    INTEGER::it                                                 ! ocean tracer index
    INTEGER::ia                                                 ! atmosphere tracer index
    INTEGER::t_int                                              ! integration step
    INTEGER::m_nptop                                            ! uppermost sediment below base of mixed layer
    INTEGER::m_npbot                                            ! bottommost sediment in contact with ocean
    REAL(KIND=fp_1)::r13C_pCO2                                  ! atmospheric 13C/12C tracer isotopic ratio
    REAL(KIND=fp_1)::r18O_pO2                                   ! atmospheric 18O/16O tracer isotopic ratios
    REAL(KIND=fp_1)::Fe_excess                                  ! excess Fe (above ligand concentration)
    REAL(KIND=fp_1)::len_scav                                   ! scavenging length (m)
    REAL(KIND=fp_1)::delta_reminf_Corg                          ! 
    REAL(KIND=fp_1)::delta_reminf_opal                          ! 
    REAL(KIND=fp_1)::delta_reminf_cal                           ! 
    REAL(KIND=fp_1)::delta_reminf_arg                           ! 
    REAL(KIND=fp_1)::delta_reminf_refrac                        ! 
    REAL(KIND=fp_1)::ratio_reminf_Corg                          ! 
    REAL(KIND=fp_1)::ratio_reminf_opal                          ! 
    REAL(KIND=fp_1)::ratio_reminf_cal                           ! 
    REAL(KIND=fp_1)::ratio_reminf_arg                           ! 
    REAL(KIND=fp_1)::ratio_reminf_refrac                        ! 
    REAL(KIND=fp_1)::k_scav_Fe_Corg                             ! Fe scavenging constant
    REAL(KIND=fp_1)::k_scav_Fe_opal                             ! 
    REAL(KIND=fp_1)::k_scav_Fe_cal                              ! 
    REAL(KIND=fp_1)::k_scav_Fe_arg                              ! 
    REAL(KIND=fp_1)::k_scav_Fe_refrac                           ! 
    REAL(KIND=fp_1),DIMENSION(n_lay)::fscav_Fe_Corg             ! scavenged Fe flux
    REAL(KIND=fp_1),DIMENSION(n_lay)::fscav_Fe_opal             ! 
    REAL(KIND=fp_1),DIMENSION(n_lay)::fscav_Fe_cal              ! 
    REAL(KIND=fp_1),DIMENSION(n_lay)::fscav_Fe_arg              ! 
    REAL(KIND=fp_1),DIMENSION(n_lay)::fscav_Fe_refrac           ! 
    REAL(KIND=fp_1),DIMENSION(n_col,n_lay)::r13C_CO2            ! ocean 13C/12C tracer isotopic ratios
    REAL(KIND=fp_1),DIMENSION(n_col,n_lay)::r18O_O2             ! ocean 18O/16O tracer isotopic ratios
    REAL(KIND=fp_1),DIMENSION(n_col)::fug_CO2                   ! CO2 fugacity in surface ocean
    REAL(KIND=fp_1),DIMENSION(n_col)::conc_O2_old               ! [O2] in surface ocean (old)
    REAL(KIND=fp_1),DIMENSION(n_col)::conc_O2_new               ! [O2] in surface ocean (new)
    REAL(KIND=fp_1),DIMENSION(n_col)::conc_O2_eqm               ! [O2] in surface ocean (equilibrium)
    REAL(KIND=fp_1),DIMENSION(n_col)::conc_CO2                  ! surface [CO2]
    REAL(KIND=fp_1),DIMENSION(n_col)::conc_CO3                  ! surface [CO3]
    REAL(KIND=fp_1),DIMENSION(n_col)::conc_HCO3                 ! surface [HCO3]
    REAL(KIND=fp_1),DIMENSION(n_col)::d13C_ocn_atm              ! ocean-atm 13C fractionation
    REAL(KIND=fp_1),DIMENSION(n_col)::d13C_atm_ocn              ! atm-ocean 13C fractionation
    REAL(KIND=fp_1),DIMENSION(n_col)::d13C_DIC_cal              !
    REAL(KIND=fp_1),DIMENSION(n_col)::d13C_DIC_arg              !
    REAL(KIND=fp_1),DIMENSION(n_col)::d13C_DIC_Corg_dia         ! DIC->Corg 13C fractionation (dia)
    REAL(KIND=fp_1),DIMENSION(n_col)::d13C_DIC_Corg_coc         ! DIC->Corg 13C fractionation (coc)
    REAL(KIND=fp_1),DIMENSION(n_col)::d13C_DIC_Corg_Qsce        ! Rau et al. [1997] factor
    REAL(KIND=fp_1),DIMENSION(n_col)::epsilon_beta              ! pCO2     -> fCO2   d13C fractionation fact
    REAL(KIND=fp_1),DIMENSION(n_col)::epsilon_1                 ! [HCO3]   -> [CO2]  d13C fractionation factor
    REAL(KIND=fp_1),DIMENSION(n_col)::epsilon_2                 ! [CO3]    -> [HCO3] d13C fractionation factor
    REAL(KIND=fp_1),DIMENSION(n_col)::epsilon_3                 ! [CO3]    -> [CO2]  d13C fractionation factor
    REAL(KIND=fp_1),DIMENSION(n_col)::epsilon_SUM               ! SUM(CO2) -> [CO2] d13C fractionation factor
    REAL(KIND=fp_1),DIMENSION(n_tracer)::ft_abio_del            ! abioloigcal ocean tracer flux change
    REAL(KIND=fp_1),DIMENSION(n_col,n_lay,n_tracer)::ft_bio     ! bioloigcal ocean tracer fluxes
    REAL(KIND=fp_1),DIMENSION(n_col,n_lay,n_tracer)::ft_abio    ! abioloigcal ocean tracer fluxes
    REAL(KIND=fp_1),DIMENSION(n_lay,n_tracer)::ft_abio_conv     ! abioloigcal ocean tracer fluxes (convection)
    REAL(KIND=fp_1),DIMENSION(n_col,n_tracer)::ft_SSbionp       ! ocean surface tracer fluxes to support np
    REAL(KIND=fp_1),DIMENSION(n_atm)::fa_bio                    ! bioloigcal atmospheric tracer fluxes
    REAL(KIND=fp_1),DIMENSION(n_atm)::fa_abio                   ! abioloigcal atmospheric tracer fluxes
    REAL(KIND=fp_1),DIMENSION(n_col,n_bionp)::bionp             ! bioloigcal new production
    REAL(KIND=fp_1),DIMENSION(n_lay,n_bionp)::fremin_bionp      ! remineralized np flux in each layer
    REAL(KIND=fp_1),DIMENSION(n_bionp)::sink_bionp              ! sinking np (above each sediment layer)
    REAL(KIND=fp_1),DIMENSION(n_lay)::fexport_POM               ! POM export flux at each layer's base (molC)
    REAL(KIND=fp_1),DIMENSION(n_lay)::sed_lay_frac              ! sediment/ocean layer fractional adjacencies
    REAL(KIND=fp_1),DIMENSION(n_col,n_sscc)::SScarbchem         ! ocean surface carbonate equilibrium
    REAL(KIND=fp_1),DIMENSION(n_col)::pvgas_k_CO2               ! gas transfer velocity (CO2)
    REAL(KIND=fp_1),DIMENSION(n_col)::pvgas_Sc_CO2              ! Schmidt number (CO2)
    REAL(KIND=fp_1),DIMENSION(n_col)::fCO2_ocn_atm              ! ocean -> atmosphere CO2 flux
    REAL(KIND=fp_1),DIMENSION(n_col)::fCO2_atm_ocn              ! atmosphere -> ocean CO2 flux
    REAL(KIND=fp_1),DIMENSION(n_col)::f13CO2_ocn_atm            ! ocean -> atmosphere 13CO2 flux
    REAL(KIND=fp_1),DIMENSION(n_col)::f13CO2_atm_ocn            ! atmosphere -> ocean 13CO2 flux
    REAL(KIND=fp_1),DIMENSION(n_col)::fO2_ocn_atm               ! ocean -> atmosphere O2 flux
    REAL(KIND=fp_1),DIMENSION(n_col)::fO2_atm_ocn               ! atmosphere -> ocean O2 flux
    REAL(KIND=fp_1),DIMENSION(n_col)::f18O2_ocn_atm             ! ocean -> atmosphere 18O2 flux
    REAL(KIND=fp_1),DIMENSION(n_col)::f18O2_atm_ocn             ! atmosphere -> ocean 18O2 flux
    INTEGER::conv_l                                             ! ocean layer number (in a convecting column)
    INTEGER::conv_l_top                                         ! top ocean layer of a convecting column
    INTEGER::conv_l_bot                                         ! bottom ocean layer of a convecting column
    INTEGER::conv_mixD_l_bot                                    ! bottom ocean layer of mixed layer
    REAL(KIND=fp_1)::conv_M_tot                                 ! total mass in a convecting column
    REAL(KIND=fp_1)::conv_T_tot                                 ! total temperature in a convecting column
    REAL(KIND=fp_1)::conv_S_top                                 ! total salinity in a convecting column
    REAL(KIND=fp_1)::conv_O2_top                                ! total O2 in a convecting column
    REAL(KIND=fp_1)::conv_M_top                                 ! top mass in a convecting column
    REAL(KIND=fp_1)::conv_T_top                                 ! top temperature in a convecting column
    REAL(KIND=fp_1)::conv_S_tot                                 ! top salinity in a convecting column
    REAL(KIND=fp_1)::conv_O2_tot                                ! top O2 in a convecting column
    REAL(KIND=fp_1)::conv_M_bot                                 ! bottom mass in a convecting column
    REAL(KIND=fp_1)::conv_T_bot                                 ! bottom temperature in a convecting column
    REAL(KIND=fp_1)::conv_S_bot                                 ! bottom salinity in a convecting column
    REAL(KIND=fp_1)::conv_O2_bot                                ! bottom O2 in a convecting column
    REAL(KIND=fp_1)::conv_rho_u                                 ! upper layer density
    REAL(KIND=fp_1)::conv_rho_d                                 ! lower layer density
    REAL(KIND=fp_1),DIMENSION(n_tracer)::conv_ftracer_tot       ! convected tracers in a convecting column
    REAL(KIND=fp_1),DIMENSION(n_lay)::conv_adj                  ! convection adjustment strength
    REAL(KIND=fp_1),DIMENSION(n_lay)::conv_adj_dw               ! deepwater adjustment strength
    INTEGER,DIMENSION(n_col)::conv_sur_l_top                    ! top layer of conv column reaching surface
    INTEGER,DIMENSION(n_col)::conv_sur_l_bot                    ! bottom layer of conv column reaching surface
    REAL(KIND=fp_1)::hdif_interA                                ! interface area for horizontal eddy diffusion
    REAL(KIND=fp_1)::hdif_lat_N_s                               ! N lat of start grid point of flux vector
    REAL(KIND=fp_1)::hdif_lat_S_s                               ! S lat of start grid point of flux vector
    REAL(KIND=fp_1)::hdif_lat_N_e                               ! N lat of end grid point of flux vector
    REAL(KIND=fp_1)::hdif_lat_S_e                               ! S lat of end grid point of flux vector
    REAL(KIND=fp_1),DIMENSION(n_col)::SST_new                   ! new SSTs
    REAL(KIND=fp_1),DIMENSION(n_col)::SST_old                   ! old SSTs
    REAL(KIND=fp_1),DIMENSION(n_col)::SSS_new                   ! new SSSs
    REAL(KIND=fp_1),DIMENSION(n_col)::SSS_old                   ! old SSSs
    REAL(KIND=fp_1),DIMENSION(n_col)::SSO2_eqm                  ! 
    REAL(KIND=fp_1),DIMENSION(n_col)::SSO2_new                  ! 
    REAL(KIND=fp_1),DIMENSION(n_col)::SSO2_old                  ! 


! *** function initialzation ***

! initialize function value
    dsue%idf_t = 0.0
    dsue%idf_a = 0.0
    dsue%idf_s = 0.0
    dsue%idf_c = 0.0
    dsue%idf_b = 0.0
    dsue%idf_v = 0.0
    dsue%idf_g = 0.0
! initialize local variables
    ft_bio  = 0.0
    ft_abio = 0.0
    fa_bio  = 0.0
    fa_abio = 0.0
! calculate ocean surface carbonate equilibrium
    CALL calc_sur_carbchem_Hin(SScarbchem,phys(:,1,:),oatracer%ioat_t(:,1,:),SSdis)
! set local ocean surface carbonate equilibrium variables
    fug_CO2   = SScarbchem(:,isscc_fCO2)
    conc_CO2  = SScarbchem(:,isscc_cCO2)
    conc_CO3  = SScarbchem(:,isscc_cCO3)
    conc_HCO3 = SScarbchem(:,isscc_cHCO3)
! calculate ratios of 13C:C and 18O:O
    r13C_CO2  = oatracer%ioat_t(:,:,it_13C_CO2) / oatracer%ioat_t(:,:,it_CO2)
    r13C_pCO2 = oatracer%ioat_a(ia_13C_pCO2)    / oatracer%ioat_a(ia_pCO2)
    r18O_O2   = oatracer%ioat_t(:,:,it_18O_O2)  / oatracer%ioat_t(:,:,it_O2)
    r18O_pO2  = oatracer%ioat_a(ia_18O_pO2)     / oatracer%ioat_a(ia_pO2)


! *** calculate d13C fractionation ***

! ocean-atmosphere CO2 system
    SELECT CASE (miscopt(mopt_d13atm_alt))
    CASE ("1")
! scheme #1 : after Mook et al. [1974], Mook [1986], Siegenthaler and Munnich [1981], and Vogel et al. [1970]
! NOTE: similar to that used by Yamanaka and Tajika [1996],
!       other than the kinetic fractionation is larger
!       and the expresion for 'alpha_2' is taken from Vogel et al. [1970] rather than Mook et al. [1974]
      epsilon_beta(:) = +00.19 - 0373.0 / phys(:,1,ip_T)
      epsilon_1(:)    = +24.12 - 9866.0 / phys(:,1,ip_T)
      epsilon_2(:)    = -07.10
      epsilon_SUM(:)  = (epsilon_1(:) * conc_HCO3(:) + epsilon_2(:) * conc_CO3(:)) / &
	& (conc_CO2(:) + conc_HCO3(:) + conc_CO3(:))
      d13C_atm_ocn(:) = epsilon_beta(:) + -00.90
      d13C_ocn_atm(:) = epsilon_SUM(:)  + -00.90
    CASE ("2")
! scheme #2 : after Zhang et al. [1995]
! NOTE: very similar to that used by Marchal et al. [1998]
! NOTE: atm->ocn fractionation corrected by -0.2 o/oo as suggested by Zhang et al. [1995] 
!       in order to compare their artificial solution experimental results with seawater, 
!       one must account for the fractionation during the reaction of CO2 with hydroxide
      epsilon_beta(:) = -02.65 + 0.005 * phys(:,1,ip_T)
      epsilon_1(:)    = -44.57 + 0.119 * phys(:,1,ip_T)
      epsilon_2(:)    = -24.07 + 0.057 * phys(:,1,ip_T)
      epsilon_SUM(:)  = (epsilon_1(:) * conc_HCO3(:) + epsilon_2(:) * conc_CO3(:)) / &
	& (conc_CO2(:) + conc_HCO3(:) + conc_CO3(:))
      d13C_atm_ocn(:) = epsilon_beta(:) + -00.88 + -00.20
      d13C_ocn_atm(:) = epsilon_SUM(:)  + -00.88
    CASE ("3")
! scheme #3 : after Mook et al. [1974]
! NOTE: as used by Maier-Reimer [1993] (and others) for atm->ocn and ocn->atm gas transfer
! NOTE: the expresions for 'alpha_beta', 'alpha_1', and 'alpha_2' taken from Mook et al. [1974]
!       in order to provide values required for calculating the CaCO3 13C fractionation
      epsilon_beta(:) = +00.19 - 0373.0 / phys(:,1,ip_T)
      epsilon_1(:)    = +24.12 - 9866.0 / phys(:,1,ip_T)
      epsilon_2(:)    = +21.60 - 8999.0 / phys(:,1,ip_T)
      epsilon_SUM(:)  = (epsilon_1(:) * conc_HCO3(:) + epsilon_2(:) * conc_CO3(:)) / &
	& (conc_CO2(:) + conc_HCO3(:) + conc_CO3(:))
      d13C_atm_ocn(:) = +00.00
      d13C_ocn_atm(:) = +23.89 - 9483.0 / phys(:,1,ip_T)
    CASE ("4")
! scheme #4 : simple global averages, after Yamanaka and Tajika [1996]
! NOTE: the expresions for 'alpha_beta', 'alpha_1', and 'alpha_2' taken from Mook et al. [1974]
!       in order to provide values required for calculating the CaCO3 13C fractionation
      epsilon_beta(:) = +00.19 - 0373.0 / phys(:,1,ip_T)
      epsilon_1(:)    = +24.12 - 9866.0 / phys(:,1,ip_T)
      epsilon_2(:)    = +21.60 - 8999.0 / phys(:,1,ip_T)
      epsilon_SUM(:)  = (epsilon_1(:) * conc_HCO3(:) + epsilon_2(:) * conc_CO3(:)) / &
	& (conc_CO2(:) + conc_HCO3(:) + conc_CO3(:))
      d13C_atm_ocn(:) = -02.00
      d13C_ocn_atm(:) = -10.40
    END SELECT
! photosynthetic formation of POC
! NOTE: d13C POM fractionation is calculated seperately for diatom- and coccolithophore-based new production
    SELECT CASE (bioopt(bopt_d13Corg_alt))
    CASE ("1")
! scheme #1 : fixed fractionation
      d13C_DIC_Corg_dia(:) = par_bionp_13C_DIC_Corg_dia
      d13C_DIC_Corg_coc(:) = par_bionp_13C_DIC_Corg_coc
    CASE ("2")
! scheme #2 : [CO2(aq)]- and T-dependent fractionation (after Rau et al. [1996,1997])
! NOTE: a 2nd order approximation to '(Qs / ce) * (r/DT + 1/P)' 
!      (represented by the local variable 'd13C_DIC_Corg_Qsce') can be used
      d13C_DIC_Corg_Qsce = (d13C_DIC_Corg_Q2_x2 * phys(:,1,ip_T)**2 + &
	& d13C_DIC_Corg_Q2_x * phys(:,1,ip_T) + &
	& d13C_DIC_Corg_Q2_c) / conc_CO2(:)
      d13C_DIC_Corg_dia(:) = epsilon_SUM(:) - par_bionp_13C_DIC_Corg_ef_dia + &
	& (par_bionp_13C_DIC_Corg_ef_dia - d13C_DIC_Corg_ed) * d13C_DIC_Corg_Qsce
      d13C_DIC_Corg_coc(:) = epsilon_SUM(:) - par_bionp_13C_DIC_Corg_ef_coc + &
	& (par_bionp_13C_DIC_Corg_ef_coc - d13C_DIC_Corg_ed) * d13C_DIC_Corg_Qsce
    END SELECT
! 13C fractionation of (bulk) calcite and aragonite
! NOTE: T-dependent fractionation after Mook [1986],
!       otherwise [CO32-]-dependent fractionation adapted from Spero et al. [1997]
! NOTE: the Mook [1986] fractionation is approximated by assuming that 
!       the d13C of HCO3- is the same as that of total DIC 
!       (this is in order to be consistent with the calculation of benthic fractionation where 
!       the fractionation within the aqueous carbonate system is not explicitly calculated)
    d13C_DIC_cal(:) = 15.10 - 4232.0 / phys(:,1,ip_T)
    d13C_DIC_arg(:) = 16.90 - 4232.0 / phys(:,1,ip_T)


! *** ocean circulation mass exchange ***

! loop through all the mass flux vectors representing the ocean circulation,
! then seas through all the ocean tracers in turn
! NOTE: fluxes are in units of (yr-1)
! NOTE: mass transferred to one ocean box must also be removed from the originating box
! NOTE: care must be taken with the sign of the mass flow 
!       in terms of determining the cell from which mass is removed
    DO v = 1,par_n_wflux
      IF (oc(v)%ioc_wf >= 0) THEN
	ft_abio_del(:) = dt * ABS(oc(v)%ioc_wf) * oatracer%ioat_t(oc(v)%ioc_cs,oc(v)%ioc_ls,:)
	ft_abio(oc(v)%ioc_cs,oc(v)%ioc_ls,:) = ft_abio(oc(v)%ioc_cs,oc(v)%ioc_ls,:) - ft_abio_del(:)
	ft_abio(oc(v)%ioc_ce,oc(v)%ioc_le,:) = ft_abio(oc(v)%ioc_ce,oc(v)%ioc_le,:) + ft_abio_del(:)
      ELSE
	ft_abio_del(:) = dt * ABS(oc(v)%ioc_wf) * oatracer%ioat_t(oc(v)%ioc_ce,oc(v)%ioc_le,:)
	ft_abio(oc(v)%ioc_cs,oc(v)%ioc_ls,:) = ft_abio(oc(v)%ioc_cs,oc(v)%ioc_ls,:) + ft_abio_del(:)
	ft_abio(oc(v)%ioc_ce,oc(v)%ioc_le,:) = ft_abio(oc(v)%ioc_ce,oc(v)%ioc_le,:) - ft_abio_del(:)
      ENDIF
    END DO


! *** horizontal eddy diffusivity ***
! loop through all the mass flux vectors representing the ocean circulation, 
! where a flux vector is found that links two boxes with the same layer number identifier
! (i.e., nominally at the same depth), allow exchange of tracers by horzintal eddy diffusion
! NOTE: there is no easy way in which to assign a cross-sectioal area for the interface between 
!       two grid points and a certain depth in the water column,
!       particularly as the ocean configuration is general enough to allow mass flow between 
!       boxes which could not in reality be topographically connected, therefore the following is carried out
!       (a) determine which grid points are connected by a mass flux vector at ocean level 'l'
!       (b) determine the latitude and longitude of the boundaries of each of the two grid points
!       (c) test all combinations of N/S latitudes between 
!           the grid points to determine is they share a common interface
!       (d) assign the area value of the common interface
! NOTE: only N-S connectivity is assumed, E-W exchange has to be prescribed explicitly as a 
!       mass flux vector (from file)
! NOTE: if no N-S grid point connectivity is found then no horizontal eddy diffusivity is allowed
! NOTE: the distance between a grip point center latitude and its boundary is actually
!       '0.5 * (2 * PI * radius_Earth * Dlat / 360)', but is shortened to
!       'PI * radius_Earth * Dlat / 360'
! NOTE: with this simple scheme there will be additional horizontal diffusivity 
!       where ever more than one vector between two boxes at the same level in differennt columns occurs,
!       e.g., where there is an exchange flux prescribed (one vector each way), 
!       horizontal diffusivity will be twice as great, as as exchange will be calculated once 
!       for each flux vector
! NOTE: this algorithm could have been done so that from each column a search was made of whether
!       any other column adoined the N-S (or even E-W) directions, 
!       and from there calculated eddy diffusion between each ocean layer in this pair of column, 
!       but this would have required a long search of all possible combination 
!       (scaling as the 2nd power of the number of columns) 
!       with each combination requiring the full routine for adjacency testing being carried out
    DO v = 1,par_n_wflux
      IF (oc(v)%ioc_ls == oc(v)%ioc_le) THEN
	l = oc(v)%ioc_ls
	hdif_lat_N_s = phys(oc(v)%ioc_cs,l,ip_lat) + 0.5 * phys(oc(v)%ioc_cs,l,ip_dlat)
	hdif_lat_S_s = phys(oc(v)%ioc_cs,l,ip_lat) - 0.5 * phys(oc(v)%ioc_cs,l,ip_dlat)
	hdif_lat_N_e = phys(oc(v)%ioc_ce,l,ip_lat) + 0.5 * phys(oc(v)%ioc_ce,l,ip_dlat)
	hdif_lat_S_e = phys(oc(v)%ioc_ce,l,ip_lat) - 0.5 * phys(oc(v)%ioc_ce,l,ip_dlat)
	hdif_interA = 0.0
	IF (hdif_lat_N_s == hdif_lat_S_e) THEN
	  hdif_interA = MIN(phys(oc(v)%ioc_cs,l,ip_csA_N),phys(oc(v)%ioc_ce,l,ip_csA_S))
	ENDIF
	IF (hdif_lat_S_s == hdif_lat_N_e) THEN
	  hdif_interA = MIN(phys(oc(v)%ioc_cs,l,ip_csA_S),phys(oc(v)%ioc_ce,l,ip_csA_N))
	ENDIF
	ft_abio_del(:) = dt * par_misc_hdif * &
	  & hdif_interA * &
	  & conv_m3_kg * (oatracer%ioat_t(oc(v)%ioc_ce,l,:) - oatracer%ioat_t(oc(v)%ioc_cs,l,:)) / &
	  & ( &
	  & 0.5 * (const_PI * const_rEarth  + phys(oc(v)%ioc_ce,1,ip_dlat) / 360.0) + &
	  & 0.5 * (const_PI * const_rEarth  + phys(oc(v)%ioc_cs,1,ip_dlat) / 360.0) &
	  & )
	ft_abio(oc(v)%ioc_cs,l,:) = ft_abio(oc(v)%ioc_cs,l,:) + ft_abio_del(:)
	ft_abio(oc(v)%ioc_ce,l,:) = ft_abio(oc(v)%ioc_ce,l,:) - ft_abio_del(:)
      END IF
    END DO


! *** vertical eddy diffusivity ***

! loop through the ocean columns, and allow eddy diffusive transfer of tracers between adjacent layers
! (in the vertical direction) according to a prescribed vertical eddy diffusivity constant
! NOTE: the eddy diffusive fluxes are added to the abiotic ones calculated on the basis of ocean circulation
! NOTE: the flux is calculated as 
!       diffusivity * interface area * units conversion / (distance between the mid-depth of each layer)
! NOTE: values of vertical diffusivity are stored for the base of each ocean layer
!       (excluding the bottom layer)
! NOTE: fluxes are positive into layer 'l'
    DO c = 1,n_col
      DO l = 1,(n_lay - 1)
	ft_abio_del(:) = dt * phys(c,l,ip_vdif) * &
	  & phys(c,l,ip_A_bot) * &
	  & conv_m3_kg * (oatracer%ioat_t(c,l + 1,:) - oatracer%ioat_t(c,l,:)) / &
	  & ( &
	  & 0.5 * (phys(c,l,ip_D_bot) - phys(c,l,ip_D_top)) + &
	  & 0.5 * (phys(c,l + 1,ip_D_bot) - phys(c,l + 1,ip_D_top)) &
	  & )
	ft_abio(c,l,:)     = ft_abio(c,l,:)     + ft_abio_del(:)
	ft_abio(c,l + 1,:) = ft_abio(c,l + 1,:) - ft_abio_del(:)
      END DO
    END DO


! *** ocean convective adjustment scheme choice ***

! options:
! (01-02) schemes involving no internally-generated convective adjustments:
!       (01) no convective adjustment
!       (02) prescribed convective adjustments (taken from file)
! (03-09) schemes involving internally-generated convective adjustments:
!       (03) deep-water generation only
!       (04) 'mixed-layer' mixing only
!            NOTE: 'mixed-layer' depth taken from file
!       (05) general ocean convective adjustments only (pair-wise mixing algorithm)
!       (06) deep-water generation and 'mixed-layer' mixing
!       (07) deep-water generation and general ocean convective adjustments
!       (08) general ocean convective adjustments only (homogeneous column mixing algorithm)
!       (09) deep-water generation and prescribed convective adjustments


! *** ocean convective adjustement - from file ***

!!$! loop through the ocean columns,
!!$! then loop through the ocean layers - 
!!$! find series of continuous layers which all have convective adjustments between neighbours,
!!$! and thus form a convective column
!!$! make convective adjustments to the boxes in each such convective column with the ocean column
!!$! NOTE: convective adjustment of tracers is implemented by 
!!$!       (a) removing tracers from boxes undergoing convective adjustment
!!$!           in proportion to the mass fraction that is being convected
!!$!       (b) replace with the average tracer concentration across the entire convective column
!!$!           (calculated as a mean weighted to the mass fraction convected in each box) -
!!$!           the actual quantities (mol) added to each box
!!$!           are in proportion to the the mass fraction convected in each box
!!$! NOTE: ensure that convective adjustment is occuring before 
!!$!       attempting to calculate a value for 'ft_abio_del'
!!$!       (by checking that 'conv_l_bot' > 'conv_l_top')
!!$! NOTE: check when the layer number is advanced for 'conv_l' reaching 'n_lay',
!!$!       because there is not convective overturning associated with the bottom-most convective layer
!!$!       (as convective exchange is with the layer below)
!!$    IF (                                     &
!!$      & (miscopt(mopt_conv_adj) == "2") .OR. &
!!$      & (miscopt(mopt_conv_adj) == "9")      &
!!$      & ) THEN
!!$      DO c = 1,n_col
!!$	l = 1
!!$	conv_l_top          = l
!!$	conv_l_bot          = l
!!$	conv_adj(l)         = oconvadjfixed(c,l,tint)
!!$	conv_M_tot          = conv_adj(l) * phys(c,l,ip_M)
!!$	conv_ftracer_tot(:) = conv_adj(l) * phys(c,l,ip_M) * oatracer%ioat_t(c,l,:)
!!$	DO
!!$	  IF (oconvadjfixed(c,l,tint) > 0.0) THEN
!!$	    dsue%idf_v(c,l,tint) = conv_adj(l)
!!$	    conv_l_bot           = conv_l_bot + 1
!!$	    conv_adj(conv_l_bot) = oconvadjfixed(c,l,tint)
!!$	    conv_M_tot           = conv_M_tot + &
!!$	      & conv_adj(conv_l_bot) * phys(c,conv_l_bot,ip_M)
!!$	    conv_ftracer_tot(:)  = conv_ftracer_tot(:) + &
!!$	      & conv_adj(conv_l_bot) * phys(c,conv_l_bot,ip_M) * oatracer%ioat_t(c,conv_l_bot,:)
!!$	    l = l + 1
!!$	    IF (l == n_lay) THEN
!!$	      DO conv_l = conv_l_top,conv_l_bot
!!$		ft_abio_del(:) =                                                                      &
!!$		  & (                                                                                 &
!!$		  &   ((conv_adj(conv_l) * phys(c,conv_l,ip_M) / conv_M_tot) * conv_ftracer_tot(:)) - &
!!$		  &   ((conv_adj(conv_l) * phys(c,conv_l,ip_M)) * oatracer%ioat_t(c,conv_l,:))        &
!!$		  & )
!!$		ft_abio(c,conv_l,:) = ft_abio(c,conv_l,:) + ft_abio_del(:)
!!$	      END DO
!!$	      EXIT
!!$	    END IF
!!$	  ELSE
!!$	    IF (conv_l_bot > conv_l_top) THEN
!!$	      DO conv_l = conv_l_top,conv_l_bot
!!$		ft_abio_del(:) =                                                                      &
!!$		  & (                                                                                 &
!!$		  &   ((conv_adj(conv_l) * phys(c,conv_l,ip_M) / conv_M_tot) * conv_ftracer_tot(:)) - &
!!$		  &   ((conv_adj(conv_l) * phys(c,conv_l,ip_M)) * oatracer%ioat_t(c,conv_l,:))        &
!!$		  & )
!!$		ft_abio(c,conv_l,:) = ft_abio(c,conv_l,:) + ft_abio_del(:)
!!$	      END DO
!!$	    END IF
!!$	    l = l + 1
!!$	    IF (l == n_lay) THEN
!!$	      EXIT
!!$	    END IF
!!$	    conv_l_top          = l
!!$	    conv_l_bot          = l
!!$	    conv_adj(l)         = dt * phys(c,l,ip_conv)
!!$	    conv_M_tot          = conv_adj(l) * phys(c,l,ip_M)
!!$	    conv_ftracer_tot(:) = conv_adj(l) * phys(c,l,ip_M) * oatracer%ioat_t(c,l,:)
!!$	  END IF
!!$	END DO
!!$      END DO
!!$    END IF

! loop through the ocean columns,
! then loop through the ocean layers - 
! determine pairs of layers which have convective adjustments between them
! make convective adjustments to these box pairs
! NOTE: ensure that convective adjustment is occuring before 
!       attempting to calculate a value for 'ft_abio_del'
!       (by checking that 'conv_l_bot' > 'conv_l_top')
! NOTE: ocean convective overturning scheme is pair-wise
    IF (                                     &
      & (miscopt(mopt_conv_adj) == "2") .OR. &
      & (miscopt(mopt_conv_adj) == "9")      &
      & ) THEN
      DO c = 1,n_col
!       initialize local variables
	ft_abio_conv(:,:) = 0.0
	conv_ftracer_tot(:) = 0.0
!       convective overturning - down from ocean surface layer
	l = 1
	DO
	  conv_l_top = l
	  conv_l_bot = l + 1
	  IF (oconvadjfixed(c,l,tint) > 0.0) THEN
	    dsue%idf_v(c,l,tint) = oconvadjfixed(c,l,tint)
	    conv_M_top = oconvadjfixed(c,l,tint) * phys(c,conv_l_top,ip_M)
	    conv_M_bot = oconvadjfixed(c,l,tint) * phys(c,conv_l_bot,ip_M)
	    conv_M_tot =                                          &
	      & oconvadjfixed(c,l,tint) * phys(c,conv_l_top,ip_M) + &
	      & oconvadjfixed(c,l,tint) * phys(c,conv_l_bot,ip_M)
	    conv_ftracer_tot(:) =                                                                     &
	      & oconvadjfixed(c,l,tint) * (phys(c,conv_l_top,ip_M) * oatracer%ioat_t(c,conv_l_top,:)) + &
	      & ft_abio_conv(conv_l_top,:)  +                                                         &
	      & oconvadjfixed(c,l,tint) * (phys(c,conv_l_bot,ip_M) * oatracer%ioat_t(c,conv_l_bot,:)) + &
	      & ft_abio_conv(conv_l_bot,:)
	    ft_abio_del(:) =  &
	      & (                                                   & 
	      &   (conv_M_top / conv_M_tot) * conv_ftracer_tot(:) - &
	      &   (                                                 &
	      &     conv_M_top * oatracer%ioat_t(c,conv_l_top,:) +  &
	      &     ft_abio_conv(conv_l_top,:)                      &
	      &   )                                                 &
	      & )
	    ft_abio_conv(conv_l_top,:) = ft_abio_conv(conv_l_top,:) + ft_abio_del(:)
	    ft_abio_del(:) =                                        &
	      & (                                                   &
	      &   (conv_M_bot / conv_M_tot) * conv_ftracer_tot(:) - &
	      &   (                                                 & 
	      &     conv_M_bot * oatracer%ioat_t(c,conv_l_bot,:) +  &
	      &     ft_abio_conv(conv_l_bot,:)                      &
	      &   )                                                 &
	      & )
	    ft_abio_conv(conv_l_bot,:) = ft_abio_conv(conv_l_bot,:) + ft_abio_del(:)
	  END IF
	  l = l + 1
	  IF (l == n_lay) THEN
	    EXIT
	  END IF
	END DO
	ft_abio(c,:,:) = ft_abio(c,:,:) + ft_abio_conv(:,:)
      END DO
    ENDIF


! *** ocean convective adjustement - internally generated  ***

! if a pair of vertically adjacent ocean zones in the same column are convectively unstable, 
! i.e. the upper zone has a higher of the same density as the lower one, 
! then perform a convective adjustment between the two 
! by assigning each of them the mean (mass-wieghted) properties of the two;
! taking this new comboned as a new upper zone, test against the next zone (layer) down;
! repeat down through the water column
! NOTE: as the equation for state of sea water used is not a function of pressure,
!       it will be assumed for internally-generated convective adjustment that 
!       a column of sequentially unstable layers can simply be mixed after Wright and Stocker [1992],
!       although only one pass down through the water column will be allowed for computational efficiency

! (A) deep-water formation
!     if insolation is below the critical threshold, then
!     a zone extending from the base of the mixed layer down to the bottom of the ocean has its T and S
!     properties modified by replacing a volume equal to the surface layer with the surface layer,
!     the effective temperature of which is suppressed
!     NOTE: O2 concentrations are manipulated in a similar manner
!     NOTE: in addition to the insolation criteria,
!           carry out deep-water formation only when the deep-water formation factor is greater than zero
    IF (                                     &
      & (miscopt(mopt_conv_adj) == "3") .OR. &
      & (miscopt(mopt_conv_adj) == "6") .OR. &
      & (miscopt(mopt_conv_adj) == "7") .OR. &
      & (miscopt(mopt_conv_adj) == "9")      &
      & ) THEN
      DO c = 1,n_col
	conv_adj_dw(:) = par_misc_conv_adj_max * phys(c,:,ip_conv_dw)
	IF (                                                                              &
	  & (                                                                             &
	  &   (                                                                           &
	  &     SUM(seas_t_np_insol(((tint - 1) * par_n_t_np):(tint * par_n_t_np - 1),c)) &
	  &     /                                                                         &
	  &     REAL(par_n_t_np,KIND=fp_1)                                                &
	  &   )                                                                           &
	  &   <                                                                           &
	  &   par_misc_conv_Ithresh                                                       &
	  & )                                                                             &
	  & .AND.                                                                         &
	  &   ((par_misc_conv_adj_max * phys(c,1,ip_conv_dw)) > 0.0)                      &
	  & ) THEN
	  l = 1
	  conv_l_top  = l
	  conv_l_bot  = l
	  conv_M_top  = conv_adj_dw(l) * phys(c,l,ip_M)
	  conv_T_top  = conv_adj_dw(l) * phys(c,l,ip_M) * &
	    & MAX((oatracer%ioat_t(c,l,it_T) - par_misc_conv_dT),par_misc_conv_Tmin)
	  conv_S_top  = conv_adj_dw(l) * phys(c,l,ip_M) * &
	    & MIN((oatracer%ioat_t(c,l,it_S) + par_misc_conv_dS),par_misc_conv_Smax)
	  conv_O2_top = conv_adj_dw(l) * phys(c,l,ip_M) * &
	    & oatracer%ioat_t(c,l,it_O2)
	  conv_M_tot  = 0.0
	  conv_T_tot  = 0.0
	  conv_S_tot  = 0.0
	  conv_O2_tot = 0.0
	  DO
	    l = l + 1
	    conv_rho_u = calc_rho(conv_T_top / conv_M_top,conv_S_top / conv_M_top)
	    conv_rho_d = calc_rho(oatracer%ioat_t(c,l,it_T),oatracer%ioat_t(c,l,it_S))
	    IF ((conv_rho_u > conv_rho_d) .OR. (miscopt(mopt_conv_dw_maxD) == yes)) THEN
	      conv_l_bot = l
	      conv_M_tot = conv_M_tot + &
		& conv_adj_dw(l) * phys(c,conv_l_bot,ip_M)
	      conv_T_tot = conv_T_tot +                &
		& (                                    &
		&   conv_adj_dw(conv_l_bot) *    &
		&   phys(c,conv_l_bot,ip_M) *          &
		&   oatracer%ioat_t(c,conv_l_bot,it_T) &
		& )
	      conv_S_tot = conv_S_tot +                &
		& (                                    &
		&   conv_adj_dw(conv_l_bot) *    &
		&   phys(c,conv_l_bot,ip_M) *          &
		&   oatracer%ioat_t(c,conv_l_bot,it_S) &
		& )
	      conv_O2_tot = conv_O2_tot +               &
		& (                                     &
		&   conv_adj_dw(conv_l_bot) *     &
		&   phys(c,conv_l_bot,ip_M) *           &
		&   oatracer%ioat_t(c,conv_l_bot,it_O2) &
		& )
	      IF (l == n_lay) THEN
		DO conv_l = conv_l_top,conv_l_top
		  ft_abio_del(it_T) =                            &
		    & (                                          &                                           
		    &   (conv_M_top / conv_M_tot) * conv_T_tot - &
		    &   conv_T_top                               &
		    & )
		  ft_abio_del(it_S) =                            &
		    & (                                          &                                           
		    &   (conv_M_top / conv_M_tot) * conv_S_tot - &
		    &   conv_S_top                               &
		    & )
		  ft_abio_del(it_O2) =                            &
		    & (                                           &                                           
		    &   (conv_M_top / conv_M_tot) * conv_O2_tot - &
		    &   conv_O2_top                               &
		    & )
		  ft_abio(c,conv_l,it_T)  = ft_abio(c,conv_l,it_T)  + ft_abio_del(it_T)
		  ft_abio(c,conv_l,it_S)  = ft_abio(c,conv_l,it_S)  + ft_abio_del(it_S)
		  ft_abio(c,conv_l,it_O2) = ft_abio(c,conv_l,it_O2) + ft_abio_del(it_O2)
		END DO
		DO conv_l = conv_l_top + 1,conv_l_bot
		  ft_abio_del(it_T) =                               &
		    & (                                             &
		    &   (conv_adj_dw(conv_l) * phys(c,conv_l,ip_M) / conv_M_tot) * &
		    &   conv_T_top                                  &
		    &   -                                           &
		    &   (conv_M_top / conv_M_tot) *        &
		    &   conv_adj_dw(conv_l) * phys(c,conv_l,ip_M) * oatracer%ioat_t(c,conv_l,it_T) &
		    & )
		  ft_abio_del(it_S) =                               &
		    & (                                             &
		    &   (conv_adj_dw(conv_l) * phys(c,conv_l,ip_M) / conv_M_tot) *        &
		    &   conv_S_top                                  &
		    &    -                                          &
		    &   (conv_M_top / conv_M_tot) *        &
		    &   conv_adj_dw(conv_l) * phys(c,conv_l,ip_M) * oatracer%ioat_t(c,conv_l,it_S) &
		    & )
		  ft_abio_del(it_O2) =                               &
		    & (                                              &
		    &   (conv_adj_dw(conv_l) * phys(c,conv_l,ip_M) / conv_M_tot) *         &
		    &   conv_O2_top                                  &
		    &    -                                           &
		    &   (conv_M_top / conv_M_tot) *         &
		    &   conv_adj_dw(conv_l) * phys(c,conv_l,ip_M) * oatracer%ioat_t(c,conv_l,it_O2) &
		    & )
		  ft_abio(c,conv_l,it_T)  = ft_abio(c,conv_l,it_T)  + ft_abio_del(it_T)
		  ft_abio(c,conv_l,it_S)  = ft_abio(c,conv_l,it_S)  + ft_abio_del(it_S)
		  ft_abio(c,conv_l,it_O2) = ft_abio(c,conv_l,it_O2) + ft_abio_del(it_O2)
		END DO
		EXIT
	      ENDIF
	    ELSE
	      IF (conv_l_bot > 1) THEN
		DO conv_l = conv_l_top,conv_l_top
		  ft_abio_del(it_T) =                            &
		    & (                                          &                                           
		    &   (conv_M_top / conv_M_tot) * conv_T_tot - &
		    &   conv_T_top                               &
		    & )
		  ft_abio_del(it_S) =                            &
		    & (                                          &                                           
		    &   (conv_M_top / conv_M_tot) * conv_S_tot - &
		    &   conv_S_top                               &
		    & )
		  ft_abio_del(it_O2) =                            &
		    & (                                           &                                           
		    &   (conv_M_top / conv_M_tot) * conv_O2_tot - &
		    &   conv_O2_top                               &
		    & )
		  ft_abio(c,conv_l,it_T)  = ft_abio(c,conv_l,it_T)  + ft_abio_del(it_T)
		  ft_abio(c,conv_l,it_S)  = ft_abio(c,conv_l,it_S)  + ft_abio_del(it_S)
		  ft_abio(c,conv_l,it_O2) = ft_abio(c,conv_l,it_O2) + ft_abio_del(it_O2)
		END DO
		DO conv_l = conv_l_top + 1,conv_l_bot
		  ft_abio_del(it_T) =                               &
		    & (                                             &
		    &   (conv_adj_dw(conv_l) * phys(c,conv_l,ip_M) / conv_M_tot) *        &
		    &   conv_T_top                                  &
		    &   -                                           &
		    &   (conv_M_top / conv_M_tot) *        &
		    &   conv_adj_dw(conv_l) * phys(c,conv_l,ip_M) * oatracer%ioat_t(c,conv_l,it_T) &
		    & )
		  ft_abio_del(it_S) =                               &
		    & (                                             &
		    &   (conv_adj_dw(conv_l) * phys(c,conv_l,ip_M) / conv_M_tot) *        &
		    &   conv_S_top                                  &
		    &    -                                          &
		    &   (conv_M_top / conv_M_tot) *        &
		    &   conv_adj_dw(conv_l) * phys(c,conv_l,ip_M) * oatracer%ioat_t(c,conv_l,it_S) &
		    & )
		  ft_abio_del(it_O2) =                               &
		    & (                                              &
		    &   (conv_adj_dw(conv_l) * phys(c,conv_l,ip_M) / conv_M_tot) *         &
		    &   conv_O2_top                                  &
		    &    -                                           &
		    &   (conv_M_top / conv_M_tot) *         &
		    &   conv_adj_dw(conv_l) * phys(c,conv_l,ip_M) * oatracer%ioat_t(c,conv_l,it_O2) &
		    & )
		  ft_abio(c,conv_l,it_T)  = ft_abio(c,conv_l,it_T)  + ft_abio_del(it_T)
		  ft_abio(c,conv_l,it_S)  = ft_abio(c,conv_l,it_S)  + ft_abio_del(it_S)
		  ft_abio(c,conv_l,it_O2) = ft_abio(c,conv_l,it_O2) + ft_abio_del(it_O2)
		END DO
		EXIT
	      ELSE
		EXIT
	      ENDIF
	    END IF
	  END DO
	END IF
      END DO
    END IF

! (B) 'mixed-layer' mixing
!     if insolation is below the critical threshold, then
!     convective adjustment is made where convective instability occurs,
!     to a maximum depth of the prescribed depth of the 'mixed layer'
    IF (                                     &
      & (miscopt(mopt_conv_adj) == "4") .OR. &
      & (miscopt(mopt_conv_adj) == "6")      &
      & ) THEN
      DO c = 1,n_col
!       calculate ocean layers comprising mixed-layer depth
	DO l = 1,n_lay
	  conv_mixD_l_bot = n_lay
	  IF (phys(c,l,ip_D_bot) >= phys(c,l,ip_conv_mixD)) THEN
	    conv_mixD_l_bot = l
	    EXIT
	  END IF
	END DO
!       initialize local variables
	ft_abio_conv(:,:) = 0.0
	conv_ftracer_tot(:) = 0.0
!       test for insolation below threshold value
	IF (                                                                                                &
	  & (                                                                                               &
	  &   ( &
	  &     SUM(seas_t_np_insol(((tint - 1) * par_n_t_np):(tint * par_n_t_np - 1),c)) &
	  &     / &
	  &     REAL(par_n_t_np,KIND=fp_1) &
	  &   ) &
	  &   <                                                                                             &
	  &   par_misc_conv_Ithresh                                                                         &
	  & )                                                                                               &
	  & .AND.                                                                                           &
	  & (conv_mixD_l_bot > 1)                                                                           &
	  & ) THEN
!       convective overturning - down from ocean surface layer
	  l = 1
	  DO
	    conv_l_top = l
	    conv_l_bot = l + 1
	    conv_rho_u = calc_rho                                           &
	      & (                                                           &
	      &   (                                                         &
	      &     oatracer%ioat_t(c,conv_l_top,it_T) +                    &
	      &     ft_abio_conv(conv_l_top,it_T) / phys(c,conv_l_top,ip_M) &
	      &   )                                                         &
	      &   ,                                                         &
	      &   (                                                         &
	      &     oatracer%ioat_t(c,conv_l_top,it_S) +                    &
	      &     ft_abio_conv(conv_l_top,it_S) / phys(c,conv_l_top,ip_M) &
	      &   )                                                         &
	      & )
	    conv_rho_d = calc_rho                                           &
	      & (                                                           &
	      &   (                                                         &
	      &     oatracer%ioat_t(c,conv_l_bot,it_T) +                    &
	      &     ft_abio_conv(conv_l_bot,it_T) / phys(c,conv_l_bot,ip_M) &
	      &   )                                                         &
	      &   ,                                                         &
	      &   (                                                         &
	      &     oatracer%ioat_t(c,conv_l_bot,it_S) +                    &
	      &     ft_abio_conv(conv_l_bot,it_S) / phys(c,conv_l_bot,ip_M) &
	      &   )                                                         &
	      & )
	    IF (conv_rho_u > conv_rho_d) THEN
	      dsue%idf_v(c,l,tint) = dsue%idf_v(c,l,tint) + par_misc_conv_adj_max
	      conv_M_top = par_misc_conv_adj_max * phys(c,conv_l_top,ip_M)
	      conv_M_bot = par_misc_conv_adj_max * phys(c,conv_l_bot,ip_M)
	      conv_M_tot =                                          &
		& par_misc_conv_adj_max * phys(c,conv_l_top,ip_M) + &
		& par_misc_conv_adj_max * phys(c,conv_l_bot,ip_M)
	      conv_ftracer_tot(:) =                                                                     &
		& par_misc_conv_adj_max * (phys(c,conv_l_top,ip_M) * oatracer%ioat_t(c,conv_l_top,:)) + &
		& ft_abio_conv(conv_l_top,:)  +                                                         &
		& par_misc_conv_adj_max * (phys(c,conv_l_bot,ip_M) * oatracer%ioat_t(c,conv_l_bot,:)) + &
		& ft_abio_conv(conv_l_bot,:)
	      ft_abio_del(:) =  &
		& (                                                   & 
		&   (conv_M_top / conv_M_tot) * conv_ftracer_tot(:) - &
		&   (                                                 &
		&     conv_M_top * oatracer%ioat_t(c,conv_l_top,:) +  &
		&     ft_abio_conv(conv_l_top,:)                      &
		&   )                                                 &
		& )
	      ft_abio_conv(conv_l_top,:) = ft_abio_conv(conv_l_top,:) + ft_abio_del(:)
	      ft_abio_del(:) =                                        &
		& (                                                   &
		&   (conv_M_bot / conv_M_tot) * conv_ftracer_tot(:) - &
		&   (                                                 & 
		&     conv_M_bot * oatracer%ioat_t(c,conv_l_bot,:) +  &
		&     ft_abio_conv(conv_l_bot,:)                      &
		&   )                                                 &
		& )
	      ft_abio_conv(conv_l_bot,:) = ft_abio_conv(conv_l_bot,:) + ft_abio_del(:)
	    ELSE
	      EXIT
	    END IF
	    l = l + 1
	    IF (l == conv_mixD_l_bot) THEN
	      EXIT
	    END IF
	  END DO
	  ft_abio(c,:,:) = ft_abio(c,:,:) + ft_abio_conv(:,:)
	END IF
      END DO
    ENDIF

! (C) general ocean convective overturning - pair-wise
!     regardless of whether insolation is below the critical threshold or not,
!     starting at layer 1 , then
!     convective adjustment is made where convective instability occurs,
!     to a maximum depth of the ocean bottom
    IF (                                     &
      & (miscopt(mopt_conv_adj) == "5") .OR. &
      & (miscopt(mopt_conv_adj) == "7")      &
      & ) THEN
      DO c = 1,n_col
!       initialize local variables
	ft_abio_conv(:,:) = 0.0
	conv_ftracer_tot(:) = 0.0
!       convective overturning - down from ocean surface layer
	l = 1
	DO
	  conv_l_top = l
	  conv_l_bot = l + 1
	  conv_rho_u = calc_rho                                           &
	    & (                                                           &
	    &   (                                                         &
	    &     oatracer%ioat_t(c,conv_l_top,it_T) +                    &
	    &     ft_abio_conv(conv_l_top,it_T) / phys(c,conv_l_top,ip_M) &
	    &   )                                                         &
	    &   ,                                                         &
	    &   (                                                         &
	    &     oatracer%ioat_t(c,conv_l_top,it_S) +                    &
	    &     ft_abio_conv(conv_l_top,it_S) / phys(c,conv_l_top,ip_M) &
	    &   )                                                         &
	    & )
	  conv_rho_d = calc_rho                                           &
	    & (                                                           &
	    &   (                                                         &
	    &     oatracer%ioat_t(c,conv_l_bot,it_T) +                    &
	    &     ft_abio_conv(conv_l_bot,it_T) / phys(c,conv_l_bot,ip_M) &
	    &   )                                                         &
	    &   ,                                                         &
	    &   (                                                         &
	    &     oatracer%ioat_t(c,conv_l_bot,it_S) +                    &
	    &     ft_abio_conv(conv_l_bot,it_S) / phys(c,conv_l_bot,ip_M) &
	    &   )                                                         &
	    & )
	  IF (conv_rho_u > conv_rho_d) THEN
	    dsue%idf_v(c,l,tint) = par_misc_conv_adj_max
	    conv_M_top = par_misc_conv_adj_max * phys(c,conv_l_top,ip_M)
	    conv_M_bot = par_misc_conv_adj_max * phys(c,conv_l_bot,ip_M)
	    conv_M_tot =                                          &
	      & par_misc_conv_adj_max * phys(c,conv_l_top,ip_M) + &
	      & par_misc_conv_adj_max * phys(c,conv_l_bot,ip_M)
	    conv_ftracer_tot(:) =                                                                     &
	      & par_misc_conv_adj_max * (phys(c,conv_l_top,ip_M) * oatracer%ioat_t(c,conv_l_top,:)) + &
	      & ft_abio_conv(conv_l_top,:)  +                                                         &
	      & par_misc_conv_adj_max * (phys(c,conv_l_bot,ip_M) * oatracer%ioat_t(c,conv_l_bot,:)) + &
	      & ft_abio_conv(conv_l_bot,:)
	    ft_abio_del(:) =  &
	      & (                                                   & 
	      &   (conv_M_top / conv_M_tot) * conv_ftracer_tot(:) - &
	      &   (                                                 &
	      &     conv_M_top * oatracer%ioat_t(c,conv_l_top,:) +  &
	      &     ft_abio_conv(conv_l_top,:)                      &
	      &   )                                                 &
	      & )
	    ft_abio_conv(conv_l_top,:) = ft_abio_conv(conv_l_top,:) + ft_abio_del(:)
	    ft_abio_del(:) =                                        &
	      & (                                                   &
	      &   (conv_M_bot / conv_M_tot) * conv_ftracer_tot(:) - &
	      &   (                                                 & 
	      &     conv_M_bot * oatracer%ioat_t(c,conv_l_bot,:) +  &
	      &     ft_abio_conv(conv_l_bot,:)                      &
	      &   )                                                 &
	      & )
	    ft_abio_conv(conv_l_bot,:) = ft_abio_conv(conv_l_bot,:) + ft_abio_del(:)
	  END IF
	  l = l + 1
	  IF (l == n_lay) THEN
	    EXIT
	  END IF
	END DO
	ft_abio(c,:,:) = ft_abio(c,:,:) + ft_abio_conv(:,:)
      END DO
    ENDIF

! (D) general ocean convective overturning - homogonous column
!     regardless of whether insolation is below the critical threshold or not,
!     starting at layer 1 , then
!     convective adjustment is made where convective instability occurs,
!     to a maximum depth of the ocean bottom
    IF (                                &
      & (miscopt(mopt_conv_adj) == "8") &
      & ) THEN
      DO c = 1,n_col
!       convective overturning - down from ocean surface layer
	l = 1
	conv_l_top          = l
	conv_l_bot          = l
	conv_M_tot          = par_misc_conv_adj_max * phys(c,l,ip_M)
	conv_ftracer_tot(:) = par_misc_conv_adj_max * phys(c,l,ip_M) * oatracer%ioat_t(c,l,:)
	DO
	  conv_rho_u = calc_rho(conv_ftracer_tot(it_T) / conv_M_tot,conv_ftracer_tot(it_S) / conv_M_tot)
	  conv_rho_d = calc_rho(oatracer%ioat_t(c,l + 1,it_T),oatracer%ioat_t(c,l + 1,it_S))
	  IF (conv_rho_u > conv_rho_d) THEN
	    dsue%idf_v(c,l,tint) = par_misc_conv_adj_max
	    conv_l_bot = conv_l_bot + 1
	    conv_M_tot = conv_M_tot + par_misc_conv_adj_max * phys(c,conv_l_bot,ip_M)
	    conv_ftracer_tot(:) = conv_ftracer_tot(:) + &
	      & (                                       &
	      &   par_misc_conv_adj_max *               &
	      &   phys(c,conv_l_bot,ip_M) *             &
	      &   oatracer%ioat_t(c,conv_l_bot,:)       &
	      & )
	    l = l + 1
	    IF (l == n_lay) THEN
	      DO conv_l = conv_l_top,conv_l_bot
		ft_abio_del(:) =                                                                           &
		  & (                                                                                      &
		  &   ((par_misc_conv_adj_max * phys(c,conv_l,ip_M) / conv_M_tot) * conv_ftracer_tot(:)) - &
		  &   ((par_misc_conv_adj_max * phys(c,conv_l,ip_M)) * oatracer%ioat_t(c,conv_l,:))        &
		  & )
		ft_abio(c,conv_l,:) = ft_abio(c,conv_l,:) + ft_abio_del(:)
	      END DO
	      EXIT
	    END IF
	  ELSE
	    IF (conv_l_bot > conv_l_top) THEN
	      DO conv_l = conv_l_top,conv_l_bot
		ft_abio_del(:) =                                                                           &
		  & (                                                                                      &
		  &   ((par_misc_conv_adj_max * phys(c,conv_l,ip_M) / conv_M_tot) * conv_ftracer_tot(:)) - &
		  &   ((par_misc_conv_adj_max * phys(c,conv_l,ip_M)) * oatracer%ioat_t(c,conv_l,:))        &
		  & )
		ft_abio(c,conv_l,:) = ft_abio(c,conv_l,:) + ft_abio_del(:)
	      END DO
	    END IF
	    l = l + 1
	    IF (l == n_lay) THEN
	      EXIT
	    END IF
	    conv_l_top          = l
	    conv_l_bot          = l
	    conv_M_tot          = par_misc_conv_adj_max * phys(c,l,ip_M)
	    conv_ftracer_tot(:) = par_misc_conv_adj_max * phys(c,l,ip_M) * oatracer%ioat_t(c,l,:)
	  END IF
	END DO
      END DO
    ENDIF


! *** biological new production ***
    
! (1) calculate new production and derive isotopic fractionation
!     NOTE: if the "abiotic" configuration option is selected set new production values to zero
    IF (runopt(ropt_bio) == yes) THEN
      SELECT CASE (bioopt(bopt_np_alt))
      CASE ("3")
	DO c = 1,n_col
	  bionp(c,:) = dt * bionpfixed(c,:) * phys(c,1,ip_A_bot)
	END DO
      CASE default
	ft_SSbionp(:,:) = ft_abio(:,1,:) + dt * goatracer%ioat_t(:,1,:)
	CALL calc_bionp(                                                         &
	  & dt,tint,oatracer%ioat_t(:,1,:),ft_SSbionp(:,:),phys(:,1,:),SSPO4(:), &
	  & bionpfixed(:,:),dsue%idf_v(c,1,tint),                                &
	  & bionp(:,:)                                                           &
	  & )
	DO c = 1,n_col
	  bionp(c,ibionp_13C_Corg_dia) = (r13C_CO2(c,1) + calc_d13Cfrac(d13C_DIC_Corg_dia(c))) * & 
	    & bionp(c,ibionp_Corg_dia)
	  bionp(c,ibionp_13C_Corg_coc) = (r13C_CO2(c,1) + calc_d13Cfrac(d13C_DIC_Corg_coc(c))) * & 
	    & bionp(c,ibionp_Corg_coc)
	  bionp(c,ibionp_13C_Corg_tot) = bionp(c,ibionp_13C_Corg_dia) + bionp(c,ibionp_13C_Corg_coc)
	  bionp(c,ibionp_13C_cal_coc)  = (r13C_CO2(c,1) + calc_d13Cfrac(d13C_DIC_cal(c))) * &
	    & bionp(c,ibionp_cal_coc)
	  bionp(c,ibionp_13C_arg_coc)  = (r13C_CO2(c,1) + calc_d13Cfrac(d13C_DIC_arg(c))) * &
	    & bionp(c,ibionp_arg_coc)
	  bionp(c,ibionp_13C_DOC_dia)  = (r13C_CO2(c,1) + calc_d13Cfrac(d13C_DIC_Corg_dia(c))) * & 
	    & bionp(c,ibionp_DOC_dia)
	  bionp(c,ibionp_13C_DOC_coc)  = (r13C_CO2(c,1) + calc_d13Cfrac(d13C_DIC_Corg_coc(c))) * & 
	    & bionp(c,ibionp_DOC_coc)
	  bionp(c,ibionp_13C_DOC_tot)  = bionp(c,ibionp_13C_DOC_dia) + bionp(c,ibionp_13C_DOC_coc)
	END DO
      END SELECT
    ELSE
      bionp(:,:) = 0.0
    ENDIF
! (2) calculate changes in ocean surface tracers due to biological POM new production
    ft_bio(:,1,it_PO4)     = ft_bio(:,1,it_PO4) - &
      & (bionp(:,ibionp_Porg_tot))
    ft_bio(:,1,it_CO2)     = ft_bio(:,1,it_CO2) - &
      & (bionp(:,ibionp_Corg_tot) + bionp(:,ibionp_cal_coc) + bionp(:,ibionp_arg_coc))
    ft_bio(:,1,it_13C_CO2) = ft_bio(:,1,it_13C_CO2) - &
      & (bionp(:,ibionp_13C_Corg_tot) + bionp(:,ibionp_13C_cal_coc) + bionp(:,ibionp_13C_arg_coc))
    ft_bio(:,1,it_alk)     = ft_bio(:,1,it_alk) - &
      & (-bionp(:,ibionp_Norg_tot) + 2.0 * bionp(:,ibionp_cal_coc) + 2.0 * bionp(:,ibionp_arg_coc))
    ft_bio(:,1,it_Si)      = ft_bio(:,1,it_Si) - &
      & (bionp(:,ibionp_opal_dia))
    ft_bio(:,1,it_Fe)      = ft_bio(:,1,it_Fe) - &
      & (bionp(:,ibionp_Feorg_tot))
! (3) calculate oxygen generated as a result of biological POM new production
!     NOTE: assume that O2 generated as a result of photosynthesis goes into solution in the surface ocean
!     NOTE: assume that this O2 comes from pgotosynthstic conversion of CO2 and thus
!           is not subtracted from the atmosphere reservoir in mass balance calculations
    ft_bio(:,1,it_O2)      = ft_bio(:,1,it_O2) + &
      & red_Porg_O2 * bionp(:,ibionp_Porg_tot)
    ft_bio(:,1,it_18O_O2)  = 0.0
    fa_bio(ia_pO2)         = 0.0
    fa_bio(ia_18O_pO2)     = 0.0
! (4) calculate changes in ocean surface tracers due to biological DOM new production
    ft_bio(:,1,it_PO4)     = ft_bio(:,1,it_PO4) - &
      & red_Corg_Porg * bionp(:,ibionp_DOC_tot)
    ft_bio(:,1,it_CO2)     = ft_bio(:,1,it_CO2) - &
      & bionp(:,ibionp_DOC_tot)
    ft_bio(:,1,it_13C_CO2) = ft_bio(:,1,it_13C_CO2) - &
      & bionp(:,ibionp_13C_DOC_tot)
    ft_bio(:,1,it_alk)     = ft_bio(:,1,it_alk) - &
      & -red_Corg_Norg * bionp(:,ibionp_DOC_tot)
    ft_bio(:,1,it_Fe)      = ft_bio(:,1,it_Fe) - &
      & bionp(:,ibionp_DOFe_tot)
    ft_bio(:,1,it_DOC)     = ft_bio(:,1,it_DOC) + &
      & bionp(:,ibionp_DOC_tot)
    ft_bio(:,1,it_13C_DOC) = ft_bio(:,1,it_13C_DOC) + &
      & bionp(:,ibionp_13C_DOC_tot)
    ft_bio(:,1,it_DOFe)    = ft_bio(:,1,it_DOFe) + &
      & bionp(:,ibionp_DOFe_tot)
! (5) calculate oxygen generated as a result of biological DOM new production
    ft_bio(:,1,it_O2)      = ft_bio(:,1,it_O2) + &
      & red_Corg_O2 * bionp(:,ibionp_DOC_tot)
    ft_bio(:,1,it_18O_O2)  = 0.0
    fa_bio(ia_pO2)         = 0.0
    fa_bio(ia_18O_pO2)     = 0.0


! *** ocean-atmosphere CO2 exchange ***

! *** calculate CO2 and 13CO2 exchange flux between atmosphere and each surface box ***
! surface ocean-atmosphere CO2 gaseous mass exchange - surface ocean changes
! calculate CO2 exchange fluxes (from atmosphere to surface and surface to atmosphere boxes),
! and net CO2 flux from atmosphere to surface ocean (i.e., pCO2 - fCO2) (mol)
    IF (miscopt(mopt_uniform_kgastrans) == no) THEN
! calculate Schmidt number for CO2 gas transfer
      pvgas_Sc_CO2(:) = &
	& pvgas_Sc_CO2_A                                     - &
	& pvgas_Sc_CO2_B * (phys(:,1,ip_T) - const_zeroC)    + &
	& pvgas_Sc_CO2_C * (phys(:,1,ip_T) - const_zeroC)**2 - &
	& pvgas_Sc_CO2_D * (phys(:,1,ip_T) - const_zeroC)**3
! calculate CO2 gas transfer velocity (piston velocity)
! NOTE: from Wanninkhof [1992] equation 8
! NOTE: convert from units of (cm hr-1) to (m yr-1)
      pvgas_k_CO2(:) = &
	& ( &
	& 2.5 * &
	& ( &
	& pvgas_k_CO2_a                                     + &
	& pvgas_k_CO2_b * (phys(:,1,ip_T) - const_zeroC)    + &
	& pvgas_k_CO2_c * (phys(:,1,ip_T) - const_zeroC)**2   &
	& ) + &
	& 0.38 * phys(:,1,ip_swind)**2 &
	& ) * &
	& (pvgas_Sc_CO2(:) / 660.0)**(-0.5)
      pvgas_k_CO2(:) = conv_cm_m * conv_yr_hr * pvgas_k_CO2(:)
! calculate omni-directional CO2 gas transfer
      fCO2_atm_ocn(:) = (SSdis(:,idis_QCO2) * oatracer%ioat_a(ia_pCO2)) * &
	& (dt * conv_m3_kg * pvgas_k_CO2(:) * (1.0 - seas_t_int_seaice(tint,:)) * phys(:,1,ip_A_top))
      fCO2_ocn_atm(:) = (SSdis(:,idis_QCO2) * fug_CO2(:)) * &
	& (dt * conv_m3_kg * pvgas_k_CO2(:) * (1.0 - seas_t_int_seaice(tint,:)) * phys(:,1,ip_A_top))
    ELSE
! calculate omni-directional CO2 gas transfer
      fCO2_atm_ocn(:) = (conv_mol_umol * oatracer%ioat_a(ia_pCO2)) * &
	& (dt * par_misc_kgastrans_coeff_CO2 * (1.0 - seas_t_int_seaice(tint,:)) * phys(:,1,ip_A_top))
      fCO2_ocn_atm(:) = (conv_mol_umol * fug_CO2(:)) * &
	& (dt * par_misc_kgastrans_coeff_CO2 * (1.0 - seas_t_int_seaice(tint,:)) * phys(:,1,ip_A_top))
    ENDIF
! calculate omni-directional 13CO2 gas transfer
    DO c = 1,n_col
      f13CO2_atm_ocn(c) = (r13C_pCO2     + calc_d13Cfrac(d13C_atm_ocn(c))) * fCO2_atm_ocn(c)
      f13CO2_ocn_atm(c) = (r13C_CO2(c,1) + calc_d13Cfrac(d13C_ocn_atm(c))) * fCO2_ocn_atm(c)
    END DO
! calculate net atmosphere -> surface ocean CO2 and 13CO2 flux
    ft_abio(:,1,it_CO2)     = ft_abio(:,1,it_CO2)     + &
      & (fCO2_atm_ocn(:)   - fCO2_ocn_atm(:))
    ft_abio(:,1,it_13C_CO2) = ft_abio(:,1,it_13C_CO2) + &
      & (f13CO2_atm_ocn(:) - f13CO2_ocn_atm(:))
! calculate net surface ocean -> atmosphere CO2 and 13CO2 mass flux
!     take the change in atmospheric tracers as the sum of the net flux out of each of the surface ocean cells
!     NOTE: this assumes that the atmosphere is well mixed
    fa_abio(ia_pCO2)     = SUM((fCO2_ocn_atm(:)   - fCO2_atm_ocn(:)))
    fa_abio(ia_13C_pCO2) = SUM((f13CO2_ocn_atm(:) - f13CO2_atm_ocn(:)))
    

! *** remineralization in water column, scavenging, and sedimentation - integral sediment sub-system ***

! loop through ocean water columns
! find the uppermost sediment layer which lies at least partly below base of mixed (surface) ocean layer
! loop through fractions of water column horizontal area (defined by sediment layer depths (from top down))
! NOTE: the remineralization depth is the depth (in m) 
!       below which remineralization in the water column starts to occur
! NOTE: the remineralization area fraction is the fractional area of each sediment layer
!       w.r.t. the area of the bottom of the mixed layer
! NOTE: before looping through sediment depths, determine the first sediment depth interval
!       which falls at least partly below the base of the surface (mixed) layer - 
!       use this as the loop starting point to avoid wasting time on sediment depths 
!       wholly within the mixed layer (which recieve no new production)
! NOTE: the bottom-most sediment layer included in the loop ('m_npbot')
!       is always the deepest sediment layer ('n_sed_depth'),
!       although this might mean that the layer falls completely below the bottom depth of the ocean,
!       this is acocunted for, and is connected in a virtual sense with this bottom ocean layer

    DO c = 1,n_col

      m_nptop = 1
      DO m = 1,n_sed_depth
	IF (sedpar(c,m,ised_par_D_bot) > phys(c,1,ip_D_bot)) THEN
	  m_nptop = m
	  EXIT
	ENDIF
      END DO
      m_npbot = n_sed_depth

      DO m = m_nptop,m_npbot

! (1) zero temporary array
	fremin_bionp(:,:) = 0.0
! (2) calculate value of the sinking new production flux above each sedimentary layer
!     NOTE: this depends on the area fraction occuped by the sedimentary layer w.r.t. 
!           the base of the mixed layer
	sink_bionp(:) = bionp(c,:) * sedpar(c,m,ised_par_Afrac_np)
! (3) calculate particulate matter remineralization/dissolution in the water column and sedimentation flux
!     NOTE: for sake of computational efficiency, a remineralization function value is used 
!           in proportion to the pre-calculated value for the whole ocean layer thickness
!     NOTE: when the first ocean layer is reached whose bottom is below the middle of the sedimentary layer,
!           remineralize from the top of that ocean layer down to the the mid depth of the sediment layer,
!           then sediment the remaining new production flux
!     NOTE: as default there is no remineralization (dissolution) of calcite in the water column
!     NOTE: to ensure that no particulate matter is lost from the system
!           the flux of PM to the sediments is calculated at the end of the ocean layer loop
!           as the difference between the new production flux and the remineralized flux
!     NOTE: Corg, Norg, and Feorg, components are explicitly represented
!     NOTE: assume that the ocean is undersaturated w.r.t. opal at all depths
	DO l = 2,n_lay
	  IF (phys(c,l,ip_D_bot) >= sedpar(c,m,ised_par_D_mid)) THEN
	    delta_reminf_Corg = phys(c,l - 1,ip_reminf_Corg) - sedpar(c,m,ised_par_reminf_Corg)
	    delta_reminf_opal = phys(c,l - 1,ip_reminf_opal) - sedpar(c,m,ised_par_reminf_opal)
	    delta_reminf_cal  = phys(c,l - 1,ip_reminf_cal)  - sedpar(c,m,ised_par_reminf_cal)
	    delta_reminf_arg  = phys(c,l - 1,ip_reminf_arg)  - sedpar(c,m,ised_par_reminf_arg)
	  ELSE
	    delta_reminf_Corg = phys(c,l - 1,ip_reminf_Corg) - phys(c,l,ip_reminf_Corg)
	    delta_reminf_opal = phys(c,l - 1,ip_reminf_opal) - phys(c,l,ip_reminf_opal)
	    delta_reminf_cal  = phys(c,l - 1,ip_reminf_cal)  - phys(c,l,ip_reminf_cal)
	    delta_reminf_arg  = phys(c,l - 1,ip_reminf_arg)  - phys(c,l,ip_reminf_arg)
	  ENDIF
	  IF (delta_reminf_Corg < 0.0) delta_reminf_Corg = 0.0
	  IF (delta_reminf_opal < 0.0) delta_reminf_opal = 0.0
	  IF (delta_reminf_cal  < 0.0) delta_reminf_cal  = 0.0
	  IF (delta_reminf_arg  < 0.0) delta_reminf_arg  = 0.0
	  fremin_bionp(l,ibionp_Corg_tot)     = sink_bionp(ibionp_Corg_tot)     * delta_reminf_Corg
	  fremin_bionp(l,ibionp_Porg_tot)     = sink_bionp(ibionp_Porg_tot)     * delta_reminf_Corg
	  fremin_bionp(l,ibionp_Norg_tot)     = sink_bionp(ibionp_Norg_tot)     * delta_reminf_Corg
	  fremin_bionp(l,ibionp_Feorg_tot)    = sink_bionp(ibionp_Feorg_tot)    * delta_reminf_Corg
	  fremin_bionp(l,ibionp_13C_Corg_tot) = sink_bionp(ibionp_13C_Corg_tot) * delta_reminf_Corg
	  fremin_bionp(l,ibionp_opal_dia)     = sink_bionp(ibionp_opal_dia)     * delta_reminf_opal
	  fremin_bionp(l,ibionp_cal_coc)      = sink_bionp(ibionp_cal_coc)      * delta_reminf_cal
	  fremin_bionp(l,ibionp_13C_cal_coc)  = sink_bionp(ibionp_13C_cal_coc)  * delta_reminf_cal
	  fremin_bionp(l,ibionp_arg_coc)      = sink_bionp(ibionp_arg_coc)      * delta_reminf_arg
	  fremin_bionp(l,ibionp_13C_arg_coc)  = sink_bionp(ibionp_13C_arg_coc)  * delta_reminf_arg
	  IF (phys(c,l,ip_D_bot) >= sedpar(c,m,ised_par_D_mid)) THEN
	    EXIT
	  ENDIF
	END DO
	dsue%idf_s(c,m,ised_fnp_Corg)     = sink_bionp(ibionp_Corg_tot)     - &
	  & SUM(fremin_bionp(:,ibionp_Corg_tot))
	dsue%idf_s(c,m,ised_fnp_Porg)     = sink_bionp(ibionp_Porg_tot)     - &
	  & SUM(fremin_bionp(:,ibionp_Porg_tot))
	dsue%idf_s(c,m,ised_fnp_Norg)     = sink_bionp(ibionp_Norg_tot)     - &
	  & SUM(fremin_bionp(:,ibionp_Norg_tot))
	dsue%idf_s(c,m,ised_fnp_Feorg)    = sink_bionp(ibionp_Feorg_tot)    - &
	  & SUM(fremin_bionp(:,ibionp_Feorg_tot))
	dsue%idf_s(c,m,ised_fnp_13C_Corg) = sink_bionp(ibionp_13C_Corg_tot) - &
	  & SUM(fremin_bionp(:,ibionp_13C_Corg_tot))
	dsue%idf_s(c,m,ised_fnp_opal)     = sink_bionp(ibionp_opal_dia)     - &
	  & SUM(fremin_bionp(:,ibionp_opal_dia))
	dsue%idf_s(c,m,ised_fnp_cal)      = sink_bionp(ibionp_cal_coc)      - &
	  & SUM(fremin_bionp(:,ibionp_cal_coc))
	dsue%idf_s(c,m,ised_fnp_13C_cal)  = sink_bionp(ibionp_13C_cal_coc)  - &
	  & SUM(fremin_bionp(:,ibionp_13C_cal_coc))
	dsue%idf_s(c,m,ised_fnp_arg)      = sink_bionp(ibionp_arg_coc)      - &
	  & SUM(fremin_bionp(:,ibionp_arg_coc))
	dsue%idf_s(c,m,ised_fnp_13C_arg)  = sink_bionp(ibionp_13C_arg_coc)  - &
	  & SUM(fremin_bionp(:,ibionp_13C_arg_coc))
! (4) calculate dissolved iron scavenging in the water column and additional sedimentation flux
!     NOTE: ensure that the particle flux used on which the flux-dependent scavenging rate is based
!           is not zero
!           => add a small flux offset to achieve this
!     NOTE: flux-dependent Fe scavenging rate rate is already in the correct units (mol m-2)
!     NOTE: calculate Fe scavenged leaving ocean layer #l 
!           (whether through the base of the layer or to the underlying sediments) 
!           flux as the fraction of the Fe scavenged flux entering that layer 
!           PLUS what is scavenged from within that layer
!     NOTE: make the seperate scavenging rate parts of the non-flux dependent scavenging rate 
!           proportional to the requested flux-depednent scavenging rate factors
!           i.e., in proportion to the fraction of 'par_k_scav_Fe_fPMlaw_tot'
!     NOTE: estimate scavenged Fe flux added to particulate flux at the bottom of each layer,
!           after the remineralization in the layer above has been calculated;
!           the flux into the top of the layer is immaterial, as some of the particulate flux 
!           will scavenge iron and subsequently be remineralized again, all in the same box
!     NOTE: for simplicity, deduct scavenged Fe from the calculated 'remin_bionp' flux;
!           this may result in this quantity becoming -ve. (this shouldn't matter)
!     NOTE: the scavenged Fe flux is updated for each sediment layer by;
!           (a) removing a fraction of the scavenged Fe flux entering the layer
!               (proportional to the fraction of POM which is remineralized within that layer) 
!           (b) adding Fe in proportion to the excess of [Fe] over and above the 'apparent solubility' of Fe
!     NOTE: the difference between the scavenged flux entering and leaving the layer is 
!           equal in magnitude and opposite in sign to the change in the iron inventory of the 
!           part of that particular ocean layer above the sedimentary layer currently considered
!     NOTE: the volume of water in the ocean layer lying immediately above the sediment layer is approximated
!     NOTE: ONLY scavenge Fe if the concentration of total dissolved Fe is greater than 
!           the 'apparent solubility' of Fe (i.e., 'par_apparsol_Fe' set in the parameter file)
!     NOTE: if the bottom depth of an ocean layer is less than the sediment subsystem mid depth AND
!           the bottom-most ocean layer has been reached,
!           sediment the entirety of the remaining scavenged Fe flux
!     NOTE: scavenging by refractory material is allowed in the surface layer (only)
!           in order to implicitly account for some of the effects of
!           reduced apparent Fe solubility with loading and self-scavenging
!     Fe scavenging from surface ocean layer
	l = 1
	len_scav = phys(c,l,ip_D_bot) - phys(c,l,ip_D_top)
	SELECT CASE (bioopt(bopt_scav_Fe_alt))
	CASE ("1")
!     scheme #1 : no Fe scavenging
	  k_scav_Fe_Corg   = 0.0
	  k_scav_Fe_opal   = 0.0
	  k_scav_Fe_cal    = 0.0
	  k_scav_Fe_arg    = 0.0
	  k_scav_Fe_refrac = 0.0
	CASE ("2")
!     scheme #2 : fixed scavenging rate Fe scavenging
	  k_scav_Fe_Corg    = (par_k_scav_Fe_fPMlaw_Corg    / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	  k_scav_Fe_opal    = (par_k_scav_Fe_fPMlaw_opal    / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	  k_scav_Fe_cal     = (par_k_scav_Fe_fPMlaw_cal     / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	  k_scav_Fe_arg     = (par_k_scav_Fe_fPMlaw_arg     / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	  k_scav_Fe_refrac  = 0.0
	CASE ("3")
!     scheme #3 : particulate matter flux dependent scavenging rate Fe scavenging
	  k_scav_Fe_Corg   = par_k_scav_Fe_fPMlaw_Corg                       * &
	    & phys(c,l,ip_reminf_Corg) * (sink_bionp(ibionp_Corg_tot) + 1.0) / &
	    & sedpar(c,m,ised_par_A)
	  k_scav_Fe_opal   = par_k_scav_Fe_fPMlaw_opal                       * &
	    & phys(c,l,ip_reminf_opal) * (sink_bionp(ibionp_opal_dia) + 1.0) / &
	    & sedpar(c,m,ised_par_A)
	  k_scav_Fe_cal    = par_k_scav_Fe_fPMlaw_cal                        * &
	    & phys(c,l,ip_reminf_cal)  * (sink_bionp(ibionp_cal_coc) + 1.0)  / &
	    & sedpar(c,m,ised_par_A)
	  k_scav_Fe_arg    = par_k_scav_Fe_fPMlaw_arg                        * &
	    & phys(c,l,ip_reminf_arg)  * (sink_bionp(ibionp_arg_coc) + 1.0)  / &
	    & sedpar(c,m,ised_par_A)
	  k_scav_Fe_refrac = par_k_scav_Fe_fPMlaw_refrac                     * &
	    & fdust(c)
	END SELECT
	IF (oatracer%ioat_t(c,l,it_Fe) > par_apparsol_Fe) THEN
	  Fe_excess = (oatracer%ioat_t(c,l,it_Fe) - par_apparsol_Fe) * &
	    & conv_m3_kg * len_scav * sedpar(c,m,ised_par_Afrac_np) * phys(c,l,ip_A_bot)
	ELSE
	  Fe_excess = 0.0
	ENDIF
	fscav_Fe_Corg(l)   = 0.0 + &
	  & dt * k_scav_Fe_Corg   * Fe_excess
	fscav_Fe_opal(l)   = 0.0 + &
	  & dt * k_scav_Fe_opal   * Fe_excess
	fscav_Fe_cal(l)    = 0.0 + &
	  & dt * k_scav_Fe_cal    * Fe_excess
	fscav_Fe_arg(l)    = 0.0 + &
	  & dt * k_scav_Fe_arg    * Fe_excess
	fscav_Fe_refrac(l) = 0.0 + &
	  & dt * k_scav_Fe_refrac * Fe_excess
	fremin_bionp(l,ibionp_Feorg_tot) = fremin_bionp(l,ibionp_Feorg_tot) - &
	  & (fscav_Fe_Corg(l)   - 0.0)                                      - &
	  & (fscav_Fe_opal(l)   - 0.0)                                      - &
	  & (fscav_Fe_cal(l)    - 0.0)                                      - &
	  & (fscav_Fe_arg(l)    - 0.0)                                      - &
	  & (fscav_Fe_refrac(l) - 0.0)
!     Fe scavenging from remaining ocean layers
	DO l = 2,n_lay
	  IF (phys(c,l,ip_D_bot) >= sedpar(c,m,ised_par_D_mid)) THEN
	    IF (l == 2) THEN
	      dsue%idf_s(c,m,ised_fnp_Feorg) = dsue%idf_s(c,m,ised_fnp_Feorg) + &
		& fscav_Fe_Corg(l - 1)                                        + &
		& fscav_Fe_opal(l - 1)                                        + &
		& fscav_Fe_cal(l - 1)                                         + &
		& fscav_Fe_arg(l - 1)                                         + &
		& fscav_Fe_refrac(l - 1)
	      EXIT
	    END IF
	    IF (phys(c,1,ip_remin_topD) > phys(c,l,ip_D_top)) THEN
	      len_scav = sedpar(c,m,ised_par_D_mid) - phys(c,1,ip_remin_topD)
	    ELSE
	      len_scav = sedpar(c,m,ised_par_D_mid) - phys(c,l,ip_D_top)
	    END IF
	    IF (len_scav < 0.0) len_scav = 0.0
	    ratio_reminf_Corg = sedpar(c,m,ised_par_reminf_Corg) / (phys(c,l - 1,ip_reminf_Corg) + 0.000001)
	    IF (ratio_reminf_Corg > 1.0) ratio_reminf_Corg = 1.0
	    ratio_reminf_opal = sedpar(c,m,ised_par_reminf_opal) / (phys(c,l - 1,ip_reminf_opal) + 0.000001)
	    IF (ratio_reminf_opal > 1.0) ratio_reminf_opal = 1.0
	    ratio_reminf_cal  = sedpar(c,m,ised_par_reminf_cal)  / (phys(c,l - 1,ip_reminf_cal)  + 0.000001)
	    IF (ratio_reminf_cal  > 1.0) ratio_reminf_cal  = 1.0
	    ratio_reminf_arg  = sedpar(c,m,ised_par_reminf_arg)  / (phys(c,l - 1,ip_reminf_arg)  + 0.000001)
	    IF (ratio_reminf_arg  > 1.0) ratio_reminf_arg  = 1.0
	    ratio_reminf_refrac = 1.0
	    SELECT CASE (bioopt(bopt_scav_Fe_alt))
	    CASE ("1")
!     scheme #1 : no Fe scavenging
	      k_scav_Fe_Corg   = 0.0
	      k_scav_Fe_opal   = 0.0
	      k_scav_Fe_cal    = 0.0
	      k_scav_Fe_arg    = 0.0
	      k_scav_Fe_refrac = 0.0
	    CASE ("2")
!     scheme #2 : fixed scavenging rate Fe scavenging
	      k_scav_Fe_Corg   = (par_k_scav_Fe_fPMlaw_Corg / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_opal   = (par_k_scav_Fe_fPMlaw_opal / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_cal    = (par_k_scav_Fe_fPMlaw_cal  / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_arg    = (par_k_scav_Fe_fPMlaw_arg  / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_refrac = 0.0
	    CASE ("3")
!     scheme #3 : particulate matter flux dependent scavenging rate Fe scavenging
	      k_scav_Fe_Corg   = par_k_scav_Fe_fPMlaw_Corg                               * &
		& sedpar(c,m,ised_par_reminf_Corg) * (sink_bionp(ibionp_Corg_tot) + 1.0) / &
		& sedpar(c,m,ised_par_A)
	      k_scav_Fe_opal   = par_k_scav_Fe_fPMlaw_opal                               * &
		& sedpar(c,m,ised_par_reminf_opal) * (sink_bionp(ibionp_opal_dia) + 1.0) / &
		& sedpar(c,m,ised_par_A)
	      k_scav_Fe_cal    = par_k_scav_Fe_fPMlaw_cal                                * &
		& sedpar(c,m,ised_par_reminf_cal)  * (sink_bionp(ibionp_cal_coc) + 1.0)  / &
		& sedpar(c,m,ised_par_A)
	      k_scav_Fe_arg    = par_k_scav_Fe_fPMlaw_arg                                * &
		& sedpar(c,m,ised_par_reminf_arg)  * (sink_bionp(ibionp_arg_coc) + 1.0)  / &
		& sedpar(c,m,ised_par_A)
	      k_scav_Fe_refrac = 0.0
	    END SELECT
	  ELSE
	    IF (phys(c,1,ip_remin_topD) > phys(c,l,ip_D_top)) THEN
	      len_scav = phys(c,l,ip_D_bot) - phys(c,1,ip_remin_topD)
	    ELSE
	      len_scav = phys(c,l,ip_D_bot) - phys(c,l,ip_D_top)
	    END IF
	    IF (len_scav < 0.0) len_scav = 0.0
	    len_scav = phys(c,l,ip_D_bot) - phys(c,l,ip_D_top)
	    ratio_reminf_Corg = phys(c,l,ip_reminf_Corg) / (phys(c,l - 1,ip_reminf_Corg) + 0.000001)
	    IF (ratio_reminf_Corg > 1.0) ratio_reminf_Corg = 1.0
	    ratio_reminf_opal = phys(c,l,ip_reminf_opal) / (phys(c,l - 1,ip_reminf_opal) + 0.000001)
	    IF (ratio_reminf_opal > 1.0) ratio_reminf_opal = 1.0
	    ratio_reminf_cal  = phys(c,l,ip_reminf_cal)  / (phys(c,l - 1,ip_reminf_cal)  + 0.000001)
	    IF (ratio_reminf_cal  > 1.0) ratio_reminf_cal  = 1.0
	    ratio_reminf_arg  = phys(c,l,ip_reminf_arg)  / (phys(c,l - 1,ip_reminf_arg)  + 0.000001)
	    IF (ratio_reminf_arg  > 1.0) ratio_reminf_arg  = 1.0
	    ratio_reminf_refrac = 1.0
	    SELECT CASE (bioopt(bopt_scav_Fe_alt))
	    CASE ("1")
!     scheme #1 : no Fe scavenging
	      k_scav_Fe_Corg   = 0.0
	      k_scav_Fe_opal   = 0.0
	      k_scav_Fe_cal    = 0.0
	      k_scav_Fe_arg    = 0.0
	      k_scav_Fe_refrac = 0.0
	    CASE ("2")
!     scheme #2 : fixed scavenging rate Fe scavenging
	      k_scav_Fe_Corg   = (par_k_scav_Fe_fPMlaw_Corg / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_opal   = (par_k_scav_Fe_fPMlaw_opal / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_cal    = (par_k_scav_Fe_fPMlaw_cal  / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_arg    = (par_k_scav_Fe_fPMlaw_arg  / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_refrac = 0.0
	    CASE ("3")
!     scheme #3 : particulate matter flux dependent scavenging rate Fe scavenging
	      k_scav_Fe_Corg   = par_k_scav_Fe_fPMlaw_Corg                       * &
		& phys(c,l,ip_reminf_Corg) * (sink_bionp(ibionp_Corg_tot) + 1.0) / &
		& sedpar(c,m,ised_par_A)
	      k_scav_Fe_opal   = par_k_scav_Fe_fPMlaw_opal                       * &
		& phys(c,l,ip_reminf_opal) * (sink_bionp(ibionp_opal_dia) + 1.0) / &
		& sedpar(c,m,ised_par_A)
	      k_scav_Fe_cal    = par_k_scav_Fe_fPMlaw_cal                        * &
		& phys(c,l,ip_reminf_cal)  * (sink_bionp(ibionp_cal_coc) + 1.0)  / &
		& sedpar(c,m,ised_par_A)
	      k_scav_Fe_arg    = par_k_scav_Fe_fPMlaw_arg                        * &
		& phys(c,l,ip_reminf_arg)  * (sink_bionp(ibionp_arg_coc) + 1.0)  / &
		& sedpar(c,m,ised_par_A)
	      k_scav_Fe_refrac = 0.0
	    END SELECT
	  ENDIF
	  IF (oatracer%ioat_t(c,l,it_Fe) > par_apparsol_Fe) THEN
	    Fe_excess = (oatracer%ioat_t(c,l,it_Fe) - par_apparsol_Fe) * &
	      & conv_m3_kg * len_scav * sedpar(c,m,ised_par_Afrac_np) * phys(c,1,ip_A_bot)
	  ELSE
	    Fe_excess = 0.0
	  ENDIF
	  fscav_Fe_Corg(l)   = ratio_reminf_Corg   * fscav_Fe_Corg(l - 1)    + &
	    & dt * k_scav_Fe_Corg * Fe_excess
	  fscav_Fe_opal(l)   = ratio_reminf_opal   * fscav_Fe_opal(l - 1)    + &
	    & dt * k_scav_Fe_opal * Fe_excess
	  fscav_Fe_cal(l)    = ratio_reminf_cal    * fscav_Fe_cal(l - 1)     + &
	    & dt * k_scav_Fe_cal  * Fe_excess
	  fscav_Fe_arg(l)    = ratio_reminf_arg    * fscav_Fe_arg(l - 1)     + &
	    & dt * k_scav_Fe_arg  * Fe_excess
	  fscav_Fe_refrac(l) = ratio_reminf_refrac * fscav_Fe_refrac(l - 1)  + &
	    & dt * k_scav_Fe_refrac * Fe_excess
	  fremin_bionp(l,ibionp_Feorg_tot) = fremin_bionp(l,ibionp_Feorg_tot) - &
	    & (fscav_Fe_Corg(l)   - fscav_Fe_Corg(l - 1))                     - &
	    & (fscav_Fe_opal(l)   - fscav_Fe_opal(l - 1))                     - &
	    & (fscav_Fe_cal(l)    - fscav_Fe_cal(l - 1))                      - &
	    & (fscav_Fe_arg(l)    - fscav_Fe_arg(l - 1))                      - &
	    & (fscav_Fe_refrac(l) - fscav_Fe_refrac(l - 1))
	  IF (phys(c,l,ip_D_bot) >= sedpar(c,m,ised_par_D_mid)) THEN
	    dsue%idf_s(c,m,ised_fnp_Feorg) = dsue%idf_s(c,m,ised_fnp_Feorg) + &
	      & fscav_Fe_Corg(l)                                            + &
	      & fscav_Fe_opal(l)                                            + &
	      & fscav_Fe_cal(l)                                             + &
	      & fscav_Fe_arg(l)                                             + &
	      & fscav_Fe_refrac(l)
	    EXIT
	  ELSE IF (l == n_lay) THEN
	    dsue%idf_s(c,m,ised_fnp_Feorg) = dsue%idf_s(c,m,ised_fnp_Feorg) + &
	      & fscav_Fe_Corg(l)                                            + &
	      & fscav_Fe_opal(l)                                            + &
	      & fscav_Fe_cal(l)                                             + &
	      & fscav_Fe_arg(l)                                             + &
	      & fscav_Fe_refrac(l)
	    EXIT
	  END IF
	END DO
! (5) total up CaCO3 flux
	dsue%idf_s(c,m,ised_fnp_CaCO3) = dsue%idf_s(c,m,ised_fnp_cal) + dsue%idf_s(c,m,ised_fnp_arg)
! (6) add remineralized material to 'sum' tracer change
	ft_bio(c,:,it_PO4)     = ft_bio(c,:,it_PO4) + &
	  & fremin_bionp(:,ibionp_Porg_tot)
	ft_bio(c,:,it_CO2)     = ft_bio(c,:,it_CO2) + &
	  & fremin_bionp(:,ibionp_Corg_tot)         + &
	  & fremin_bionp(:,ibionp_cal_coc)          + &
	  & fremin_bionp(:,ibionp_arg_coc)
	ft_bio(c,:,it_13C_CO2) = ft_bio(c,:,it_13C_CO2) + &
	  & fremin_bionp(:,ibionp_13C_Corg_tot)         + &
	  & fremin_bionp(:,ibionp_13C_cal_coc)          + &
	  & fremin_bionp(:,ibionp_13C_arg_coc)
	ft_bio(c,:,it_alk)     = ft_bio(c,:,it_alk) + &
	  & 2.0 * fremin_bionp(:,ibionp_cal_coc)    + &
	  & 2.0 * fremin_bionp(:,ibionp_arg_coc)    - &
	  & fremin_bionp(:,ibionp_Norg_tot)
	ft_bio(c,:,it_Si)      = ft_bio(c,:,it_Si) + &
	  & fremin_bionp(:,ibionp_opal_dia)
	ft_bio(c,:,it_Fe)      = ft_bio(c,:,it_Fe) + &
	  & fremin_bionp(:,ibionp_Feorg_tot)
	ft_bio(c,:,it_O2)      = ft_bio(c,:,it_O2) - &
	  & red_Corg_O2 * fremin_bionp(:,ibionp_Corg_tot)
	ft_bio(c,:,it_18O_O2)  = ft_bio(c,:,it_18O_O2)  - &
	  & r18O_O2(c,:) * red_Corg_O2 * fremin_bionp(:,ibionp_Corg_tot)
! (7) forced remineralization of sedimenting flux
!     NOTE: If the sediment sub-system option is not selected
!           remineralize all the sedimentating particulate material in the ocean boxes adjoining 
!           each sediment layer, then re-zero sediment flux
	IF (runopt(ropt_sed) == no) THEN
	  sed_lay_frac(:) = calc_sed_lay_frac(sedpar(c,m,:),phys(c,:,:))
	  ft_bio(c,:,it_PO4)     = ft_bio(c,:,it_PO4) + sed_lay_frac(:) * &
	    & dsue%idf_s(c,m,ised_fnp_Porg)
	  ft_bio(c,:,it_CO2)     = ft_bio(c,:,it_CO2) + sed_lay_frac(:) * ( & 
	    & dsue%idf_s(c,m,ised_fnp_Corg) + &
	    & dsue%idf_s(c,m,ised_fnp_cal) + &
	    & dsue%idf_s(c,m,ised_fnp_arg))
	  ft_bio(c,:,it_13C_CO2) = ft_bio(c,:,it_13C_CO2) + sed_lay_frac(:) * ( &
	    & dsue%idf_s(c,m,ised_fnp_13C_Corg) + &
	    & dsue%idf_s(c,m,ised_fnp_13C_cal) + &
	    & dsue%idf_s(c,m,ised_fnp_13C_arg))
	  ft_bio(c,:,it_alk)     = ft_bio(c,:,it_alk) + sed_lay_frac(:) * ( &
	    & 2.0 * dsue%idf_s(c,m,ised_fnp_cal) + &
	    & 2.0 * dsue%idf_s(c,m,ised_fnp_arg) - &
	    & dsue%idf_s(c,m,ised_fnp_Norg))
	  ft_bio(c,:,it_Si)      = ft_bio(c,:,it_Si) + sed_lay_frac(:) * &
	    & dsue%idf_s(c,m,ised_fnp_opal)
	  ft_bio(c,:,it_Fe)      = ft_bio(c,:,it_Fe) + sed_lay_frac(:) * &
	    & dsue%idf_s(c,m,ised_fnp_Feorg)
	  ft_bio(c,:,it_O2)      = ft_bio(c,:,it_O2) - sed_lay_frac(:) * &
	    & red_Corg_O2 * dsue%idf_s(c,m,ised_fnp_Corg)
	  ft_bio(c,:,it_18O_O2)  = ft_bio(c,:,it_18O_O2) - sed_lay_frac(:) * &
	    & r18O_O2(c,:) * red_Corg_O2 * dsue%idf_s(c,m,ised_fnp_Corg)
	  dsue%idf_s(c,m,:) = 0.0
	ENDIF

      END DO! (end of sedimentary depth loop)

    END DO! (end of water column loop)


! *** scavenging and sedimentation - user-defined sediment cores ***

    IF ((runopt(ropt_sed) == yes) .AND. (runopt(ropt_sedcor) == yes)) THEN

      DO p = 1,n_sedcor
	
! (A) set ocean column number
	c = INT(sedcorpar(p,ised_par_colnum))
! (B) calculate value of the sinking new production flux above sediment core
	sink_bionp(:) = bionp(c,:)
! (C) calculate calcite remineralization and sedimentation
	dsue%idf_c(p,ised_fnp_cal)     = sink_bionp(ibionp_cal_coc)     * &
	  & sedcorpar(p,ised_par_reminf_cal)
	dsue%idf_c(p,ised_fnp_13C_cal) = sink_bionp(ibionp_13C_cal_coc) * &
	  & sedcorpar(p,ised_par_reminf_cal)
! (D) calculate aragonite remineralization and sedimentation
	dsue%idf_c(p,ised_fnp_arg)     = sink_bionp(ibionp_arg_coc)     * &
	  & sedcorpar(p,ised_par_reminf_arg)
	dsue%idf_c(p,ised_fnp_13C_arg) = sink_bionp(ibionp_13C_arg_coc) * &
	  & sedcorpar(p,ised_par_reminf_arg)
! (E) calculate particulate orgnaic matter remineralization and sedimentation
	dsue%idf_c(p,ised_fnp_Corg)     = sink_bionp(ibionp_Corg_tot)     * &
	  & sedcorpar(p,ised_par_reminf_Corg)
	dsue%idf_c(p,ised_fnp_Porg)     = sink_bionp(ibionp_Porg_tot)     * &
	  & sedcorpar(p,ised_par_reminf_Corg)
	dsue%idf_c(p,ised_fnp_Norg)     = sink_bionp(ibionp_Norg_tot)     * &
	  & sedcorpar(p,ised_par_reminf_Corg)
	dsue%idf_c(p,ised_fnp_Feorg)    = sink_bionp(ibionp_Feorg_tot)    * &
	  & sedcorpar(p,ised_par_reminf_Corg)
	dsue%idf_c(p,ised_fnp_13C_Corg) = sink_bionp(ibionp_13C_Corg_tot) * &
	  & sedcorpar(p,ised_par_reminf_Corg)
! (F) calculate opal remineralization and sedimentation
	dsue%idf_c(p,ised_fnp_opal)     = sink_bionp(ibionp_opal_dia) * &
	  & sedcorpar(p,ised_par_reminf_opal)

! (G) scavenge (and remineralize) dissolved Fe
!     Fe scavenging from surface ocean layer
	l = 1
	len_scav = phys(c,l,ip_D_bot) - phys(c,l,ip_D_top)
	SELECT CASE (bioopt(bopt_scav_Fe_alt))
	CASE ("1")
!     scheme #1 : no Fe scavenging
	  k_scav_Fe_Corg   = 0.0
	  k_scav_Fe_opal   = 0.0
	  k_scav_Fe_cal    = 0.0
	  k_scav_Fe_arg    = 0.0
	  k_scav_Fe_refrac = 0.0
	CASE ("2")
!     scheme #2 : fixed scavenging rate Fe scavenging
	  k_scav_Fe_Corg   = (par_k_scav_Fe_fPMlaw_Corg / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	  k_scav_Fe_opal   = (par_k_scav_Fe_fPMlaw_opal / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	  k_scav_Fe_cal    = (par_k_scav_Fe_fPMlaw_cal  / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	  k_scav_Fe_arg    = (par_k_scav_Fe_fPMlaw_arg  / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	  k_scav_Fe_refrac = 0.0
	CASE ("3")
!     scheme #3 : particulate matter flux dependent scavenging rate Fe scavenging
	  k_scav_Fe_Corg   = par_k_scav_Fe_fPMlaw_Corg *                       &
	    & phys(c,l,ip_reminf_Corg) * (sink_bionp(ibionp_Corg_tot) + 1.0) / &
	    & sedcorpar(p,ised_par_A)
	  k_scav_Fe_opal   = par_k_scav_Fe_fPMlaw_opal *                       &
	    & phys(c,l,ip_reminf_opal) * (sink_bionp(ibionp_opal_dia) + 1.0) / &
	    & sedcorpar(p,ised_par_A)
	  k_scav_Fe_cal    = par_k_scav_Fe_fPMlaw_cal  *                       &
	    & phys(c,l,ip_reminf_cal)  * (sink_bionp(ibionp_cal_coc) + 1.0)  / &
	    & sedcorpar(p,ised_par_A)
	  k_scav_Fe_arg    = par_k_scav_Fe_fPMlaw_arg  *                       &
	    & phys(c,l,ip_reminf_arg)  * (sink_bionp(ibionp_arg_coc) + 1.0)  / &
	    & sedcorpar(p,ised_par_A)
	  k_scav_Fe_refrac = par_k_scav_Fe_fPMlaw_refrac                     * &
	    & fdust(c)
	END SELECT
	IF (oatracer%ioat_t(c,l,it_Fe) > par_apparsol_Fe) THEN
	  Fe_excess = (oatracer%ioat_t(c,l,it_Fe) - par_apparsol_Fe) * &
	    & conv_m3_kg * len_scav * phys(c,1,ip_A_bot)
	ELSE
	  Fe_excess = 0.0
	ENDIF
	fscav_Fe_Corg(l)   = 0.0 + &
	  & dt * k_scav_Fe_Corg   * Fe_excess
	fscav_Fe_opal(l)   = 0.0 + &
	  & dt * k_scav_Fe_opal   * Fe_excess
	fscav_Fe_cal(l)    = 0.0 + &
	  & dt * k_scav_Fe_cal    * Fe_excess
	fscav_Fe_arg(l)    = 0.0 + &
	  & dt * k_scav_Fe_arg    * Fe_excess
	fscav_Fe_refrac(l) = 0.0 + &
	  & dt * k_scav_Fe_refrac * Fe_excess
!     Fe scavenging from remaining ocean layers
	DO l = 2,n_lay
	  IF (phys(c,l,ip_D_bot) >= sedcorpar(p,ised_par_D_mid)) THEN
	    IF (l == 2) THEN
	      dsue%idf_c(p,ised_fnp_Feorg) = dsue%idf_c(p,ised_fnp_Feorg) + &
		& fscav_Fe_Corg(l - 1) +                                    &
		& fscav_Fe_opal(l - 1) +                                    &
		& fscav_Fe_cal(l - 1) +                                     &
		& fscav_Fe_arg(l - 1) +                                     &
		& fscav_Fe_refrac(l - 1)
	      EXIT
	    END IF
	    IF (phys(c,1,ip_remin_topD) > phys(c,l,ip_D_top)) THEN
	      len_scav = sedcorpar(p,ised_par_D_mid) - phys(c,1,ip_remin_topD)
	    ELSE
	      len_scav = sedcorpar(p,ised_par_D_mid) - phys(c,l,ip_D_top)
	    END IF
	    IF (len_scav < 0.0) len_scav = 0.0
	    ratio_reminf_Corg = sedcorpar(p,ised_par_reminf_Corg) / (phys(c,l - 1,ip_reminf_Corg) + 0.000001)
	    IF (ratio_reminf_Corg > 1.0) ratio_reminf_Corg = 1.0
	    ratio_reminf_opal = sedcorpar(p,ised_par_reminf_opal) / (phys(c,l - 1,ip_reminf_opal) + 0.000001)
	    IF (ratio_reminf_opal > 1.0) ratio_reminf_opal = 1.0
	    ratio_reminf_cal  = sedcorpar(p,ised_par_reminf_cal)  / (phys(c,l - 1,ip_reminf_cal)  + 0.000001)
	    IF (ratio_reminf_cal  > 1.0) ratio_reminf_cal  = 1.0
	    ratio_reminf_arg  = sedcorpar(p,ised_par_reminf_arg)  / (phys(c,l - 1,ip_reminf_arg)  + 0.000001)
	    IF (ratio_reminf_arg  > 1.0) ratio_reminf_arg  = 1.0
	    ratio_reminf_refrac = 1.0
	    SELECT CASE (bioopt(bopt_scav_Fe_alt))
	    CASE ("1")
!     scheme #1 : no Fe scavenging
	      k_scav_Fe_Corg   = 0.0
	      k_scav_Fe_opal   = 0.0
	      k_scav_Fe_cal    = 0.0
	      k_scav_Fe_arg    = 0.0
	      k_scav_Fe_refrac = 0.0
	    CASE ("2")
!     scheme #2 : fixed scavenging rate Fe scavenging
	      k_scav_Fe_Corg   = (par_k_scav_Fe_fPMlaw_Corg / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_opal   = (par_k_scav_Fe_fPMlaw_opal / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_cal    = (par_k_scav_Fe_fPMlaw_cal  / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_arg    = (par_k_scav_Fe_fPMlaw_arg  / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_refrac = 0.0
	    CASE ("3")
!     scheme #3 : particulate matter flux dependent scavenging rate Fe scavenging
	      k_scav_Fe_Corg   = par_k_scav_Fe_fPMlaw_Corg *                                &
		& sedcorpar(p,ised_par_reminf_Corg) * (sink_bionp(ibionp_Corg_tot) + 1.0) / &
		& phys(c,1,ip_A_bot)
	      k_scav_Fe_opal   = par_k_scav_Fe_fPMlaw_opal *                                &
		& sedcorpar(p,ised_par_reminf_opal) * (sink_bionp(ibionp_opal_dia) + 1.0) / &
		& phys(c,1,ip_A_bot)
	      k_scav_Fe_cal    = par_k_scav_Fe_fPMlaw_cal *                                 &
		& sedcorpar(p,ised_par_reminf_cal)  * (sink_bionp(ibionp_cal_coc) + 1.0)  / &
		& phys(c,1,ip_A_bot)
	      k_scav_Fe_arg    = par_k_scav_Fe_fPMlaw_arg *                                 &
		& sedcorpar(p,ised_par_reminf_arg)  * (sink_bionp(ibionp_arg_coc) + 1.0)  / &
		& phys(c,1,ip_A_bot)
	      k_scav_Fe_refrac = 0.0
	    END SELECT
	  ELSE
	    IF (phys(c,1,ip_remin_topD) > phys(c,l,ip_D_top)) THEN
	      len_scav = phys(c,l,ip_D_bot) - phys(c,1,ip_remin_topD)
	    ELSE
	      len_scav = phys(c,l,ip_D_bot) - phys(c,l,ip_D_top)
	    END IF
	    IF (len_scav < 0.0) len_scav = 0.0
	    ratio_reminf_Corg = phys(c,l,ip_reminf_Corg) / (phys(c,l - 1,ip_reminf_Corg) + 0.000001)
	    IF (ratio_reminf_Corg > 1.0) ratio_reminf_Corg = 1.0
	    ratio_reminf_opal = phys(c,l,ip_reminf_opal) / (phys(c,l - 1,ip_reminf_opal) + 0.000001)
	    IF (ratio_reminf_opal > 1.0) ratio_reminf_opal = 1.0
	    ratio_reminf_cal  = phys(c,l,ip_reminf_cal)  / (phys(c,l - 1,ip_reminf_cal)  + 0.000001)
	    IF (ratio_reminf_cal  > 1.0) ratio_reminf_cal  = 1.0
	    ratio_reminf_arg  = phys(c,l,ip_reminf_arg)  / (phys(c,l - 1,ip_reminf_arg)  + 0.000001)
	    IF (ratio_reminf_arg  > 1.0) ratio_reminf_arg  = 1.0
	    ratio_reminf_refrac = 1.0
	    SELECT CASE (bioopt(bopt_scav_Fe_alt))
	    CASE ("1")
!     scheme #1 : no Fe scavenging
	      k_scav_Fe_Corg   = 0.0
	      k_scav_Fe_opal   = 0.0
	      k_scav_Fe_cal    = 0.0
	      k_scav_Fe_arg    = 0.0
	      k_scav_Fe_refrac = 0.0
	    CASE ("2")
!     scheme #2 : fixed scavenging rate Fe scavenging
	      k_scav_Fe_Corg   = (par_k_scav_Fe_fPMlaw_Corg / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_opal   = (par_k_scav_Fe_fPMlaw_opal / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_cal    = (par_k_scav_Fe_fPMlaw_cal  / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_arg    = (par_k_scav_Fe_fPMlaw_arg  / par_k_scav_Fe_fPMlaw_tot) * par_k_scav_Fe
	      k_scav_Fe_refrac = 0.0
	    CASE ("3")
!     scheme #3 : particulate matter flux dependent scavenging rate Fe scavenging
	      k_scav_Fe_Corg   = par_k_scav_Fe_fPMlaw_Corg *                       &
		& phys(c,l,ip_reminf_Corg) * (sink_bionp(ibionp_Corg_tot) + 1.0) / &
		& phys(c,1,ip_A_bot)
	      k_scav_Fe_opal   = par_k_scav_Fe_fPMlaw_opal *                       &
		& phys(c,l,ip_reminf_opal) * (sink_bionp(ibionp_opal_dia) + 1.0) / &
		& phys(c,1,ip_A_bot)
	      k_scav_Fe_cal    = par_k_scav_Fe_fPMlaw_cal *                        &
		& phys(c,l,ip_reminf_cal)  * (sink_bionp(ibionp_cal_coc) + 1.0)  / &
		& phys(c,1,ip_A_bot)
	      k_scav_Fe_arg    = par_k_scav_Fe_fPMlaw_arg *                        &
		& phys(c,l,ip_reminf_arg)  * (sink_bionp(ibionp_arg_coc) + 1.0)  / &
		& phys(c,1,ip_A_bot)
	      k_scav_Fe_refrac = 0.0
	    END SELECT
	  ENDIF
	  IF (oatracer%ioat_t(c,l,it_Fe) > par_apparsol_Fe) THEN
	    Fe_excess = (oatracer%ioat_t(c,l,it_Fe) - par_apparsol_Fe) * &
	      & conv_m3_kg * len_scav * phys(c,1,ip_A_bot)
	  ELSE
	    Fe_excess = 0.0
	  ENDIF
	  fscav_Fe_Corg(l)   = ratio_reminf_Corg * fscav_Fe_Corg(l - 1) + &
	    & dt * k_scav_Fe_Corg * Fe_excess
	  fscav_Fe_opal(l)   = ratio_reminf_opal * fscav_Fe_opal(l - 1) + &
	    & dt * k_scav_Fe_opal * Fe_excess
	  fscav_Fe_cal(l)    = ratio_reminf_cal * fscav_Fe_cal(l - 1) + &
	    & dt * k_scav_Fe_cal  * Fe_excess
	  fscav_Fe_arg(l)    = ratio_reminf_arg * fscav_Fe_arg(l - 1) + &
	    & dt * k_scav_Fe_arg  * Fe_excess
	  fscav_Fe_refrac(l) = ratio_reminf_refrac * fscav_Fe_refrac(l - 1) + &
	    & dt * k_scav_Fe_refrac * Fe_excess
	  IF (phys(c,l,ip_D_bot) >= sedcorpar(p,ised_par_D_mid)) THEN
	    dsue%idf_c(p,ised_fnp_Feorg) = dsue%idf_c(p,ised_fnp_Feorg) + &
	      & fscav_Fe_Corg(l)                                        + &
	      & fscav_Fe_opal(l)                                        + &
	      & fscav_Fe_cal(l)                                         + &
	      & fscav_Fe_arg(l)                                         + &
	      & fscav_Fe_refrac(l)
	    EXIT
	  ELSE IF (l == n_lay) THEN
	    dsue%idf_c(p,ised_fnp_Feorg) = dsue%idf_c(p,ised_fnp_Feorg) + &
	      & fscav_Fe_Corg(l)                                        + &
	      & fscav_Fe_opal(l)                                        + &
	      & fscav_Fe_cal(l)                                         + &
	      & fscav_Fe_arg(l)                                         + &
	      & fscav_Fe_refrac(l)
	    EXIT
	  END IF
	END DO
! (H) total up CaCO3 flux
	dsue%idf_c(p,ised_fnp_CaCO3) = dsue%idf_c(p,ised_fnp_cal) + dsue%idf_c(p,ised_fnp_arg)

      END DO! (end of sediment core loop)

    ENDIF


! *** decay DOC ***

! remove remineralized DOM from ocean
    ft_bio(:,:,it_DOC)     = ft_bio(:,:,it_DOC) - &
      & (dt / par_bionp_DOM_tconst) * phys(:,:,ip_M) * oatracer%ioat_t(:,:,it_DOC)
    ft_bio(:,:,it_13C_DOC) = ft_bio(:,:,it_13C_DOC) - &
      & (dt / par_bionp_DOM_tconst) * phys(:,:,ip_M) * oatracer%ioat_t(:,:,it_13C_DOC)
    ft_bio(:,:,it_DOFe)    = ft_bio(:,:,it_DOFe) - &
      & (dt / par_bionp_DOM_tconst) * phys(:,:,ip_M) * oatracer%ioat_t(:,:,it_DOFe)
! add released tracers to ocean
    ft_bio(:,:,it_CO2)     = ft_bio(:,:,it_CO2) + &
      & (dt / par_bionp_DOM_tconst) * phys(:,:,ip_M) * oatracer%ioat_t(:,:,it_DOC)
    ft_bio(:,:,it_13C_CO2) = ft_bio(:,:,it_13C_CO2) + &
      & (dt / par_bionp_DOM_tconst) * phys(:,:,ip_M) * oatracer%ioat_t(:,:,it_13C_DOC)
    ft_bio(:,:,it_alk)     = ft_bio(:,:,it_alk) - &
      & red_Corg_Norg * (dt / par_bionp_DOM_tconst) * phys(:,:,ip_M) * oatracer%ioat_t(:,:,it_DOC)
    ft_bio(:,:,it_PO4)     = ft_bio(:,:,it_PO4) + &
      & red_Corg_Porg * (dt / par_bionp_DOM_tconst) * phys(:,:,ip_M) * oatracer%ioat_t(:,:,it_DOC)
    ft_bio(:,:,it_Fe)      = ft_bio(:,:,it_Fe) + &
      & (dt / par_bionp_DOM_tconst) * phys(:,:,ip_M) * oatracer%ioat_t(:,:,it_DOFe)
! consume O2
    ft_bio(:,:,it_O2)      = ft_bio(:,:,it_O2) - &
      & red_Corg_O2 * (dt / par_bionp_DOM_tconst) * phys(:,:,ip_M) * oatracer%ioat_t(:,:,it_DOC)


! *** relax toward prescribed surface ocean conditions ***

! (1) relax toward prescribed surface ocean T and S conditions
    SST_old(:) = oatracer%ioat_t(:,1,it_T) + (ft_abio(:,1,it_T) / phys(:,1,ip_M))
    SSS_old(:) = oatracer%ioat_t(:,1,it_S) + (ft_abio(:,1,it_S) / phys(:,1,ip_M))
    SST_new(:) = SST_old(:) + &
      & (SST(:) - SST_old(:)) * (dt / par_misc_SS_tconst)
    SSS_new(:) = SSS_old(:) + &
      & (SSS(:) - SSS_old(:)) * (dt / par_misc_SS_tconst)
    ft_abio(:,1,it_T) = ft_abio(:,1,it_T) + (SST_new(:) - SST_old(:)) * phys(:,1,ip_M)
    ft_abio(:,1,it_S) = ft_abio(:,1,it_S) + (SSS_new(:) - SSS_old(:)) * phys(:,1,ip_M)
! (2) relax toward saturated surface ocean dissolved oxygen concentrations
    SSO2_eqm(:) = SSdis(:,idis_QO2) * oatracer%ioat_a(ia_pO2)
    SSO2_old(:) = oatracer%ioat_t(:,1,it_O2) + ((ft_abio(:,1,it_O2) + ft_bio(:,1,it_O2)) / phys(:,1,ip_M))
    SSO2_new(:) = SSO2_old(:) + &
      & (SSO2_eqm(:) - SSO2_old(:)) * (1.0 - seas_t_int_seaice(tint,:)) * (dt / par_misc_SSO2_tconst)
    ft_abio(:,1,it_O2) = ft_abio(:,1,it_O2) + (SSO2_new(:) - SSO2_old(:)) * phys(:,1,ip_M)
    fa_abio(ia_pO2)    = fa_abio(ia_pO2) - SUM((SSO2_new(:) - SSO2_old(:)) * phys(:,1,ip_M))


! *** return function value ***

! sum the seperate biological and abiological flux components
! NOTE: correct units from (mol) to (mol kg-1) for ocean tracers, 
!       and from (mol) to (atm) for atmospheric tracers
! NOTE: restore SSTs and SSSs towards prescribed sea surface conditions according to 
!       a characteristic time constant (set in the parameter file)
!      if no convective adjustments have been made (as the restoring is done as part of this)
! (1) ocean tracers
    DO it = 1,n_tracer
      dsue%idf_t(:,:,it) = (1.0 / phys(:,:,ip_M)) * &
	& (ft_abio(:,:,it) + ft_bio(:,:,it) + dt * goatracer%ioat_t(:,:,it))
    END DO
! (2) atmospheric tracers
    DO ia = 1,n_atm
      dsue%idf_a(ia) = conv_mol_atm * &
	& (fa_abio(ia) + fa_bio(ia) + dt * goatracer%ioat_a(ia))
    END DO
! (3) export to integral sediment sub-system sediments
!     NOTE: the returned value has already been set earlier
! (4) export to user-defined sediment cores
!     NOTE: the returned value has already been set earlier
! (5) new production
    dsue%idf_b = bionp(:,:)
! (6) update value of 'tracer' audit total
!     NOTE: add atmospheric tracer masses to ocean tracer totals for simplicity
    dsue%idf_g(it_T)       = dt * SUM(goatracer%ioat_t(:,:,it_T))
    dsue%idf_g(it_S)       = dt * SUM(goatracer%ioat_t(:,:,it_S))
    dsue%idf_g(it_PO4)     = dt * SUM(goatracer%ioat_t(:,:,it_PO4))
    dsue%idf_g(it_alk)     = dt * SUM(goatracer%ioat_t(:,:,it_alk))
    dsue%idf_g(it_Si)      = dt * SUM(goatracer%ioat_t(:,:,it_Si))
    dsue%idf_g(it_Fe)      = dt * SUM(goatracer%ioat_t(:,:,it_Fe))
    dsue%idf_g(it_CO2)     = dt * SUM(goatracer%ioat_t(:,:,it_CO2))     + dt * goatracer%ioat_a(ia_pCO2)
    dsue%idf_g(it_13C_CO2) = dt * SUM(goatracer%ioat_t(:,:,it_13C_CO2)) + dt * goatracer%ioat_a(ia_13C_pCO2)
    dsue%idf_g(it_O2)      = dt * SUM(goatracer%ioat_t(:,:,it_O2))      + dt * goatracer%ioat_a(ia_pO2)
    dsue%idf_g(it_18O_O2)  = dt * SUM(goatracer%ioat_t(:,:,it_18O_O2))  + dt * goatracer%ioat_a(ia_18O_pO2)
! (7) store (integration) time step tracer data
    seas_t_int_oatracer(tint)%ioat_t(:,:,:) = oatracer%ioat_t(:,:,:)
    seas_t_int_oatracer(tint)%ioat_a        = oatracer%ioat_a
! (8) store (integration) time step new production data
    seas_t_int_bionp(tint,:,:) = bionp(:,:)


  END FUNCTION dsue
! *************************************************************************************************


! *************************************************************************************************
! *** Update ocean and atmosphere *****************************************************************
! *************************************************************************************************
  SUBROUTINE update_ocnatm(dt,oatracer,goatracer, &
    & phys,oconvadjfixed,oceancir,ssdis,sedpar,sedcorpar, &
    & bionpfixed,fdust, &
    & SST,SSS,SSPO4, &
    & oatracerannual,sedfnp,sedcorfnp,bionp, &
    & oconvadj,dtot)
    IMPLICIT NONE
! dummy variables
    REAL(KIND=fp_1),INTENT(in)::dt
    TYPE(ocean_atm_tracer),INTENT(inout)::oatracer
    TYPE(ocean_atm_tracer),INTENT(in)::goatracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_t_int_max)::oconvadjfixed
    TYPE(ocean_circulation),INTENT(in),DIMENSION(1:par_n_wflux)::oceancir
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_dis)::ssdis
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sed_depth,n_sed_par)::sedpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_sed_par)::sedcorpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_bionp)::bionpfixed
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::fdust
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::SST
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::SSS
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::SSPO4
    TYPE(ocean_atm_tracer),INTENT(out)::oatracerannual
    REAL(KIND=fp_1),INTENT(out),DIMENSION(n_col,n_sed_depth,n_sed_fnp)::sedfnp
    REAL(KIND=fp_1),INTENT(out),DIMENSION(n_sedcor,n_sed_fnp)::sedcorfnp
    REAL(KIND=fp_1),INTENT(out),DIMENSION(n_col,n_bionp)::bionp
    REAL(KIND=fp_1),INTENT(out),DIMENSION(n_col,n_lay,n_t_int_max)::oconvadj
    REAL(KIND=fp_1),INTENT(out),DIMENSION(n_tracer)::dtot
! local variables
    INTEGER::t_int
    TYPE(derivative_function)::dy
    TYPE(derivative_function)::dy_k1,dy_k2,dy_k3,dy_k4
    TYPE(ocean_atm_tracer)::oat_div2_dy_k1,oat_div2_dy_k2,oat_dy_k3,oat_dy_k4

! initialize return (only) variables
    oatracerannual%ioat_t = 0.0
    oatracerannual%ioat_a = 0.0
    sedfnp(:,:,:)         = 0.0
    sedcorfnp(:,:)        = 0.0
    bionp(:,:)            = 0.0
    oconvadj(:,:,:)       = 0.0
    dtot(:)               = 0.0
! integrate forward 1 yr
    SELECT CASE (par_int_scheme)
    CASE (1)
! 4th order Runge-Kutta integration routine
      DO t_int = 1,par_n_t_int
! calculate coefficient 'k1'
	dy_k1 = dsue(dt,t_int,oatracer,goatracer,oceancir,ssdis,phys,oconvadjfixed,sedpar,sedcorpar, &
	  & SST,SSS,SSPO4,bionpfixed,fdust)
	dy_k1%idf_t = dy_k1%idf_t
	dy_k1%idf_a = dy_k1%idf_a
	dy_k1%idf_s = dy_k1%idf_s
	dy_k1%idf_c = dy_k1%idf_c
	dy_k1%idf_b = dy_k1%idf_b
	dy_k1%idf_v = dy_k1%idf_v
	dy_k1%idf_g = dy_k1%idf_g
! calculate coefficient 'k2'
	oat_div2_dy_k1%ioat_t = 0.5 * dy_k1%idf_t + oatracer%ioat_t
	oat_div2_dy_k1%ioat_a = 0.5 * dy_k1%idf_a + oatracer%ioat_a
	dy_k2 = dsue(dt,t_int,oat_div2_dy_k1,goatracer,oceancir,ssdis,phys,oconvadjfixed,sedpar,sedcorpar, &
	  & SST,SSS,SSPO4,bionpfixed,fdust)
	dy_k2%idf_t = dy_k2%idf_t
	dy_k2%idf_a = dy_k2%idf_a
	dy_k2%idf_s = dy_k2%idf_s
	dy_k2%idf_c = dy_k2%idf_c
	dy_k2%idf_b = dy_k2%idf_b
	dy_k2%idf_v = dy_k2%idf_v
	dy_k2%idf_g = dy_k2%idf_g
! calculate coefficient 'k3'
	oat_div2_dy_k2%ioat_t = 0.5 * dy_k2%idf_t + oatracer%ioat_t
	oat_div2_dy_k2%ioat_a = 0.5 * dy_k2%idf_a + oatracer%ioat_a
	dy_k3 = dsue(dt,t_int,oat_div2_dy_k2,goatracer,oceancir,ssdis,phys,oconvadjfixed,sedpar,sedcorpar, &
	  & SST,SSS,SSPO4,bionpfixed,fdust)
	dy_k3%idf_t = dy_k3%idf_t
	dy_k3%idf_a = dy_k3%idf_a
	dy_k3%idf_s = dy_k3%idf_s
	dy_k3%idf_c = dy_k3%idf_c
	dy_k3%idf_b = dy_k3%idf_b
	dy_k3%idf_v = dy_k3%idf_v
	dy_k3%idf_g = dy_k3%idf_g
! calculate coefficient 'k4'
	oat_dy_k3%ioat_t = dy_k3%idf_t + oatracer%ioat_t
	oat_dy_k3%ioat_a = dy_k3%idf_a + oatracer%ioat_a
	dy_k4 = dsue(dt,t_int,oat_dy_k3,goatracer,oceancir,ssdis,phys,oconvadjfixed,sedpar,sedcorpar, &
	  & SST,SSS,SSPO4,bionpfixed,fdust)
	dy_k4%idf_t = dy_k4%idf_t
	dy_k4%idf_a = dy_k4%idf_a
	dy_k4%idf_s = dy_k4%idf_s
	dy_k4%idf_c = dy_k4%idf_c
	dy_k4%idf_b = dy_k4%idf_b
	dy_k4%idf_v = dy_k4%idf_v
	dy_k4%idf_g = dy_k4%idf_g
! update the system at forward time t + dt
	oatracer%ioat_t = oatracer%ioat_t + (dy_k1%idf_t+2.0*dy_k2%idf_t+2.0*dy_k3%idf_t+dy_k4%idf_t) / 6.0
	oatracer%ioat_a = oatracer%ioat_a + (dy_k1%idf_a+2.0*dy_k2%idf_a+2.0*dy_k3%idf_a+dy_k4%idf_a) / 6.0
	sedfnp          = sedfnp          + (dy_k1%idf_s+2.0*dy_k2%idf_s+2.0*dy_k3%idf_s+dy_k4%idf_s) / 6.0
	sedcorfnp       = sedcorfnp       + (dy_k1%idf_c+2.0*dy_k2%idf_c+2.0*dy_k3%idf_c+dy_k4%idf_c) / 6.0
	bionp           = bionp           + (dy_k1%idf_b+2.0*dy_k2%idf_b+2.0*dy_k3%idf_b+dy_k4%idf_b) / 6.0
	oconvadj        = oconvadj        + (dy_k1%idf_v+2.0*dy_k2%idf_v+2.0*dy_k3%idf_v+dy_k4%idf_v) / 6.0
	dtot            = dtot            + (dy_k1%idf_g+2.0*dy_k2%idf_g+2.0*dy_k3%idf_g+dy_k4%idf_g) / 6.0
! calculate annual average values
	oatracerannual%ioat_t = oatracerannual%ioat_t + dt * oatracer%ioat_t
	oatracerannual%ioat_a = oatracerannual%ioat_a + dt * oatracer%ioat_a
      END DO
    CASE (2)
! Euler integration routine
      DO t_int = 1,par_n_t_int
! calculate derivative coefficient
	dy = dsue(dt,t_int,oatracer,goatracer,oceancir,ssdis,phys,oconvadjfixed,sedpar,sedcorpar, &
	  & SST,SSS,SSPO4,bionpfixed,fdust)
! update the system at forward time t + dt
	oatracer%ioat_t = oatracer%ioat_t + dy%idf_t
	oatracer%ioat_a = oatracer%ioat_a + dy%idf_a
	sedfnp          = sedfnp          + dy%idf_s
	sedcorfnp       = sedcorfnp       + dy%idf_c
	bionp           = bionp           + dy%idf_b
	oconvadj        = oconvadj        + dy%idf_v
	dtot            = dtot            + dy%idf_g
! calculate annual average values
! NOTE: returned variables other than 'tracer' already represent annual (total) values 
	oatracerannual%ioat_t = oatracerannual%ioat_t + dt * oatracer%ioat_t
	oatracerannual%ioat_a = oatracerannual%ioat_a + dt * oatracer%ioat_a
      END DO
    CASE default
      PRINT*,""
      PRINT*,"FATAL ERROR [sue_box.f90, update_ocnatm]:"
      PRINT*,"option 'par_int_scheme' is out of range;"
      PRINT*,"selected number = ",par_int_scheme
      PRINT*,""
      STOP
    END SELECT

  END SUBROUTINE update_ocnatm
! *************************************************************************************************


END MODULE sue_box

























